#ifndef TESTER_H
#define TESTER_H

/**
  This is the main header file for all Educational Kit applications.
  Includes all necessary QT library header files.
  Includes basic data type defines.
*/

#define BUILD_FOR_UNIX

//////// QT library Header Files ////////

#include <qapplication.h>
#include <qapp.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qpainter.h>

#include <qmessagebox.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qradiobutton.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qaccel.h>
#include <signal.h>
#include <qprocess.h>
#include <qdir.h>
#include <qtimer.h>
#include <qdatetime.h>
#include <qlistview.h>
#include <qtextedit.h>
#include <qwaitcondition.h>
#include <qfile.h> 

//////// Protocol APIs Header Files ////////
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>

#if defined(BUILD_FOR_UNIX)
#include <unistd.h>
#include <signal.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <wanpipe.h>	//sdla_exec_t
#include <sdla_edu.h>	//edu_exec_cmd_t

#endif

#include <time.h>


//////////////////////////////////////////

#define MAX_LINES	50

// Macros for QListBox
#define ADDSTR(list_box, x)	{																		\
					list_box->insertItem( QString::fromLatin1( x ), -1);	\
					list_box->setCurrentItem(list_box->count() - 1);	    \
					list_box->ensureCurrentVisible();                     \
					if(list_box->count() > MAX_LINES)                     \
							list_box->removeItem(0);                          \
}

#define ADDSTRING(list_box, s, str)	{ strcat(s, str); ADDSTR(list_box, s); }


// Macros for QListView
#define LIST_VIEW_ADDSTR(list_view, list_view_item)	{	                \
		list_view->insertItem(list_view_item);	                          \
    list_view->setSelected(list_view->lastItem(), TRUE);              \
    list_view->ensureItemVisible(list_view->lastItem());              \
		if(list_view->childCount() > MAX_LINES){                          \
		  list_view->takeItem(list_view->firstChild());                   \
    }                                                                 \
}

//
// Various definitions for UniTester application
//
#define PROTOCOL_OPTION_NO_PROTOCOL		0x0F
#define PROTOCOL_OPTION_X25				    0x00
#define	PROTOCOL_OPTION_FR				    0x01
#define	PROTOCOL_OPTION_BSYNC			    0x02

//
// Card 0 will be DCE, card 1 DTE
//
extern int card_number;//global, available for all code in the application
extern int protocol;

#define CARD_DCE    0
#define CARD_DTE    1
#define INVAL_CARD  -1

#define DCE_DEFAULT_POS_X 25
#define DCE_DEFAULT_POS_Y 25

#define DTE_DEFAULT_POS_X 150
#define DTE_DEFAULT_POS_Y 330

// timers ...
#define MAX_DLCIS				100
#define TX_DELAY  50
#define RX_DELAY  25

#define MAX_WAIT  1000

/////////////////////////////////////////////////////////////////////////////
#define CODE_BASE_ADDR		0x100	// the Z80 code base address
#define SIZEOF_VER_NO 		0x04	// the number of bytes in the version number, eg 1.01
/////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)
	// Defines for data types
	#define FALSE	0
	#define TRUE	1

	#define BOOL	  unsigned char
	#define BYTE 	  unsigned char
	#define WORD 	  unsigned short
	#define DWORD   unsigned int
	#define ULONG	  unsigned int
	#define UCHAR	  unsigned char
	#define USHORT  unsigned short
	#define UINT    unsigned short

	//dismiss codes of modal dilalogs
	#define IDOK 0      //OK button
	#define IDCANCEL 1  //Cancel button
#endif

#endif // TESTER_H
