// Fr_LineTrace_Impl.cpp: implementation of the Fr_LineTrace_Impl class.
//
//////////////////////////////////////////////////////////////////////

#include "MainWizardImpl.h"

extern TesterWizardImpl * globalTesterWizPtr;

#define DBG_FR_TRACE 0

char no_trace_file_message[] = 
	"No trace file name is set. Please close the Line Trace window and set the name of the trace file.";
//trace columns
//Common columns
#define DIR_COL     0
#define LEN_COL     1
#define TIME_COL    2
#define DATA_COL    3

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Fr_LineTrace_Impl::Fr_LineTrace_Impl(QWidget* parent, const char* name,
								   WFlags fl):Fr_LineTrace(parent, name, fl)

{
	set_widget_caption(this);
/*
#if defined(BUILD_FOR_UNIX)
	strcpy(globalTesterWizPtr->trace_file_name, "./trace.txt");
#elif defined(WIN32)
	strcpy(globalTesterWizPtr->trace_file_name, ".\\trace.txt");
#endif
*/
	//remove the default column
	ListView_FrLineTrace->removeColumn(0);

	ListView_FrLineTrace->setSorting(-1);
	ListView_FrLineTrace->addColumn("Frame Direction"); //0
	ListView_FrLineTrace->setColumnAlignment(DIR_COL, Qt::AlignHCenter);
	ListView_FrLineTrace->addColumn("Length");			//1
	ListView_FrLineTrace->setColumnAlignment(LEN_COL, Qt::AlignHCenter);
	ListView_FrLineTrace->addColumn("Time Stamp");		//2
	ListView_FrLineTrace->setColumnAlignment(TIME_COL, Qt::AlignHCenter);
	ListView_FrLineTrace->addColumn("Data                                                                                                                                                                 ");    //3
	ListView_FrLineTrace->clear();

	tracing_enabled = FALSE;
	raw_data = 0;
	trace_ind = 0;

}

Fr_LineTrace_Impl::~Fr_LineTrace_Impl()
{

}

void Fr_LineTrace_Impl::showEvent(QShowEvent * ev)
{
	set_widget_position_on_screen(this);
}

void Fr_LineTrace_Impl::closeEvent(QCloseEvent * e)
{
	globalTesterWizPtr->fr_menu_line_trace->
		setItemChecked(globalTesterWizPtr->fr_menu_line_trace->idAt(0), FALSE );

	//inable so file name can be changed
	globalTesterWizPtr->fr_menu_line_trace->
		setItemEnabled (globalTesterWizPtr->fr_menu_line_trace->idAt(1), TRUE);

	hide();

	if(globalTesterWizPtr->trace_file != NULL)
	{	fclose( globalTesterWizPtr->trace_file );
		globalTesterWizPtr->trace_file = NULL;
	}
}

void Fr_LineTrace_Impl::timerEvent(QTimerEvent * e)
{
	curr_pos = 0;

	globalTesterWizPtr->ResetCB(&globalTesterWizPtr->fcb);
	globalTesterWizPtr->ResetCB(&cb);

	globalTesterWizPtr->fcb.command = FR_GET_TRACE_INFO;
	globalTesterWizPtr->fcb.buffer_length = 0;
	globalTesterWizPtr->fcb.dlci = 0;
	globalTesterWizPtr->Fr_DoCommand();

	memcpy(&cb, &globalTesterWizPtr->fcb, sizeof(cb));

	if( globalTesterWizPtr->fcb.return_code == 0 &&
      globalTesterWizPtr->fcb.buffer_length != 0 &&
		  CheckBox_stop_autoscroll->isChecked() == FALSE)
	{

		num_frames = (cb.data[0] >> 1) & MAX_FRMS_TRACED;

		DbgPrint2(DBG_FR_TRACE, "num_frames = %d\n", num_frames);

		for ( i=0; i < num_frames; i++)
		{	

			list_view_item =
				new QListViewItem(ListView_FrLineTrace, ListView_FrLineTrace->lastItem());
			
			s1[0] = '\0';

			frame_info = (PFRAME_DATA)(cb.data + curr_pos);
			// frame type
			if (frame_info->status & 0x01) 
			{	sprintf((char *)outstr, "OUTGOING\t\0");
				if (trace_ind == 2)
					fprintf(globalTesterWizPtr->trace_file, "\n\nFrameDirection : OUTGOING  ");
			} else 
			{	sprintf((char *)outstr, "INCOMING\t\0");
				if (trace_ind == 2)
					fprintf( globalTesterWizPtr->trace_file, "\n\nFrameDirection : INCOMING  ");
			}

			txt.sprintf("%s", outstr);
			list_view_item->setText(DIR_COL, QString::fromLatin1((const char *)txt));

			// real length and time stamp
			sprintf((char *)(outstr + strlen((const char *)outstr)),
				"%d\t%d\t\0", frame_info->real_length, frame_info->time_stamp);

			txt.sprintf("%d", frame_info->real_length);
			list_view_item->setText(LEN_COL, QString::fromLatin1((const char *)txt));

			txt.sprintf("%d", frame_info->time_stamp);
			list_view_item->setText(TIME_COL, QString::fromLatin1((const char *)txt));

			if (trace_ind == 2 )
			{	fprintf( globalTesterWizPtr->trace_file, "FrameLength(dec) : %d   TimeStamp (dec) : %d\n", frame_info->real_length, frame_info->time_stamp);
				fprintf( globalTesterWizPtr->trace_file, ">>>> FrameData BEGINS :\n");
			}

			// first update curr_pos
			curr_pos += sizeof(FRAME_DATA);
			// the actual data
			if (frame_info->passed_length == 0)
			{	sprintf((char *)(outstr+strlen((const char *)outstr)),
					"the frame data is not available" );

				txt = "the frame data is not available";
				if (trace_ind == 2) {
					fprintf( globalTesterWizPtr->trace_file, "Frame Data is not available due to lack of space on the board - frame is too big\n");
				}
			}
			else
			{	// update curr_pos again
				curr_pos += (frame_info->real_length-1); 
				num_chars = 
					(UCHAR)((frame_info->real_length <= 35)?frame_info->real_length:35);
				if (raw_data)
				{	// show raw data
					txt = "";

					for( j=0; j<num_chars; j++ )
					{	sprintf((char *)(outstr+strlen((const char *)outstr)),
							"%02X  \0", (BYTE)frame_info->data[j]);

						qstr.sprintf("%02X ", (BYTE)frame_info->data[j]);
						txt += qstr;
					}
				}
				else
				{ // show interpreted data
					dlci = (USHORT)frame_info->data[0];
					dlci = (dlci << 2) | (frame_info->data[1] >> 4);
					if (dlci != 0)
					{	sprintf((char *)(outstr+strlen((const char *)outstr)),
							"Data Frame on DLCI %d", dlci);
						txt.sprintf("Data Frame on DLCI %d", dlci);
					} 
					else
					{	if(frame_info->data[9])
						{	sprintf((char *)(outstr+strlen((const char *)outstr)), "Link Verification");
							txt = "Link Verification";
						}else
						{	sprintf((char *)(outstr+strlen((const char *)outstr)), "Full Status");
							txt = "Full Status";
						}

						if (frame_info->data[5] == 0x75)
						{	sprintf((char *)(outstr+strlen((const char *)outstr)), "\tRequest");
							txt += "  Request";
						}else
						{	sprintf((char *)(outstr+strlen((const char *)outstr)), "\tReply");
							txt += "  Reply";
						}

						sprintf((char *)(outstr+strlen((const char *)outstr)),
							"\tSx %02X  Rx %02X  ",
							(BYTE)frame_info->data[0x0C], (BYTE)frame_info->data[0x0D] );

						sprintf(s1,	"   Sx %02X  Rx %02X  ",
							(BYTE)frame_info->data[0x0C], (BYTE)frame_info->data[0x0D] );

						txt += s1;

						if ( !frame_info->data[9] && (frame_info->data[5]==0x7D) )
						{	
							// full status reply
							for( j=0x0E; ( ((j+5)<frame_info->real_length) ); j+=5 )//show all DLCIs
							{
								dlci = (USHORT)frame_info->data[j+2];
								dlci = (dlci << 4) | ((frame_info->data[j+3]>>3) & 0x0F);
								sprintf((char *)(outstr+strlen((const char *)outstr)), "DLCI %d ", dlci);

								sprintf(s1, "  DLCI %d ", dlci);
								txt += s1;

								if (frame_info->data[j+4] & 0x08)
								{	sprintf((char *)(outstr+strlen((const char *)outstr)), "NEW ", dlci);
									sprintf(s1, "NEW ", dlci);
								}else
								{	sprintf((char *)(outstr+strlen((const char *)outstr)), "PRS ", dlci);
									sprintf(s1, "PRS ", dlci);
								}
								txt += s1;

								if (frame_info->data[j+4] & 0x02)
								{	sprintf((char *)(outstr+strlen((const char *)outstr)), "ACT\t", dlci);
									sprintf(s1, "ACT\t", dlci);
								}else
								{	sprintf((char *)(outstr+strlen((const char *)outstr)), "INACT\t", dlci);
									sprintf(s1, "INACT\t", dlci);
								}
								txt += s1;
							}
						}
					}
				}

				list_view_item->setText(DATA_COL, QString::fromLatin1((const char *)txt));

				if (trace_ind == 2)
				{
					for( j=0; j<frame_info->real_length; j++ )
					{
						if (j%25 == 0) fprintf( globalTesterWizPtr->trace_file, "\n" );
							fprintf( globalTesterWizPtr->trace_file, "%02X ", (BYTE)frame_info->data[j]);
					}
				}
			}

			if (trace_ind == 2)
			{	fprintf( globalTesterWizPtr->trace_file, "\n\n>>>> FrameData ENDS.\n");
			}
			// we have to stay on even boundary here, so update the curr_pos
			// if needed. It's done exactly the same way in driver. Si, it's cool
			if (curr_pos & 0x0001) curr_pos++;

			LIST_VIEW_ADDSTR(ListView_FrLineTrace, list_view_item);

			if (trace_ind == 2)
			{	fflush(globalTesterWizPtr->trace_file);
				// get file size
				txt.sprintf("%u bytes written", ftell(globalTesterWizPtr->trace_file));
				txt_trace_file_size->setText((const char *)txt);
			}

		}//for ( i=0; i < num_frames; i++)

		// now check if we have to force next poll
		if(cb.data[0] & MORE_TRC_DATA)
		{	//SendMessage( hDlg, MSG_POLL, 0, 0L );
		}
	}//if (globalTesterWizPtr->fcb.return_code == 0
}


BOOL Fr_LineTrace_Impl::EnableTrace()
{
	int i;

	////////////////////////////
	ListView_FrLineTrace->clear();
  
	CheckBox_stop_autoscroll->setChecked(FALSE);

	//disable so file name will not be changed while we are here
	globalTesterWizPtr->fr_menu_line_trace->
		setItemEnabled (globalTesterWizPtr->fr_menu_line_trace->idAt(1), FALSE);

	txt_trace_file_name->setText(globalTesterWizPtr->trace_file_name);
	txt_trace_file_size->setText("");
	////////////////////////////

	DisableTrace();//just in case it was not disabled

	if (tracing_enabled == FALSE)
	{
		for(i = 0; i < 5; i++)
		{
	   		cb.command = FR_SET_TRACE_CONFIGURATION;//ENABLE_TRACING;
			cb.dlci = 0;
			cb.buffer_length = 1;
			cb.data[0] = 0x37;
			memcpy(&globalTesterWizPtr->fcb, &cb, sizeof(cb));


			globalTesterWizPtr->Fr_DoCommand();

			memcpy(&cb, &globalTesterWizPtr->fcb, sizeof(cb));

			DbgPrint2(DBG_FR_TRACE, "cb.return_code : 0x%x\n", cb.return_code);

			if(cb.return_code == 0)
			{	//frame_count = 0;
				tracing_enabled = TRUE;
				break;
			}
		}

		if(cb.return_code)
		{	txt.sprintf("Failed to Enable Line Tracing. Return code: 0x%02X",
				cb.return_code );
			QMessageBox::warning(0, "", (const char *)txt);
			tracing_enabled = FALSE;
		}
	}

	return tracing_enabled;
}

void Fr_LineTrace_Impl::DisableTrace()
{
	int i;

	if (tracing_enabled == TRUE)
	{	for(i = 0; i < 5; i++)
		{	cb.command = FR_SET_TRACE_CONFIGURATION;//DISABLE_TRACING;
			//cb.buffer_length = 0;//buffer length zero indicates to DLL to disable trace
			cb.dlci = 0;
			cb.buffer_length = 1;
			cb.data[0] = 0x36;

			memcpy(&globalTesterWizPtr->fcb, &cb, sizeof(cb));

			globalTesterWizPtr->Fr_DoCommand();

			memcpy(&cb, &globalTesterWizPtr->fcb, sizeof(cb));

			if (cb.return_code == 0) 
			{	/* close the tracing window */
				tracing_enabled = FALSE;
				break;
			}
		}

		if(cb.return_code)
		{	txt.sprintf("Failed to Disable Line Tracing. Return code: 0x%02X", 
				cb.return_code );
			QMessageBox::warning(0, "", (const char *)txt);
		}
	}
}

void Fr_LineTrace_Impl::on_show_raw_hex_data()
{	raw_data = 1;
}

void Fr_LineTrace_Impl::on_show_fr_data()
{	raw_data = 0;
}

void Fr_LineTrace_Impl::on_stop_trace_to_file()
{	trace_ind = 0;
}

void Fr_LineTrace_Impl::on_start_trace_to_file()
{	
	if(globalTesterWizPtr->trace_file == NULL)
	{	QMessageBox::information(0, "", no_trace_file_message);
	}else
		trace_ind = 2;
}
