/***************************************************************************
LineTrace_Impl.cpp  -  description
-------------------
begin                : Thu Oct 18 2001
copyright            : (C) 2001 by DavidR
email                : drokhvarg@sangoma.com
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

#include "MainWizardImpl.h"

extern TesterWizardImpl * globalTesterWizPtr;

//Common columns
#define DIR_COL     0
#define LEN_COL     1
#define TIME_COL    2

//X25 level trace columns
#define Q_BIT_COL   3
#define D_BIT_COL   4
#define LCN_COL     5
#define P_R_COL     6
#define P_S_COL     7
#define M_BIT_COL   8
#define DATA_COL    9

//HDLC level trace columns
#define CMD_RESP_COL    3
#define TYPE_COL        4
#define NR_COL          5
#define PF_COL          6
#define NS_COL          7
#define HDLC_DATA_COL   8

LineTrace_Impl::LineTrace_Impl( QWidget* parent,
							   const char* name,
							   WFlags fl):LineTrace(parent, name, fl )
{
	set_widget_caption(this);
	
	trace_ind = 0;
	label_trace_File_Size->setText("");
	
	show_interpreted = TRUE;
	LoopCount = 0xff;

	window_closed = true;

	on_ShowInterpretedTrace();
	
	ListView_HDLC_Level->setSorting(-1);
	ListView_HDLC_Level->setColumnText(DIR_COL, QString::fromLatin1("Direction"));//0
	ListView_HDLC_Level->setColumnAlignment(DIR_COL, Qt::AlignHCenter);
	ListView_HDLC_Level->addColumn("Len");      //1
	ListView_HDLC_Level->setColumnAlignment(LEN_COL, Qt::AlignHCenter);
	ListView_HDLC_Level->addColumn("Time");     //2
	ListView_HDLC_Level->setColumnAlignment(TIME_COL, Qt::AlignHCenter);
	ListView_HDLC_Level->addColumn("Cmd/Resp");    //3
	ListView_HDLC_Level->setColumnAlignment(CMD_RESP_COL, Qt::AlignHCenter);
	ListView_HDLC_Level->addColumn("Type");    //4
	ListView_HDLC_Level->setColumnAlignment(TYPE_COL, Qt::AlignHCenter);

	ListView_HDLC_Level->addColumn("N(R)");      //5
	ListView_HDLC_Level->setColumnAlignment(NR_COL, Qt::AlignHCenter);
	ListView_HDLC_Level->addColumn("P/F");     //6
	ListView_HDLC_Level->setColumnAlignment(PF_COL, Qt::AlignHCenter);
	ListView_HDLC_Level->addColumn("N(S)");     //7
	ListView_HDLC_Level->setColumnAlignment(NS_COL, Qt::AlignHCenter);
	ListView_HDLC_Level->addColumn("Data(Hex)                                                ");    //8
	ListView_HDLC_Level->clear();
	
}

LineTrace_Impl::~LineTrace_Impl()
{
}

void LineTrace_Impl::on_Close_Trace_Window()
{
	close();	
}

void LineTrace_Impl::initialize()
{
	globalTesterWizPtr = globalTesterWizPtr;

	window_closed = false;

	ListView_X25_Level->clear();
	ListView_HDLC_Level->clear();
	show_interpreted = TRUE;
	chk_StopAutoScroll->setChecked(FALSE);

	label_Trace_File_Name->setText(globalTesterWizPtr->trace_file_name);
	label_trace_File_Size->setText("");
	
	//disable so file name will not be changed while we are here
	globalTesterWizPtr->x25_menu_line_trace->
		setItemEnabled (globalTesterWizPtr->x25_menu_line_trace->idAt(1), FALSE);

	if(EnableTracing() == FALSE)	
		return;
}

void LineTrace_Impl::showEvent(QShowEvent * ev)
{
	set_widget_position_on_screen(this);
}

void LineTrace_Impl::closeEvent(QCloseEvent * e)
{
	DisableTracing();

	globalTesterWizPtr->x25_menu_line_trace->
		setItemChecked (globalTesterWizPtr->x25_menu_line_trace->idAt(0), FALSE );
	
	//enable so file name can be changed while we are NOT here
	globalTesterWizPtr->x25_menu_line_trace->
		setItemEnabled (globalTesterWizPtr->x25_menu_line_trace->idAt(1), TRUE );

	window_closed = true;

	e->accept();
}

//this is over ride of QObject's timer function
void LineTrace_Impl::timerEvent(QTimerEvent * e)
{
	if(window_closed == true)
	{	return;
	}

	LoopCount = 0;
	
	do{
		globalTesterWizPtr->xcb.command = READ_TRACE_DATA;
		globalTesterWizPtr->xcb.buffer_length = 0;
		globalTesterWizPtr->X25_DoCommand();
		
		memcpy(&cb, &globalTesterWizPtr->xcb, sizeof(cb));
		
		if (cb.return_code == 0x00 && chk_StopAutoScroll->isChecked() == FALSE)
		{
			///////////////////////////////
			if(show_interpreted == TRUE)  ShowInterpretedTrace();
			else                          ShowRawTrace();
			///////////////////////////////
		}//if (cb.return_code == 0x00...
	} while ((cb.return_code == 0x00) && (++LoopCount<15));
	
	if (trace_ind == 2)
	{	// get file size
		sprintf( tmpstr, "%d bytes written", ftell(globalTesterWizPtr->trace_file) );
		label_trace_File_Size->setText( tmpstr );
		fflush(globalTesterWizPtr->trace_file);
	}
}

void LineTrace_Impl::ShowRawTrace()
{
	int indx;
	UCHAR	tt[50];	
	
	//all output goes to X25 level window
	list_view_item_X25_level =
		new QListViewItem(ListView_X25_Level, ListView_X25_Level->lastItem());
	
	switch (cb.data[2]&0x03)
	{
	case 0x00:	sprintf( StrHDLC, "RX  " );
		sprintf(list_view_buff, "RX");
		break;
	case 0x01: 	sprintf( StrHDLC, "TX  " );
		sprintf(list_view_buff, "TX");
		break;
	case 0x02:  sprintf( StrHDLC, "HDLC Err  " );
		sprintf(list_view_buff, "HDLC Err");
		break;
	}
	
	list_view_item_X25_level->setText(DIR_COL, QString::fromLatin1( list_view_buff ));
	
	sprintf( StrHDLC+strlen(StrHDLC), "%d\t", *(unsigned short *)&cb.data[0]);//length
	
	sprintf(list_view_buff, "%d", *(unsigned short *)&cb.data[0]);
	list_view_item_X25_level->setText(LEN_COL, QString::fromLatin1( list_view_buff ));
	
	sprintf( StrHDLC+strlen(StrHDLC), "%d\t", *(unsigned short *)&cb.data[9]);//timestamp
	
	sprintf(list_view_buff, "%d", *(unsigned short *)&cb.data[9]);
	list_view_item_X25_level->setText(TIME_COL, QString::fromLatin1( list_view_buff ));
	
	int FrLength = *(unsigned short *)&cb.data[0];
	
	sprintf(list_view_buff, "");
	
	for(indx = 0; indx < FrLength; indx++)
	{	sprintf((char *)tt, " %02X", cb.data[0x0b + indx]);
	strcat((char *)StrHDLC, (char *)tt);
	sprintf(list_view_buff + strlen(list_view_buff), "%02X ", cb.data[0x0b + indx]);
	}
	//it is raw Hex data column
	list_view_item_X25_level->setText(Q_BIT_COL, QString::fromLatin1( list_view_buff ));
	
	LIST_VIEW_ADDSTR(ListView_X25_Level, list_view_item_X25_level);
	
	if (trace_ind == 2 )
	{	fprintf( globalTesterWizPtr->trace_file, "\n");
	fprintf( globalTesterWizPtr->trace_file, StrHDLC);
	fprintf( globalTesterWizPtr->trace_file, "\n");
	}
}

void LineTrace_Impl::ShowInterpretedTrace()
{
	list_view_item_HDLC_level =
        new QListViewItem(ListView_HDLC_Level, ListView_HDLC_Level->lastItem());
	
	if (trace_ind == 2 ) {
		fprintf( globalTesterWizPtr->trace_file, "\n------------------------------------------------------------");
	}
	switch (cb.data[2]&0x03)
	{
	case 0x00:	sprintf( StrHDLC, "RX  " );
		sprintf(list_view_hdlc_buff, "RX");
		if (trace_ind == 2 ) {
			fprintf( globalTesterWizPtr->trace_file, "\nFrame Direction       : INCOMING\n");
		}
		break;
	case 0x01: 	sprintf( StrHDLC, "TX  " );
		sprintf(list_view_hdlc_buff, "TX");
		if (trace_ind == 2 ) {
			fprintf( globalTesterWizPtr->trace_file, "\nFrame Direction       : OUTGOING\n");
		}
		break;
	case 0x02:  sprintf( StrHDLC, "HDLC Err  " );
		sprintf(list_view_buff, "HDLC Err");
		break;
	}
	list_view_item_HDLC_level->setText(DIR_COL, QString::fromLatin1( list_view_hdlc_buff ));
	//////////
	sprintf( StrHDLC+strlen(StrHDLC), "%d\t", *(unsigned short *)&cb.data[0]);
	
	sprintf(list_view_hdlc_buff, "%d", *(unsigned short *)&cb.data[0]);
	list_view_item_HDLC_level->setText(LEN_COL, QString::fromLatin1( list_view_hdlc_buff ));
	///////////
	
	if (trace_ind == 2 ) {
		fprintf( globalTesterWizPtr->trace_file, "Frame Length in Bytes : %d\n", *(unsigned short *)&cb.data[0]);
	}
	sprintf( StrHDLC+strlen(StrHDLC), "%d\t", *(unsigned short *)&cb.data[9]);
	
	sprintf(list_view_hdlc_buff,  "%d", *(unsigned short *)&cb.data[9]);
	list_view_item_HDLC_level->setText(TIME_COL, QString::fromLatin1( list_view_hdlc_buff ));
	///////////
	if(trace_ind == 2 ) {
		fprintf( globalTesterWizPtr->trace_file, "Time Stamp            : %d\n", *(unsigned short *)&cb.data[9]);
	}
	
	IsX25 = ProcessHDLCLevel(StrHDLC+strlen(StrHDLC), &cb.data[0x0b]);
	
	if (IsX25 == (int)true)
	{
        list_view_item_X25_level =
			new QListViewItem(ListView_X25_Level, ListView_X25_Level->lastItem());
		
		// we have an X.25 info frame - process it. The X.25 frame is 4 bytes shorter than HDLC
		switch (cb.data[2]&0x03) {
		case 0x00:	sprintf( StrX25, "RX  " );
			sprintf(list_view_buff, "RX");
			break;
		case 0x01: 	sprintf( StrX25, "TX  " );
			sprintf(list_view_buff, "TX");
			break;
		}
		
        list_view_item_X25_level->setText(DIR_COL, QString::fromLatin1( list_view_buff ));
        //////////////
		sprintf( StrX25+strlen(StrX25), "%d\t", *(unsigned short *)&cb.data[0] - 4);
		
        sprintf(list_view_buff, "%d", *(unsigned short *)&cb.data[0] - 4);
        list_view_item_X25_level->setText(LEN_COL, QString::fromLatin1( list_view_buff ));
        ////////////////
		sprintf( StrX25+strlen(StrX25), "%d\t", *(unsigned short *)&cb.data[9]);
		
        sprintf(list_view_buff, "%d", *(unsigned short *)&cb.data[9]);
        list_view_item_X25_level->setText(TIME_COL, QString::fromLatin1( list_view_buff ));
        ////////////////
		ProcessX25Level(StrX25+strlen(StrX25), &cb.data[0x0d], *(unsigned short *)&cb.data[0] - 4);
		
        LIST_VIEW_ADDSTR(ListView_X25_Level, list_view_item_X25_level);
	}
	LIST_VIEW_ADDSTR(ListView_HDLC_Level, list_view_item_HDLC_level);
	
	
	if (trace_ind == 2 ) {
		fprintf( globalTesterWizPtr->trace_file, ">>>> HDLC Level Decoding :\n\n");
		fprintf( globalTesterWizPtr->trace_file, "    Len Time  Cmd/Rsp         Type     N(R)    P/F    N(S)\n");
		fprintf( globalTesterWizPtr->trace_file, "%s\n\n", StrHDLC );
		if (IsX25) {
			fprintf( globalTesterWizPtr->trace_file, ">>>> X.25 Level Decoding :\n\n");
			fprintf( globalTesterWizPtr->trace_file, "    Len Time    Q  D   LCN     P(R)    P(S)     M\n");
			fprintf( globalTesterWizPtr->trace_file, "%s\n\n", StrX25 );
		}
		// Trace data
		fprintf( globalTesterWizPtr->trace_file, ">>>> Raw Frame Data :\n");
		for( Index=0; Index<(*(unsigned short *)&cb.data[0]); Index++ ) {
			if (Index%16 == 0) fprintf( globalTesterWizPtr->trace_file, "\n");
			fprintf( globalTesterWizPtr->trace_file, "%02X ", cb.data[Index+0x0b]);
		}
		fprintf( globalTesterWizPtr->trace_file, "\n\n");
	}
}

void LineTrace_Impl::on_Flush_Trace_Window()
{
	ListView_HDLC_Level->clear();
	ListView_X25_Level->clear();
}

void LineTrace_Impl::on_Start_Trace_To_File()
{
	if(globalTesterWizPtr->trace_file == NULL)
	{	QMessageBox::information(0, "", no_trace_file_message);
	}else
		trace_ind = 2;
}

void LineTrace_Impl::Stop_Trace_To_File()
{
	trace_ind = 0;	
}

BOOL LineTrace_Impl::EnableTracing(void)
{
	char s[100];
	
	// perform enable tracing
	globalTesterWizPtr->xcb.command = OPERATE_DATALINE_MONITOR;
	globalTesterWizPtr->xcb.buffer_length = 2;
	globalTesterWizPtr->xcb.logical_channel = 0;
	globalTesterWizPtr->xcb.data[0] = 0x7f; //   01111111 - also with delayed mode. If not - put 7B
	globalTesterWizPtr->xcb.data[1] = 8;	   //	8 seconds maximum delay
	globalTesterWizPtr->X25_DoCommand();
	
	if (globalTesterWizPtr->xcb.return_code != 0)
	{
		sprintf( s, "Couldn't Enable Line Tracing. Return code %02X", cb.return_code );
		QMessageBox::warning(this, "Tracing Error", s );
		return FALSE;
	}
	return TRUE;
}

BOOL LineTrace_Impl::DisableTracing(void)
{
	// perform disable tracing
	globalTesterWizPtr->xcb.command = OPERATE_DATALINE_MONITOR;
	globalTesterWizPtr->xcb.buffer_length = 2;
	globalTesterWizPtr->xcb.logical_channel = 0;
	globalTesterWizPtr->xcb.data[0] = 0x00; //   00000000
	globalTesterWizPtr->X25_DoCommand();
	return TRUE;
}

// Make an HDLC level analisys of the given frame. Output goes to Str.
// Is this is an INFO frame the function returns TRUE. Otherwise FALSE
bool LineTrace_Impl::ProcessHDLCLevel(char *Str, BYTE *Frame)
{
	// Basic frame types
	const int	InfoFrame = 0;		// info frame
	const int	SuperFrame = 1;		// supervisory frame
	const int	UnnumFrame = 2;		// unnumbered frame
	const int	UnknownFrame =3;	// unknown frame
	

	int			FrameType;
	
	// Address field
	sprintf( Str, "%02X            ", Frame[0] );
	
	sprintf(list_view_hdlc_buff,  "%02X", Frame[0]);
	list_view_item_HDLC_level->setText(CMD_RESP_COL, QString::fromLatin1( list_view_hdlc_buff ));
	
	// Check if Info
	if (Frame[1] & 0x01) { // not info ...
		switch(Frame[1] & 0x0f) {
		case 0x01:	// RR
			FrameType = SuperFrame;
			sprintf( Str+strlen(Str), "RR    ");
			sprintf(list_view_hdlc_buff,  "RR");
			break;
		case 0x05:	// RNR
			FrameType = SuperFrame;
			sprintf( Str+strlen(Str), "RNR    ");
			sprintf(list_view_hdlc_buff,  "RNR");
			break;
		case 0x09:	// REJ
			FrameType = SuperFrame;
			sprintf( Str+strlen(Str), "REJ    ");
			sprintf(list_view_hdlc_buff,  "REJ");
			break;
		case 0x0d:	// SREJ
			FrameType = SuperFrame;
			sprintf( Str+strlen(Str), "SREJ    ");
			sprintf(list_view_hdlc_buff,  "SREJ");
			break;
		default:	// Unnumbered frames
			FrameType = UnnumFrame;
			switch(Frame[1]&0xef) {
			case 0x03:	// UI
				sprintf( Str+strlen(Str), "UI  ");
                sprintf(list_view_hdlc_buff,  "UI");
				break;
			case 0x0f:	// DM
				sprintf( Str+strlen(Str), "DM  ");
                sprintf(list_view_hdlc_buff,  "DM");
				break;
			case 0x2f:	// SABM
				sprintf( Str+strlen(Str), "SABM  ");
                sprintf(list_view_hdlc_buff,  "SABM");
				break;
			case 0x43:	// DISC
				sprintf( Str+strlen(Str), "DISC  ");
                sprintf(list_view_hdlc_buff,  "DISC");
				break;
			case 0x63:	// UA
				sprintf( Str+strlen(Str), "UA  ");
                sprintf(list_view_hdlc_buff,  "UA");
				break;
			case 0x87:	// FRMR
				sprintf( Str+strlen(Str), "FRMR  ");
                sprintf(list_view_hdlc_buff,  "FRMR");
				break;
			default:	// unknown
				FrameType = UnknownFrame;
				sprintf( Str+strlen(Str), "????  ");
                sprintf(list_view_hdlc_buff,  "????");
				break;
			}
			break;
		}
	} else { // Info frame
		FrameType = InfoFrame;
		sprintf( Str+strlen(Str), "INFO  ");
		sprintf(list_view_hdlc_buff,  "INFO");
	}
	
	list_view_item_HDLC_level->setText(TYPE_COL, QString::fromLatin1( list_view_hdlc_buff ));
	
	// N(R)
	if ((FrameType == InfoFrame) || (FrameType == SuperFrame)) {
		sprintf( Str+strlen(Str), "%d       ", (Frame[1]&0xe0)>>5);
		sprintf(list_view_hdlc_buff, "%d", (Frame[1]&0xe0)>>5);
	} else {
		sprintf( Str+strlen(Str), "         " );
		sprintf(list_view_hdlc_buff, " ");
	}
	list_view_item_HDLC_level->setText(NR_COL, QString::fromLatin1( list_view_hdlc_buff ));
	// P/F
	if (FrameType != UnknownFrame) {
		sprintf( Str+strlen(Str), "%d      ", (Frame[1]&0x10)>>4);
		sprintf(list_view_hdlc_buff, "%d", (Frame[1]&0x10)>>4);
	}
	list_view_item_HDLC_level->setText(PF_COL, QString::fromLatin1( list_view_hdlc_buff ));
	// N(S)
	if (FrameType == InfoFrame) {
		sprintf( Str+strlen(Str), "%d  ", (Frame[1]&0x0e)>>1);
		sprintf(list_view_hdlc_buff, "%d", (Frame[1]&0x0e)>>1);
		list_view_item_HDLC_level->setText(NS_COL, QString::fromLatin1( list_view_hdlc_buff ));
	}
	
	//show data in the packet
	if (FrameType == InfoFrame)
	{	unsigned short FrLength = *(unsigned short *)&cb.data[0];
	int Index;
	
    sprintf(list_view_hdlc_buff, "");
	
	if(FrLength > 4)
	{	for(Index = 0; Index < FrLength - 4; Index++)
	{
		sprintf(Str+strlen(Str), "%02X ", Frame[Index + 2]);
        sprintf(list_view_hdlc_buff+strlen(list_view_hdlc_buff), "%02X ", Frame[Index + 2]);
		//show maximum 6 bytes
		if(Index == 6) break;
	}
	if (Index == 6) sprintf(list_view_hdlc_buff+strlen(list_view_hdlc_buff), "...");
	
	list_view_item_HDLC_level->setText(HDLC_DATA_COL, QString::fromLatin1( list_view_hdlc_buff ));
	}
	}
	
	return (FrameType == InfoFrame);
}

// Make X25 level analisys of the given frame
bool LineTrace_Impl::ProcessX25Level(char *Str, BYTE *Frame, int FrLength)
{
	int	Index;
	
	// Q-bit
	sprintf(Str, "%d  ", Frame[0]>>7);
	
	sprintf(list_view_buff, "%d", Frame[0]>>7);
	list_view_item_X25_level->setText(Q_BIT_COL, QString::fromLatin1( list_view_buff ));
	
	// D-bit
	sprintf(Str+strlen(Str), "%d    ", (Frame[0]&0x40)>>6);
	
	sprintf(list_view_buff, "%d", (Frame[0]&0x40)>>6);
	list_view_item_X25_level->setText(D_BIT_COL, QString::fromLatin1( list_view_buff ));
	
	// LCN
	sprintf(Str+strlen(Str), "%d\t", (Frame[0]&0x0f)*256+Frame[1]);
	sprintf(list_view_buff, "%d", (Frame[0]&0x0f)*256+Frame[1]);
	list_view_item_X25_level->setText(LCN_COL, QString::fromLatin1( list_view_buff ));
	
	// Packet type
	FrLength -= 3;
	if (Frame[2]&0x01) {
		switch(Frame[2]) {
		case 0x0b:	// Call.Req/Incoming Call
			sprintf(Str+strlen(Str), "CALL Request/Incoming\t");
			sprintf(list_view_buff, "CALL Request/Incoming ");
			PrintDNAField(list_view_buff+strlen(list_view_buff), &Frame[3], FrLength);
			break;
		case 0x0f:	// Call Accept/Call Connect
			sprintf(Str+strlen(Str), "CALL Accept/Connect\t");
			sprintf(list_view_buff, "CALL Accept/Connect ");
			PrintDNAField(list_view_buff+strlen(list_view_buff), &Frame[3], FrLength);
			break;
		case 0x13:	// Clear Request/Clear Indication
			sprintf(Str+strlen(Str), "CLEAR Request/Indication\t");
			sprintf(list_view_buff, "CLEAR Request/Indication ");
			PrintCauseDiagnostics(list_view_buff+strlen(list_view_buff), &Frame[3], FrLength);
			FrLength -= 2;
			PrintDNAField(list_view_buff+strlen(list_view_buff), &Frame[5], FrLength);
			break;
		case 0x17:	// Clear Confirmation
			sprintf(Str+strlen(Str), "CLEAR Confirmation\t");
			sprintf(list_view_buff, "CLEAR Confirmation ");
			PrintCauseDiagnostics(list_view_buff+strlen(list_view_buff), &Frame[3], FrLength);
			FrLength -= 2;
			PrintDNAField(list_view_buff+strlen(list_view_buff), &Frame[5], FrLength);
			break;
		case 0x23:	// Interrupt
			sprintf(Str+strlen(Str), "INTERRUPT\t");
			sprintf(Str+strlen(Str), "User Data : %02X", Frame[3]);
			
			sprintf(list_view_buff, "INTERRUPT ");
			sprintf(list_view_buff+strlen(list_view_buff), "User Data : %02X", Frame[3]);
			break;
		case 0x27:	// Interrupt Confirmation
			sprintf(Str+strlen(Str), "INTERRUPT Confirmation\t");
			sprintf(list_view_buff, "INTERRUPT Confirmation ");
			break;
		case 0x1b:	// Reset Request/Reset Indication
			sprintf(Str+strlen(Str), "RESET Request/Indication\t");
			sprintf(list_view_buff, "RESET Request/Indication ");
			PrintCauseDiagnostics(list_view_buff+strlen(list_view_buff), &Frame[3], FrLength);
			break;
		case 0x1f:	// Reset Confirmation
			sprintf(Str+strlen(Str), "RESET Confirmation\t");
			sprintf(list_view_buff, "RESET Confirmation");
			break;
		case 0xfb:	// Restart Request/Restart Indication
			sprintf(Str+strlen(Str), "RESTART Request/Indication\t");
			sprintf(list_view_buff, "RESTART Request/Indication ");
			PrintCauseDiagnostics(list_view_buff+strlen(list_view_buff), &Frame[3], FrLength);
			break;
		case 0xff:	// Restart Confirmation
			sprintf(Str+strlen(Str), "RESTART Confirmation\t");
			sprintf(list_view_buff, "RESTART Confirmation ");
			break;
		case 0xf1:	// Diagnostic
			sprintf(Str+strlen(Str), "DIAGNOSTIC.\tData : ");
			for (Index = 3; Index < FrLength; Index++) {
				sprintf(Str+strlen(Str), "%02X ", Frame[Index]);
			}
			
			sprintf(list_view_buff, "DIAGNOSTIC. Data : ");
			for (Index = 3; Index < FrLength; Index++) {
				sprintf(list_view_buff+strlen(list_view_buff), "%02X ", Frame[Index]);
			}
			break;
		default:	// Others...
			switch(Frame[2]&0x1f) {
			case 0x01:	// RR
				PrintPR(Str+strlen(Str), &Frame[2]);
				sprintf(Str+strlen(Str), "RR ");
				sprintf(list_view_buff, "RR ");
				break;
			case 0x05:	// RNR
				PrintPR(Str+strlen(Str), &Frame[2]);
				sprintf(Str+strlen(Str), "RNR ");
				sprintf(list_view_buff, "RNR ");
				break;
			case 0x09:	// REJ
				PrintPR(Str+strlen(Str), &Frame[2]);
				sprintf(Str+strlen(Str), "REJ ");
				sprintf(list_view_buff, "REJ ");
				break;
			default:	// Unknown
				sprintf(list_view_buff, "???? ");
				sprintf(Str+strlen(Str), "???? ");
				break;
			}
			break;
		}
		
		list_view_item_X25_level->setText(DATA_COL, QString::fromLatin1( list_view_buff ));
		
	} else { // Data packet
		// P(R)
		sprintf(Str+strlen(Str), "%d       ", (Frame[2]&0xe0) >> 5);
		
		sprintf(list_view_buff, "%d", (Frame[2]&0xe0) >> 5);
		list_view_item_X25_level->setText(P_R_COL, QString::fromLatin1( list_view_buff ));
		// P(S)
		sprintf(Str+strlen(Str), "%d       ", (Frame[2]&0x0e) >> 1);
		
		sprintf(list_view_buff, "%d", (Frame[2]&0x0e) >> 1);
		list_view_item_X25_level->setText(P_S_COL, QString::fromLatin1( list_view_buff ));
		// M-bit
		sprintf(Str+strlen(Str), "%d\t", (Frame[2]&0x10) >> 4);
		
		sprintf(list_view_buff, "%d", (Frame[2]&0x10) >> 4);
		list_view_item_X25_level->setText(M_BIT_COL, QString::fromLatin1( list_view_buff ));
		//Data
		sprintf(list_view_buff, "DATA ");
		sprintf(Str+strlen(Str), "DATA ");
		//
		for(Index = 0; Index < FrLength; Index++)
		{
			sprintf(Str+strlen(Str), "%02X ", Frame[Index + 3]);
			sprintf(list_view_buff+strlen(list_view_buff), "%02X ", Frame[Index + 3]);
			
			if(Index == 30) break;
		}
		if (Index == 30)
		{ sprintf(list_view_buff+strlen(list_view_buff), "...");
		sprintf(Str+strlen(Str), "...");
		}
		
		list_view_item_X25_level->setText(DATA_COL, QString::fromLatin1( list_view_buff ));
	}
	return true;
}

// Prints P(R) number
void LineTrace_Impl::PrintPR(char *Str, BYTE *Frame)
{
	sprintf( Str, "%d\t\t", (Frame[0]&0xe0) >> 5 );
	
	sprintf(list_view_buff, "%d", (Frame[0]&0xe0) >> 5);
	list_view_item_X25_level->setText(P_R_COL, QString::fromLatin1( list_view_buff ));
	return;
}

void LineTrace_Impl::PrintCauseDiagnostics(char *Str, BYTE *Frame, int FrLength)
{
	if (FrLength <= 0) return;
	// Cause
	sprintf( Str, " Cause: %02X ", Frame[0] );
	if (--FrLength <= 0) return;
	// Diag
	sprintf( Str+strlen(Str), " Diag: %02X ", Frame[1] );
	return;
}

void LineTrace_Impl::PrintDNAField(char *Str, BYTE *Frame, int FrLength)
{
	int	CalledDNALen;
	int	CallingDNALen;
	int	FacilityLen;
	int	Index;
	int	Index1;
	
	if (FrLength <= 0) return;
	
	CallingDNALen = (int)(Frame[0] >> 4);
	CalledDNALen = (int)(Frame[0]&0x0f);
	if (--FrLength <= 0) return;
	// Called DNA
	if (CalledDNALen != 0) {
		sprintf( Str, "-d");
	}
	
	for(Index = 1; Index <= CalledDNALen/2; Index++) {
		sprintf(Str+strlen(Str), "%02X", Frame[Index]);
		FrLength--;
	}
	
	// If CalledDNALen is odd - print another half a byte
	if (CalledDNALen&0x0001) {
		sprintf(Str+strlen(Str), "%01X", Frame[Index]>>4);
		if (CallingDNALen != 0) {
			sprintf(Str+strlen(Str), " -s%01X", Frame[Index]&0x0f);
			CallingDNALen--;
		}
		Index++;
		FrLength--;
	} else {
		if (CallingDNALen != 0) {
			sprintf(Str+strlen(Str), " -s", Frame[Index]&0x0f);
		}
	}
	
	// Calling DNA
	for(Index1 = Index; Index1 <= Index-1+CallingDNALen/2; Index1++) {
		sprintf(Str+strlen(Str), "%02X", Frame[Index1]);
		FrLength--;
	}
	// If CallingDNALen is odd - print another half a byte
	if (CallingDNALen&0x0001) {
		sprintf(Str+strlen(Str), "%01X", Frame[Index1]>>4);
		Index1++;
		FrLength--;
	}
	if(FrLength <=0) return;
	// Index1 now points to the begining of Facility field
	FacilityLen = Frame[Index1];
	FrLength--;
	if (FacilityLen != 0) {
		sprintf(Str+strlen(Str), " -f");
	}
	for ( Index = Index1+1; Index <= Index1+FacilityLen; Index++ ) {
		sprintf(Str+strlen(Str), "%02X", Frame[Index]);
		FrLength--;
	}
	
	// Index now points to the first byte of User Data field
	if(FrLength <=0) return;
	sprintf(Str+strlen(Str), " -u");
	for (Index1 = Index; FrLength>0; Index1++) {
		sprintf(Str+strlen(Str), "%02X", Frame[Index1]);
		FrLength--;
	}
	
	// DONE !!!
	return;
}

void LineTrace_Impl::on_ShowRawTrace()
{
	show_interpreted = FALSE;
	
	//remove columns starting from Q-bit columns
	int column_count = ListView_X25_Level->columns();
	
	for(int i = column_count - 1; i >= Q_BIT_COL; i--)
	{ ListView_X25_Level->removeColumn(i);
	}
	
	
	ListView_X25_Level->addColumn
		("Raw Hex Trace                                                                                                                                          ");//9
	
	ListView_HDLC_Level->clear();
	ListView_X25_Level->clear();
	
	TextLabel2->setText("Raw Hex Trace");
}

void LineTrace_Impl::on_ShowInterpretedTrace()
{
	show_interpreted = TRUE;
	
	//remove all columns first
	int column_count = ListView_X25_Level->columns();
	
	for(int i = column_count - 1; i >= 0; i--)
	{ ListView_X25_Level->removeColumn(i);
	}
	
	ListView_X25_Level->setSorting(-1);
	ListView_X25_Level->addColumn("Direction");//0
	ListView_X25_Level->setColumnAlignment(DIR_COL, Qt::AlignHCenter);
	ListView_X25_Level->addColumn("Len");      //1
	ListView_X25_Level->setColumnAlignment(LEN_COL, Qt::AlignHCenter);
	ListView_X25_Level->addColumn("Time");     //2
	ListView_X25_Level->setColumnAlignment(TIME_COL, Qt::AlignHCenter);
	ListView_X25_Level->addColumn("Q-bit");    //3
	ListView_X25_Level->setColumnAlignment(Q_BIT_COL, Qt::AlignHCenter);
	ListView_X25_Level->addColumn("D-bit");    //4
	ListView_X25_Level->setColumnAlignment(D_BIT_COL, Qt::AlignHCenter);
	ListView_X25_Level->addColumn("LCN");      //5
	ListView_X25_Level->setColumnAlignment(LCN_COL, Qt::AlignHCenter);
	ListView_X25_Level->addColumn("P(R)");     //6
	ListView_X25_Level->setColumnAlignment(P_R_COL, Qt::AlignHCenter);
	ListView_X25_Level->addColumn("P(S)");     //7
	ListView_X25_Level->setColumnAlignment(P_S_COL, Qt::AlignHCenter);
	ListView_X25_Level->addColumn("M-bit");    //8
	ListView_X25_Level->setColumnAlignment(M_BIT_COL, Qt::AlignHCenter);
	ListView_X25_Level->addColumn
		("                                                                                                                   ");//9
	ListView_X25_Level->clear();
	
	TextLabel2->setText("X.25 Level Trace");
}

