/****************************************************************************
** header file of TesterWizardImpl class
**
** Created: Thu Sep 6 14:59:35 2001
**      by:  David Rokhvarg		<drokhvarg@sangoma.com>
**
****************************************************************************/
#ifndef TESTERWIZARDIMPL_H
#define TESTERWIZARDIMPL_H

#include "tester.h"       //all needed Qt header files
#include "MainWizard.h"   //parent class for the application's main window
#include "TesterDefines.h"//debug macros

#include "frelay.h"		//Fr code offsets, commands, structures

#include "phyapi.h"		// interface for the ATM firmware

#include "fapi.h"
#include "xapi.h"
#include "atm_api.h"	// ATM class wich does all the work

#if defined(BUILD_FOR_UNIX)

#define FR_FIRMWARE_FILE	"/etc/wanpipe/firmware/fr514.sfm"
#define X25_FIRMWARE_FILE	"/etc/wanpipe/firmware/x25_514.sfm"
#define ATM_FIRMWARE_FILE	"/etc/wanpipe/firmware/atm514.sfm"

#elif defined(WIN32)

#define FR_FIRMWARE_FILE	"\\fr514.sfm"
#define X25_FIRMWARE_FILE	"\\x25_514.sfm"
#define ATM_FIRMWARE_FILE	"\\atm514.sfm"

#endif


#include "api_macros.h"

extern X25_CONFIG_STRUCT * global_x25cfg_struct;
extern char no_trace_file_message[];

extern char dlci_list[MAX_DLCIS+1];

//////////////////////////////////////////
//global functions
unsigned short get_edit_box_value(QLineEdit * ed, bool * ok);
unsigned int get_int_hex_value_from_line_edit_box(QLineEdit * ed, bool * ok);

void set_widget_position_on_screen(QWidget * widget);
void set_widget_caption(QWidget * widget);

int ParseTextToHex(int length,
				   unsigned char * src,
				   unsigned char * destination
				   );


// End of global functions
//////////////////////////////////////////

//Frame Relay visual objects declarations
#include "Fr_SendInfoFrames_Impl.h"
#include "SetFrConfigImpl.h"
#include "FrDlciCommands_Impl.h"
#include "LineTrace_Impl.h"
#include "Fr_LinkStatus_Impl.h"
#include "Fr_GlobalStats_Impl.h"
#include "Fr_CpeSpecificStats_Impl.h"
#include "Fr_NodeSpecificStats_Impl.h"
#include "Fr_Comms_Err_Stats_Impl.h"
#include "Fr_ModemStatus_Impl.h"
#include "Fr_ConfiguredDlcis_Impl.h"
#include "Fr_LineTrace_Impl.h"
#include "Fr_DlciList_Impl.h"
#include "Fr_DLCI_Statistics_Impl.h"

//X25 visual objects declarations
#include "LinkStatus_Impl.h"
#include "HDLC_Oper_Stats_Impl.h"
#include "HDLC_Comms_Err_Stats_Impl.h"
#include "ModemStatus_Impl.h"
#include "X25_Stats_Impl.h"
#include "X25_History_Impl.h"
#include "X25_LCN_Cfg_Impl.h"
#include "X25_ActiveChannels_Impl.h"

#include "X25_HDLC_FunctionsImpl.h"
#include "X25_Timers_Impl.h"
#include "X25_LogicalChannelCfg_Impl.h"
#include "X25_GeneralFacilities_Impl.h"
#include "X25_PacketCfg_Impl.h"
#include "X25_Config_and_Response_Options_Impl.h"
#include "X25_CCITT_Facil_Impl.h"
#include "x25config_impl.h"
#include "DataEntryImpl.h"
#include "X25ManySendImpl.h"

//ATM visual objects declarations
#include "atm_set_configuration_impl.h"
#include "atm_line_trace_impl.h"
#include "atm_comms_err_stats_impl.h"
#include "atm_phy_oper_stats_impl.h"
#include "atm_set_atm_aal_configuration_impl.h"
#include "atm_add_vpi_vci_impl.h"
#include "atm_send_info_frames_impl.h"
#include "atm_vpi_vci_status_impl.h"
#include "atm_vpi_vci_list_impl.h"	//shows list, option to get operational stats.
#include "atm_delete_vpi_vci_impl.h"
#include "atm_connect_disconnect_pvc_impl.h"
#include "atm_send_atm_aal_loopback_impl.h"
#include "atm_oam_continuity_check_ctrl_impl.h"
#include "atm_phy_level_status_impl.h"

class TesterWizardImpl : public TesterWizard
{
    Q_OBJECT
		
public:
	//
	//functions
	//
  TesterWizardImpl( QWidget* parent = 0,
						        const char* name = 0,
						        bool modal = FALSE,//TRUE,
						        WFlags fl = 0 );
  ~TesterWizardImpl();


	int read_code_version();
	void Fr_OnTimer(int timer_id);
	BYTE X25_DoCommand();
	BYTE Fr_DoCommand();
	void X25_SayResult(void);
	void AddStringListBox(char * m);
	void X25_DecodeReturnCode(XCBLOCK * cb, char * s);
	void DumpAsyncData(XCBLOCK *cb, char *s);
	void CheckInABM(void);
	void X25_OnTimer(int timer_id);
	void EnableDisableButtons(bool flag);
	void DoManyRecv();
	void DoManySend();
	void OnBounce();
	void bounce();
	void copy_cb(XCBLOCK *dest, XCBLOCK *src);
	
    char qdm_buff[100];

    void FormQDMBuffer(XCBLOCK * cb);
    WORD GetDUI(BYTE *d);
	int RegisterCard();

	unsigned char m_cCard;
	
    int wait_for_poll_to_finish(UCHAR command);
    char xapi_notification;
    int rx_timer_id, tx_timer_id;
	//
	//data memebers common for all protocols
	//
	char is_registered;//is the application registered with the driver
	int connect_to_api(char * device_filename);	

	//objects which are actually the APIs to the protocols
	Fapi fapi_obj;
	Xapi xapi_obj;

#if defined(BUILD_FOR_UNIX)


#elif defined(WIN32)

	int CheckCardStatus();
	char CurrentDir[MAX_PATH];

#endif
	void FormPathToFirmwareFile();
	BOOL PrepareForLoading();
	void CompleteLoading();
	BOOL DoLoading(void);

	char codefname[256];

	//this will keep chosen protocol
	int m_intProtocol;
    //this is the loaded protocol
	int m_intProtocolLoaded;
	
	char busy_polling;
    QWaitCondition * wait_condition;
	
	FILE * trace_file;
	char trace_file_name[255];
	
	unsigned short number_of_channels_rx_data_available;
	unsigned short rx_data_channels[MAX_NUMBER_OF_LCNS];//4096
	unsigned short * lcn_ptr;
	
    //
	//x25 data members
	//
	XCBLOCK xcb;
    XCBLOCK auxb;
	
	char view_in_ascii;//if 0 view x25 incoming data in hex if 1 in ascii
	char send_hex_data;//if 1 send data as hex numbers if 0 as ascii chars
	char result_text[1000];//buffer for parsing return codes
	
	char tx_buffer[1024];
	
	unsigned short nsent, ntosend, manysize;
	bool fdomanysend;
	bool domanyrcv;
	bool dobounce;
	bool dorfes;
	WORD manychannel;
	WORD rmanychannel;
	WORD brchannel, bschannel;
	BOOL busy_in_command;
	char rewrite;
	unsigned long sent;
	
    //x25 visual objects
	X25Config_Impl x25_cfg;
	DataEntryImpl * x25_data_entry;
	X25ManySendImpl x25_send_many;
	X25_HDLC_FunctionsImpl x25_hdlc;
    LinkStatus_Impl * x25_hdlc_link_status;
    HDLC_Oper_Stats_Impl * x25_hdlc_oper_stats;
    HDLC_Comms_Err_Stats_Impl * x25_comms_err_stats;
    ModemStatus_Impl * x25_modem_status;
    X25_Stats_Impl * x25_stats;
    X25_History_Impl * x25_history;
    X25_ActiveChannels_Impl * x25_active_channels;
    LineTrace_Impl * x25_line_trace;
	
    QMenuBar * x25_menubar;
    QPopupMenu * x25_menu_hdlc_monitor;
    QPopupMenu * x25_menu_monitor;
    QPopupMenu * x25_menu_line_trace;
	
    //
	//Frame Relay data members
	//
	FCBLOCK fcb, backup_fcb;
    BYTE rcode;
	bool bConfigured;
	bool comms_enabled;
	bool fr_domanyrcv;
	void FrDoManyRecv();

	///////////////////////
	USHORT datlen, maxlen, n;
	DWORD ln;
	PFRAME_RELAY_READ_INFO pri;
	USHORT rxcount;
	UCHAR rxdata[14000];
	///////////////////////


	//FrRel visual objects
	SetFrConfigImpl set_fr_cfg;//frame rel configuration dialog
	Fr_DlciList_Impl fr_dlci_list;
    FrDlciCommands_Impl fr_dlci_commands;
    Fr_SendInfoFrames_Impl * fr_send_info_frames;

	Fr_LinkStatus_Impl * fr_link_status;
	Fr_GlobalStats_Impl * fr_global_stats;
	Fr_CpeSpecificStats_Impl * fr_cpe_stats;
	Fr_NodeSpecificStats_Impl * fr_node_stats;
	Fr_Comms_Err_Stats_Impl * fr_comms_err_stats;
	Fr_ModemStatus_Impl * fr_modem_status;
	Fr_ConfiguredDlcis_Impl * fr_configured_dlcis;
	Fr_LineTrace_Impl * fr_line_trace;

    QMenuBar * fr_menubar;
    QPopupMenu * fr_menu_link_monitor;
    QPopupMenu * fr_menu_dlci_monitor;
    QPopupMenu * fr_menu_line_trace;
	
    void Fr_DecodeReturnCode(FCBLOCK * cb);
    WORD FormDLCIList();

	void SaveCB(FCBLOCK *dest, FCBLOCK *src);
	BOOL CompCB(FCBLOCK *c1, FCBLOCK *c2);
	void ResetCB(FCBLOCK *c1);

	char s[300], s1[300], s2[300], s3[300];
    QListViewItem * list_view_item;
    QListViewItem * list_view_item_1;
    QListViewItem * list_view_item_2;
    QListViewItem * list_view_item_3;
    QListViewItem * list_view_item_4;
    QListViewItem * list_view_item_separator;
	
    QString messages;


	//
	//ATM members
	//
	unsigned char atm_rx_buffer[AAL_MAX_LENGTH_RX_USER_INFO];
	QString qstr_length;
	QString qstr_time_stamp;
	QString qstr_data;

	QString qstr_vpi;
	QString qstr_vci;

	QString qstr_pt;
	QString qstr_clp;
	QString qstr_misc_rx_bits;

	atm_api atm_api_obj;

    QMenuBar * atm_menubar;
    QPopupMenu * atm_phy_level_stats_menu;
	QPopupMenu * atm_atmal_level_status_and_stats_menu;
	QPopupMenu * atm_phy_level_trace_menu;

	void atm_read_data();

	void on_read_atm_statistics();
	//does actual 'set cfg' + 'enable comms'
	int atm_do_set_configuration();

	int atm_do_set_aal_configuration();
	int atm_do_add_vpi_vci();

	int atm_do_enable_phy_comms();
	int atm_do_aal_enable_sar();
	int atm_do_aal_connect_pvc();

	void start_poll_timers();
	void stop_poll_timers();

	void atm_read_atmal_exception_cond();
	void atm_parse_read_atmal_exception_structure(QString * result_string);

	unsigned char atm_do_command(ATM_MAILBOX_STRUCT * block, BYTE cardnumber);
	void atm_decode_return_code(ATM_MAILBOX_STRUCT * block);

	//visual object for setting ATM configuration
	atm_set_configuration_impl atm_phy_cfg;
	atm_set_atm_aal_configuration_impl atm_set_atm_aal_cfg;
	atm_add_vpi_vci_impl atm_add_vpi_vci_screen;

	atm_line_trace_impl * atm_line_trace_ptr;
	atm_comms_err_stats_impl * atm_comms_err_stats_ptr;
	atm_phy_level_status_impl * atm_phy_level_status_ptr;
	atm_phy_oper_stats_impl * atm_phy_oper_stats_ptr;
	atm_send_info_frames_impl * atm_send_info_frames_ptr;
	atm_vpi_vci_status_impl * atm_vpi_vci_status_ptr;
	atm_vpi_vci_list_impl * atm_vpi_vci_list_ptr;
	atm_delete_vpi_vci_impl * atm_delete_vpi_vci_ptr;
	atm_connect_disconnect_pvc_impl * atm_connect_disconnect_pvc_ptr;
	atm_oam_continuity_check_ctrl_impl * atm_oam_continuity_check_ctrl_ptr;
	atm_send_atm_aal_loopback_impl * atm_send_atm_aal_loopback_ptr;

	void ATM_OnTimer(int timer_id);
	unsigned char atm_read_phy_exception_condition();

	void atm_parse_phy_read_status_structure(
								PHY_STATUS_STRUCT * ptr_status_struct,
								QString * result_string,
								unsigned int command
								);

	QString phy_prefix, atmal_prefix;

	bool atm_sar_enabled;
	bool atm_pvc_connected;
	bool atm_phy_comms_enabled;

	bool atm_show_command_result;
	void atm_enable_disable_visual_controls(BOOL flag);

public slots:
    //common functions
    void defered_initialization();
    void another_page_shown();
    void showEvent(QShowEvent * ev);
    void closeEvent(QCloseEvent * e);
    bool on_load_clicked_slot();
	void next();
	void back();
    void reject();
	
	void get_trace_file_name();
	
    //X.25 functions
	void on_x25_configure();
	void timerEvent(QTimerEvent * e);
    void on_set_X25_configuration();
	
	void on_x25_place_call();
	void on_x25_reset_call();
	
    void on_x25_accept_call();
    void on_x25_clear_call();
	
    void on_x25_confirm_clear();
    void on_x25_confirm_reset();
	
    void on_x25_receive();
    void on_x25_receive_many();
    void on_x25_send();
    void on_x25_send_many();
	
    void on_x25_restart_clicked();
    void on_x25_confirm_restart();
	
    void on_x25_interrupt();
    void on_x25_confirm_interrupt();
	
    void on_x25_hdlc_clicked();
    void on_x25_flush_data_buff_clicked();
	
	void on_x25_start_monitor();
    void x25_monitor_exited();
	
    void on_x25mon_link_status();
    void on_x25mon_oper_stats();
    void on_x25mon_comm_err_stats();
    void on_x25mon_modem_stats();
	
    void on_x25mon_x25_stats();
    void on_x25mon_x25_history_table();
    void on_x25mon_x25_active_channels();
	
    void on_x25mon_open_line_trace_window();
    void on_x25mon_set_line_trace_file_name();
	
    void ResetCB(XCBLOCK *c1);
    void SaveCB(XCBLOCK *dest, XCBLOCK *src);
    bool CompCB(XCBLOCK *c1, XCBLOCK *c2);
	
    //Frame Relay functions
    int do_set_fr_configuration_command();
	void set_fr_configuration();
    void on_fr_activate_dlci_at_node();
    void on_fr_add_dlci_at_node();
    void on_fr_deactivate_dlci_at_node();
    void on_fr_delete_dlci_at_node();
    void on_fr_disable_comms();
    void on_fr_enable_comms();
    void on_fr_send_full_status_enquiry();
    void on_fr_send_info_frames();

    void on_frmon_link_status();
    void on_frmon_global_stats();
    void on_frmon_stn_specific_stats();
    void on_frmon_comm_err_stats();
    void on_frmon_modem_stats();
			
    void on_frmon_configured_dlci_and_stats();
		
    void on_frmon_open_line_trace_window();
    void on_frmon_set_line_trace_file_name();

	void TesterWizard_destroyed( QObject * p);

	void atm_on_phy_communication_err_stats();
	void atm_on_phy_operational_stats();
	void atm_on_phy_level_status();
	void atm_on_phy_open_trace();

	//called when button is clicked
	void atm_set_phy_configuration();
    void atm_set_atm_aal_configuration();
    void atm_add_vpi_vci();

	void atm_send_data();
	void atm_delete_vpi_vci();
	int atm_phy_enable_disable_communications();
	int atm_enable_disable_sar();
	int atm_connect_disconnect_pvc();

	int atm_do_aal_disable_sar();
	int atm_do_aal_disconnect_pvc();
	int atm_do_disable_phy_comms();

	void atm_on_atmal_pvc_status();
	void atm_on_atmal_pvc_statistics();

    void atm_send_atm_aal_loopback();

    void atm_recynch_phy_receiver();
    void atm_oam_continuity_check_cntrl();

};

#endif // TESTERWIZARDIMPL_H
