/***************************************************************************
                          X25_ActiveChannels_Impl.cpp  -  description
                             -------------------
    begin                : Tue Sep 17 2002
    copyright            : (C) 2002 by David Rokhvarg
    email                : drokhvarg@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "MainWizardImpl.h"

extern TesterWizardImpl * globalTesterWizPtr;

#define LCN_TYPE_COL  0
#define LCN_NUM_COL   1


X25_ActiveChannels_Impl::X25_ActiveChannels_Impl( QWidget* parent, const char* name,
																        WFlags fl):X25_ActiveChannels(parent, name, fl)
{
  //show in upper left corner of the screen
  //setGeometry(0, 0, width(), height());
  set_widget_caption(this);

  //remove all columns first
  int column_count = ListView_Active_LCNs->columns();

  for(int i = column_count - 1; i >= 0; i--)
  { ListView_Active_LCNs->removeColumn(i);
  }

  ListView_Active_LCNs->setSorting(-1);
  ListView_Active_LCNs->addColumn("Type");    //0
  ListView_Active_LCNs->setColumnAlignment(LCN_TYPE_COL, Qt::AlignHCenter);
  ListView_Active_LCNs->addColumn("LCN Number   ");  //1
  ListView_Active_LCNs->setColumnAlignment(LCN_NUM_COL, Qt::AlignHCenter);

	x25_lcn_cfg = new X25_LCN_Cfg_Impl();
}

X25_ActiveChannels_Impl::~X25_ActiveChannels_Impl()
{
	if(x25_lcn_cfg != NULL)			delete x25_lcn_cfg;
}


void X25_ActiveChannels_Impl::showEvent(QShowEvent * ev)
{
  set_widget_position_on_screen(this);
}

void X25_ActiveChannels_Impl::closeEvent(QCloseEvent * e)
{
  x25_lcn_cfg->hide();
  globalTesterWizPtr->x25_menu_monitor->
    setItemChecked (globalTesterWizPtr->x25_menu_monitor->idAt(2), FALSE );
	
  e->accept();
}

void X25_ActiveChannels_Impl::on_Close()
{
  close();
}

void X25_ActiveChannels_Impl::on_Show_LCN_Config()
{
  QListViewItem * list_view_item = ListView_Active_LCNs->currentItem();

  if(list_view_item == NULL)
  { QMessageBox::information(	this, "",	"No LCN was selected.", QMessageBox::Ok);
  }else
  { QString str = list_view_item->text(LCN_NUM_COL);

    ushort selected_lcn = str.toUShort((bool *)NULL, 10);

    x25_lcn_cfg->myShow(selected_lcn);
  }
}

//this is overwrite of QObject's timer function
void X25_ActiveChannels_Impl::timerEvent(QTimerEvent * e)
{
	int pvcl = 0;
	int pvch = 0;
	int l, ind;
	char s[100];
	char t;

	globalTesterWizPtr->xcb.command = X25_READ_HDLC_X25_CONFIGURATION;
	globalTesterWizPtr->xcb.buffer_length = 0;
	globalTesterWizPtr->xcb.logical_channel = 0;

  for(ind = 0; ind < 5; ind++)
  {
	  globalTesterWizPtr->X25_DoCommand();
    if(globalTesterWizPtr->xcb.return_code == 0)  break;
  }

	if (globalTesterWizPtr->xcb.return_code == 0)
  { pvcl = ((X25_CONFIG_STRUCT *)globalTesterWizPtr->xcb.data)->lowest_PVC;
	  pvch = ((X25_CONFIG_STRUCT *)globalTesterWizPtr->xcb.data)->highest_PVC;
  }

	globalTesterWizPtr->ResetCB(&cb);

  globalTesterWizPtr->xcb.command = X25_READ_ACTIVE_CHANNELS;
	globalTesterWizPtr->xcb.buffer_length = 0;
	globalTesterWizPtr->xcb.logical_channel = 0;

  for(ind = 0; ind < 5;ind++)
  { globalTesterWizPtr->X25_DoCommand();
    if (cb.return_code == 0)  break;
  }

  memcpy(&cb, &globalTesterWizPtr->xcb, sizeof(cb));

	if (cb.return_code == 0)
	{
	  if(globalTesterWizPtr->CompCB(&lastcb, &cb))
	  { //nothing changed since the previous poll
      return;
    }
  	globalTesterWizPtr->SaveCB(&lastcb, &cb);

    ListView_Active_LCNs->clear();

		for (int c=0; c < (int)cb.buffer_length/2; c++)
    {	
      QListViewItem * list_view_item =
        new QListViewItem(ListView_Active_LCNs, ListView_Active_LCNs->lastItem());

      l = *(unsigned short *)&cb.data[c*2];
      sprintf(s, "%u", l);
      list_view_item->setText(LCN_NUM_COL, QString::fromLatin1(s));

		  t = 'S';
  		if (l >= pvcl && l <= pvch)
	  	t = 'P';
  		sprintf(s, "%cVC", t);
      list_view_item->setText(LCN_TYPE_COL, QString::fromLatin1(s));

      LIST_VIEW_ADDSTR(ListView_Active_LCNs, list_view_item);
  	}
	}else
  { //QMessageBox::information(	this, "",	"Failed to read active LCNs.", QMessageBox::Ok);
  }
}
