/***************************************************************************
                          X25_History_Impl.cpp  -  description
                             -------------------
    begin                : Tue Oct 16 2001
    copyright            : (C) 2001 by David Rokhvarg
    email                : drokhvarg@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "MainWizardImpl.h"

extern TesterWizardImpl * globalTesterWizPtr;

static BYTE packet_codes[] =
	{ 0x0b, 0x0f, 0x13, 0x17, 0x23, 0x27, 0x1b,
		0x1f, 0xfb, 0xff, 0xf1, 0xf3, 0xf7, 0 };

static char * async_cause[] =
	{ "(Applic)", "(AutoRsp)", "(PrtclErr)", "(Timeout)", "(LinkRecov)" };

static char *packet_types[] = {
"CALL REQUEST ",
"CALL ACCEPT  ",
"CLEAR REQUEST",
"CLEAR CONFIRM",
"INTERRUPT    ",
"INT CONFIRM  ",
"RESET REQUEST",
"RESET CONFIRM",
"RESTART REQ  ",
"RESTART CONF ",
"DIAGNOSTIC   ",
"REGISTR REQ  ",
"REGISTR CONF ",
"RR           ",
"RNR          ",
"REJ          ",
"DATA         "
};


X25_History_Impl::X25_History_Impl( QWidget* parent,
																const char* name,
																WFlags fl):X25_History(parent, name, fl)
{
  set_widget_caption(this);

	update_flag = true;
	toggle_flag = false;
	button_Suspend->hide();//button not used for now, so hide it

  ListView_X25_History->setSorting(-1);
  ListView_X25_History->setColumnText(0, QString::fromLatin1("Direction"));//0
  ListView_X25_History->setColumnAlignment(0, Qt::AlignHCenter);
  ListView_X25_History->addColumn("Asynchronous Event");  //1
  ListView_X25_History->setColumnAlignment(1, Qt::AlignHCenter);
  ListView_X25_History->addColumn("LCN");                 //2
  ListView_X25_History->setColumnAlignment(2, Qt::AlignHCenter);
  ListView_X25_History->addColumn("Packet Type");         //3
  ListView_X25_History->setColumnAlignment(3, Qt::AlignHCenter);
  ListView_X25_History->addColumn("Cause");               //4
  ListView_X25_History->setColumnAlignment(4, Qt::AlignHCenter);
  ListView_X25_History->addColumn("Diagnostics");         //5
  ListView_X25_History->setColumnAlignment(5, Qt::AlignHCenter);
  ListView_X25_History->addColumn("Date & Time");         //6
  ListView_X25_History->setColumnAlignment(6, Qt::AlignHCenter);
  ListView_X25_History->clear();
}

X25_History_Impl::~X25_History_Impl()
{
}

void X25_History_Impl::on_Suspend()
{
  printf("X25_History_Impl::on_Suspend()\n");
}

void X25_History_Impl::on_Flush_Clicked()
{
	globalTesterWizPtr->xcb.command = X25_HISTORY_TABLE_CONTROL;
	globalTesterWizPtr->xcb.buffer_length = 1;
	globalTesterWizPtr->xcb.logical_channel = 0;
	globalTesterWizPtr->xcb.data[0] = 0x01;
  globalTesterWizPtr->wait_for_poll_to_finish(X25_HISTORY_TABLE_CONTROL);
}

void X25_History_Impl::on_Close_Clicked()
{
	close();
}

void X25_History_Impl::closeEvent(QCloseEvent * e)
{
	globalTesterWizPtr->x25_menu_monitor->
		setItemChecked (globalTesterWizPtr->x25_menu_monitor->idAt(1), FALSE );
	e->accept();
}

void X25_History_Impl::showEvent(QShowEvent * ev)
{
  set_widget_position_on_screen(this);
}

//this is over ride of QObject's timer function
void X25_History_Impl::timerEvent(QTimerEvent * e)
{
	globalTesterWizPtr->ResetCB(&cb);
	globalTesterWizPtr->xcb.command = X25_READ_X25_HISTORY_TABLE;
	globalTesterWizPtr->xcb.buffer_length = 0;
	globalTesterWizPtr->xcb.logical_channel = 0;	// for supervisor display
	globalTesterWizPtr->X25_DoCommand();

  memcpy(&cb, &globalTesterWizPtr->xcb, sizeof(cb));

	if (!globalTesterWizPtr->CompCB(&lastcb, &cb))
	{
		if (cb.return_code == 0)
		{
			globalTesterWizPtr->SaveCB(&lastcb, &cb);
  	  ListView_X25_History->clear();

			for (c=0; c < (cb.buffer_length / 11); c++)
			{
				i = c * 11;
        list_view_item = new QListViewItem(ListView_X25_History, ListView_X25_History->lastItem());

        //Direction
        sprintf(list_view_buff, "%cX", (cb.data[i]) ? 'T' : 'R');
        list_view_item->setText(0, QString::fromLatin1( list_view_buff ));
        //Asynchronous event
        sprintf(list_view_buff, "%s", HistCause((BYTE)cb.data[i]));
        list_view_item->setText(1, QString::fromLatin1( list_view_buff ));
        //LCN
        sprintf(list_view_buff, "%u", globalTesterWizPtr->GetDUI((BYTE *)cb.data+i+1));
        list_view_item->setText(2, QString::fromLatin1( list_view_buff ));
        //Packet type
        sprintf(list_view_buff, "%-13s", HistPacket((BYTE)cb.data[i+3]));
        list_view_item->setText(3, QString::fromLatin1( list_view_buff ));
        //Cause
        sprintf(list_view_buff, "%02X", (BYTE)cb.data[i+4]);
        list_view_item->setText(4, QString::fromLatin1( list_view_buff ));
        //Diagnostic
        sprintf(list_view_buff, "%02X", (BYTE)cb.data[i+5]);
        list_view_item->setText(5, QString::fromLatin1( list_view_buff ));
        //Date & Time
        sprintf(list_view_buff, "%02X/%02X %02X:%02X:%02X",
          (BYTE)cb.data[i+6],
          (BYTE)cb.data[i+7],
					(BYTE)cb.data[i+8],
          (BYTE)cb.data[i+9],
          (BYTE)cb.data[i+10]);
        list_view_item->setText(6, QString::fromLatin1( list_view_buff ));
        ////////////////
        LIST_VIEW_ADDSTR(ListView_X25_History, list_view_item);
			}
		}
	}
}

char * X25_History_Impl::HistPacket(BYTE type)
{
	int i;
	i=0;
	while (packet_codes[i] != type) {
		if (packet_codes[i] == 0)  return("");
		i++;
	}
	return(packet_types[i]);
}

char * X25_History_Impl::HistCause(BYTE cause)
{
	int i;
	switch(cause) {
	case 0x01:
		i = 0;
		break;
	case 0x02:
		i = 1;
		break;
	case 0x04:
		i = 2;
		break;
	case 0x08:
		i = 3;
		break;
	case 0x10:
		i = 4;
		break;
	default:
		return("-------------");
	}
	return(async_cause[i]);
}
