/***************************************************************************
api_base.cpp  -  description
-------------------
begin                : Thu Jul 31 2003
copyright            : (C) 2003 by David Rokhvarg
email                : drokhvarg@sangoma.com
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

#include "api_base.h"

#define DBG_API_BASE  0

//////////////////////////////////////////////////////////////////////////////////////////////

api_base::api_base()
{
	DbgPrint1(DBG_API_BASE,	"api_base::api_base()\n");
	
#if defined(BUILD_FOR_UNIX)
	
	/********** Strings & message tables.****************/
	
	progname = "ioctl_test";
	
	err_messages[0] = "Invalid command line syntax";  /* ERR_SYNTAX */
	err_messages[1] =	"Unknown error code";	           /* ERR_LIMIT */
	
	exec = 0;
	data = 0;
	ctrl = 0;
	dev = -1;
	
#elif defined(WIN32)
	api_base_control_mutex = CreateMutex(NULL, FALSE, NULL);
	cardoperation_mutex = CreateMutex(NULL, FALSE, NULL);
	
	DbgPrint2(DBG_API_BASE,	"api_base::atm_api() : cardoperation_mutex 0x%X\n",
		cardoperation_mutex);
	
	//	tx_mutex = CreateMutex(NULL, FALSE, "API_BASE_32tx_mutex");
	
	frdrv = INVALID_HANDLE_VALUE;

	/* set default timeout to 2 seconds */
	timeout_ticks = 2000;

#endif
	
	cardnumber = 0;
	internal_state = 0;
}

api_base::~api_base()
{
	DbgPrint1(DBG_API_BASE,	"api_base::~atm_api()\n");
}

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

int api_base::API_BASE_Setup(char * device_name)
{
	char cat_cmd[200], buff[200];
	char dev_not_configured_str[200];
	
	//Strings returned from shell command have Line Feed char
	//at the end, so for comparison we append 0x0A to the end of test string.
	memset(dev_not_configured_str, 0x00, sizeof(dev_not_configured_str));
	
	strcpy(dev_not_configured_str, "device is not configured!");
	dev_not_configured_str[strlen(dev_not_configured_str)] = 0x0A;
	
	//for the shell command string
	sprintf(cat_cmd, "cat %s", device_name);
	
	//get the shell process pointer
	FILE * proc_pipe = popen(cat_cmd, "r");
	if(!proc_pipe)
	{	DbgPrint2(DBG_API_BASE,
			"File %s is not in the proc file system. Or failed to run shell command.",
			device_name);
		return DEVICE_NOT_IN_PROCFILE_SYSTEM;
	}else
	{	memset(buff, 0x00, sizeof(buff));
		//read one line of command output
		fgets(buff, sizeof(buff), proc_pipe);
		pclose(proc_pipe);
	}
	
	//if the read line is identical to 'dev_not_configured_str', the device is not running
	if(!strcmp(buff, dev_not_configured_str))
	{	return OPEN_DEVICE_ERR;
	}
	
	dev = ::open(device_name, O_RDONLY);
	if (dev < 0)
	{	DbgPrint2(DBG_API_BASE, "Failed to open %s\n", device_name);
		return OPEN_DEVICE_ERR;
	}
	
	exec = (sdla_exec_t *)malloc(sizeof(sdla_exec_t));
	if(!exec)
	{	DbgPrint1(DBG_API_BASE, "Failed to allocate 'exec'\n");
		return MEMORY_ALLOC_ERR;
	}
	
	data = (UCHAR *)malloc(MAX_DATA_LENGTH);
	data_bakup = data;
	if(!data)
	{	DbgPrint1(DBG_API_BASE, "Failed to allocate 'data'\n");
		return MEMORY_ALLOC_ERR;
	}	
	
	ctrl = (edu_exec_cmd_t *)malloc(sizeof(edu_exec_cmd_t));
	if(!ctrl)
	{	DbgPrint1(DBG_API_BASE, "Failed to allocate 'ctrl'\n");
		return MEMORY_ALLOC_ERR;
	}	
	
	exec->magic = WANPIPE_MAGIC;
	exec->cmd = ctrl;
	exec->data = data;
	
	internal_state |= CONNECTED_TO_DRIVER;
	return 0;
}

#elif defined(WIN32)

int api_base::API_BASE_Setup(void *)
{
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_Setup()\n");
	
    if (frdrv != INVALID_HANDLE_VALUE)
	{	DbgPrint1(DBG_API_BASE, "API_BASE_Setup() : driver handle already open\n");
		return(2);
	}
	
    frdrv =
		CreateFile("\\\\.\\sdla",
		GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
		(LPSECURITY_ATTRIBUTES)NULL,
		OPEN_EXISTING, FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH,
		(HANDLE)NULL
		);
	
    if(frdrv == INVALID_HANDLE_VALUE)
		return(0);
	
	DbgPrint1(DBG_API_BASE, "API_BASE_Setup() OK\n");
	
	internal_state |= CONNECTED_TO_DRIVER;
	
	return(1);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

int api_base::API_BASE_Cleanup(void)
{
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_Cleanup()\n");
	
	if(exec){
		::free(exec);
		exec = 0;
	}
	
	if(data){
		free(data);
		data = 0;
	}
	
	if(ctrl){
		free(ctrl);
		ctrl = 0;
	}
	
	if(dev >= 0){
		close(dev);
		dev = -1;
	}
	
	internal_state &= ~CONNECTED_TO_DRIVER;
	return(1);
}

#elif defined(WIN32)

int api_base::API_BASE_Cleanup(void)
{
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_Cleanup()\n");
	
	if (frdrv != INVALID_HANDLE_VALUE)
		CloseHandle(frdrv);
	
	internal_state &= ~CONNECTED_TO_DRIVER;
	
	return(1);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

int api_base::API_BASE_RegisterCard(BYTE cardnumber)
{
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_RegisterCard()\n");
	
	int err;
	
	ctrl->ioctl = IOCTL_REGISTER_CARD;
	ctrl->buffer_length = 0;
	ctrl->offset = 0;
	err = DoIoctl();
	
	if(err){
		return err;
	}else if(ctrl->return_code == 1){
		return CARD_REGISTER_ERR;
	}else{
		return 0;
	}
}

#elif defined(WIN32)

/*
returns:
SDLAR_NOT_REGISTERED (0L)	- failed to obtain a selector
(any read or write calls to the board will result
in a general protection fault)
SDLAR_REGISTERED (1L)		- succeeded : the card's
instance count was incremented but this is not the
first registration call for this board (it is already
in use by an application)
SDLAR_ALLOCATED (2L)		- succeeded : the card's
instance count was incremented and this is the
first registration call for this board, and the
memory selector was allocated

		('cardnumber' is ignored on UNIX-based systems)
*/
int api_base::API_BASE_RegisterCard(BYTE cardnumber)
{
	LRESULT rcode;
	DWORD	mresult;
	CARD_TRANSACTION ct;
	
	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_RegisterCard()\n");
	
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED)
	{	return(0);
	}
	
	this->cardnumber = cardnumber;
	
	ct.cardnumber = cardnumber;
    DeviceIoControl(frdrv, IOCTL_SDLA_REGISTER, (LPVOID)&ct, sizeof(CARD_TRANSACTION),
		(LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	rcode = ct.rcode;
	
	if (rcode == 0) 
	{	ReleaseMutex(cardoperation_mutex);
		return(0);
	}
	
	DbgPrint3(DBG_API_BASE, "Card %u registered: return code %lx", cardnumber, rcode);
	
	ReleaseMutex(cardoperation_mutex);
	
	return((int)rcode);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

int api_base::API_BASE_DeregisterCard(BYTE cardnumber)
{
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_DeregisterCard()\n");
	
	int err;
	
	ctrl->ioctl = IOCTL_DEREGISTER_CARD;
	ctrl->buffer_length = 0;
	ctrl->offset = 0;
	err = DoIoctl();
	
	return(err);
}

#elif defined(WIN32)

/*
returns:
SDLAR_NOT_REGISTERED (0L)	- card wasn't registered
SDLAR_DEREGISTERED (1L)		- card's instance count
was decremented but isn't currently zero
(i.e. an application is still using the specified board)
SDLAR_RELEASED (2L)			- card is now not used by
any application, and the selector was freed
('cardnumber' is ignored on UNIX-based systems)
*/
int api_base::API_BASE_DeregisterCard(BYTE cardnumber)
{
	CARD_TRANSACTION ct;
	DWORD mresult;
	
	DbgPrint2(DBG_API_BASE,
		"api_base::API_BASE_DeregisterCard() : cardoperation_mutex 0x%X\n",
		cardoperation_mutex
		);
	
	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_DeregisterCard()\n");
	
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED)
	{
		if(mresult == WAIT_TIMEOUT)
		{
			DbgPrint1(DBG_API_BASE,
				"api_base::API_BASE_DeregisterCard():mresult == WAIT_TIMEOUT\n");
		}
		
		if(mresult == WAIT_FAILED)
		{
			DbgPrint1(DBG_API_BASE,
				"api_base::API_BASE_DeregisterCard():mresult == WAIT_FAILED\n");
		}
		
		////////////////////
		{
			LPVOID lpMsgBuf;
			FormatMessage(
				FORMAT_MESSAGE_ALLOCATE_BUFFER |
				FORMAT_MESSAGE_FROM_SYSTEM |
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				GetLastError(),
				0, // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL
				);
			// Process any inserts in lpMsgBuf.
			// ...
			// Display the string.
			DbgPrint1(DBG_API_BASE, (LPCTSTR)lpMsgBuf );
			// Free the buffer.
			LocalFree( lpMsgBuf );
		}
		////////////////////
		return(0);
	}
	
	ct.cardnumber = cardnumber;
    DeviceIoControl(frdrv, IOCTL_SDLA_DEREGISTER, (LPVOID)&ct, sizeof(CARD_TRANSACTION),
		(LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	
	DbgPrint3(DBG_API_BASE, "Card %u deregistration call: return code %02x",
		cardnumber, ct.rcode);
	
	ReleaseMutex(cardoperation_mutex);
	
	return((int)ct.rcode);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

int api_base::API_BASE_PrepareForLoading(BYTE cardnumber)
{
	DbgPrint1(DBG_API_BASE, "int api_base::API_BASE_PrepareForLoading(BYTE cardnumber)\n");
	
	return(SDLAR_PREPARED);
}

#elif defined(WIN32)

/*	Call API_BASE_PrepareForLoading() before loading the card.
If card is busy do not call API_BASE_LoadFirmware() - somebody is using the card.
If not busy, the card will be marked as "Loading in progress"
until API_BASE_LoadComplete() is called.
('cardnumber' is ignored on UNIX-based systems)
*/
int api_base::API_BASE_PrepareForLoading(BYTE cardnumber)
{
	CARD_TRANSACTION ct;
	DWORD mresult;
	
	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_PrepareForLoading()\n");
	
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED)
	{	return(3);
	}
		
	this->cardnumber = cardnumber;
	
	ct.cardnumber = cardnumber;
    DeviceIoControl(frdrv, IOCTL_SDLA_PREPARE_TO_LOAD, (LPVOID)&ct,
		sizeof(CARD_TRANSACTION), (LPVOID)&ct, sizeof(CARD_TRANSACTION),
		(LPDWORD)(&ln), (LPOVERLAPPED)NULL);
	
	DbgPrint3(DBG_API_BASE,
		"Card %u load preparation call: return code %02x\n", cardnumber, ct.rcode);
	
	ReleaseMutex(cardoperation_mutex);
	
	return((int)ct.rcode);
}

#endif

///////////////////////////////////////////////////////////////////////////////////////////

#define GET_FILE_LENGTH(fw_file_ptr, file_len)		\
{												\
	fseek(fw_file_ptr, 0, SEEK_END);			\
	\
	fgetpos(fw_file_ptr, (fpos_t *)&file_len);	\
	\
	fseek(fw_file_ptr, 0L, SEEK_SET );			\
}


///////////////////////////////////////////////////////////////////////////////////////////
/*
Load the firmware and start it. No other calls permitted
while loading is in progress.
('cardnumber' is ignored on UNIX-based systems)
*/
int api_base::API_BASE_LoadFirmware(char * firmware_file, BYTE cardnumber)
{
	FILE * fw_file_ptr = NULL;
	
	// JP 100 op-code for the NMI
	BYTE NMI_code[] = { 0xC3, 0x00, 0x01 };	
	
	UCHAR write_temp, read_temp, err;
	
	ULONG current_write_offset = 0;
	unsigned char * file_in_memory = 0;
	unsigned char * file_in_memory_back = 0;
	unsigned long file_len;
	
	DbgPrint1(DBG_API_BASE, "Fapi::FAPILoadFirmware()\n");
	
	internal_state &= ~FIRMWARE_LOADED;
	
	API_BASE_StopS514(cardnumber);
	//clear card's memory
	API_BASE_ZeroWindow();
	
	DbgPrint2(DBG_API_BASE, "Opening %s file...\n", firmware_file);
	
	if((  fw_file_ptr = fopen(firmware_file, "rb")) == NULL)
	{	DbgPrint2(DBG_API_BASE,
	"Error opening %s file! Check the file exist. Exiting.\n", firmware_file);
	err = API_OPEN_FIRMWARE_FILE_ERR;
	return err;
	}
	
	GET_FILE_LENGTH(fw_file_ptr, file_len);
	
	file_in_memory_back = file_in_memory = (unsigned char *)malloc(file_len);
	if(!file_in_memory)
	{	DbgPrint1(DBG_API_BASE, "Failed to allocate 'file_in_memory'\n");
	return MEMORY_ALLOC_ERR;
	}
	
	//read byte from firmware file, write to allocated buffer
	while(!feof(fw_file_ptr))
	{	*file_in_memory = (unsigned char)fgetc(fw_file_ptr);
	file_in_memory++;
	}
	
	//download the firmware on to the card
	sfm_t*	sfm = (sfm_t*)file_in_memory_back;
	file_in_memory = sfm->image;
	
	DbgPrint2(DBG_API_BASE, "sfm->info.codeoffs : 0x%X\n", sfm->info.codeoffs);
	DbgPrint2(DBG_API_BASE, "sfm->info.codesize : %u\n", sfm->info.codesize);
	
	err = API_FIRMWARE_DOWNLOADED_OK;
	
	current_write_offset = sfm->info.codeoffs;
	
	//read byte from firmware file, write to the card
	for(int ind = 0; ind < sfm->info.codesize; ind++)
	{	write_temp = (unsigned char)sfm->image[ind];
	
		API_BASE_WriteBlock(current_write_offset, 1, &write_temp);
		//read it back for comparison, as additional memory test
		API_BASE_ReadBlock(current_write_offset, 1, &read_temp);
		
		if(write_temp != read_temp)
		{	DbgPrint3(DBG_API_BASE,
					"Firmware on the card is not the same as was downloaded!! (write_temp : 0x%02X read_temp : 0x%02X)\n",
					write_temp,
					read_temp);
		
			err = API_FIRMWARE_COMPARE_FAILED;
			break;
		}
		current_write_offset++;
	}
	
	if(fw_file_ptr)
	{	DbgPrint2(DBG_API_BASE, "Closing %s file.\n", firmware_file);
		//::fclose(fw_file_ptr);//causing crash on Windows !!?? OS will close it anyway.
	}
	
	if(file_in_memory_back){
		free(file_in_memory_back);
	}
	
	if(err == API_FIRMWARE_DOWNLOADED_OK){
		
		// move the NMI code (JP 0x100) onto the card
		API_BASE_WriteBlock(0x0000, 3, NMI_code);
		API_BASE_WriteBlock(0x0066, 3, NMI_code);
		
		API_BASE_StartS514(cardnumber);
		
		//wait 1 second for code initialization
		my_sleep(1000);
	}
	
	internal_state |= FIRMWARE_LOADED;
	
	return err;
}

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

int api_base::API_BASE_LoadComplete(BYTE cardnumber)
{
	DbgPrint1(DBG_API_BASE, "int api_base::API_BASE_LoadComplete(BYTE cardnumber)\n");
	
	return(1);
}

#elif defined(WIN32)

//	Card will no longer be marked as "Loading in progress".
int api_base::API_BASE_LoadComplete(BYTE cardnumber)
{
	CARD_TRANSACTION ct;
	DWORD mresult;
	
	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_LoadComplete()\n");
	
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED)
		return(0);
	
	
	ct.cardnumber = cardnumber;
    DeviceIoControl(frdrv, IOCTL_SDLA_LOAD_COMPLETE, (LPVOID)&ct,
		sizeof(CARD_TRANSACTION), (LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln),
		(LPOVERLAPPED)NULL);
	
	DbgPrint3(DBG_API_BASE,
		"Card %u load complete call: return code %02x\n", cardnumber, ct.rcode);
	
	ReleaseMutex(cardoperation_mutex);
	
	return((int)ct.rcode);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

unsigned char api_base::API_BASE_ReadBlock(ULONG offset, ULONG length,
										   void * destination_ptr)
{
	//DbgPrint1(DBG_API_BASE, "unsigned char api_base::API_BASE_ReadBlock()\n");
	
	//restore data ptr
	data = data_bakup;
	
	ctrl->ioctl = IOCTL_READ_BLOCK;
	ctrl->buffer_length = length;
	ctrl->offset = offset;
	DoIoctl();
	
	memcpy(destination_ptr, data, length);
	
	//return only the first byte for reading one byte
	return((unsigned char)(*data));
}

#elif defined(WIN32)

unsigned char api_base::API_BASE_ReadBlock(ULONG offset, ULONG length,
										   void * destination_ptr)
{
	SDLA_BLOCKOP memblock;
	
	
	memblock.card_number	= cardnumber;
    memblock.offset			= offset;
	memblock.num_bytes		= length;
	memblock.buffer			= destination_ptr;
	
	DeviceIoControl(frdrv,
		IOCTL_SDLA_READ_BLOCK,
		(LPVOID)&memblock,
		sizeof(SDLA_BLOCKOP),
		(LPVOID)&memblock,
		sizeof(SDLA_BLOCKOP),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
	
	//return only the first byte for reading one byte
	return(*(UCHAR *)destination_ptr);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)


void api_base::API_BASE_WriteBlock(ULONG offset, ULONG length, void * source_ptr)
{
	//	DbgPrint1(DBG_API_BASE, "void api_base::API_BASE_WriteBlock()\n");
	
	//restore data ptr
	data = data_bakup;
	
	memcpy(data, source_ptr, length);
	
	ctrl->ioctl = IOCTL_WRITE_BLOCK;
	ctrl->buffer_length = length;
	ctrl->offset = offset;
	DoIoctl();
	
	return;
}

int api_base::DoIoctl()
{
	int ioctl_rc = ioctl(dev, WANPIPE_EXEC, exec );
	
	if ((dev < 0) || ioctl_rc < 0)
		return IOCTL_ERR;
	else
		return 0;
	
}

#elif defined(WIN32)

void api_base::API_BASE_WriteBlock(ULONG offset, ULONG length, void * source_ptr)
{
	SDLA_BLOCKOP memblock;
	
	memblock.card_number	= cardnumber;
    memblock.offset			= offset;
	memblock.num_bytes		= length;
	memblock.buffer			= source_ptr;
	
	DeviceIoControl(frdrv,
		IOCTL_SDLA_WRITE_BLOCK,
		(LPVOID)&memblock,
		sizeof(SDLA_BLOCKOP),
		(LPVOID)&memblock,
		sizeof(SDLA_BLOCKOP),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

void api_base::API_BASE_ZeroWindow()
{
	DbgPrint1(DBG_API_BASE, "void api_base::API_BASE_ZeroWindow()\n");
	unsigned long offset;
	unsigned char write_temp = 0;
	
	for(offset = 0; offset < MAX_SIZEOF_S514_MEMORY; offset++){
		API_BASE_WriteBlock(offset, 1, &write_temp);
	}
}

#elif defined(WIN32)

//zero adapter's memory. used before downloading the firmware.
void api_base::API_BASE_ZeroWindow()
{
	CARD_TRANSACTION ct;
	
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_ZeroWindow()\n");
	
	ct.cardnumber = cardnumber;
	DeviceIoControl(frdrv, IOCTL_SDLA_ZERO_WINDOW, (LPVOID)&ct, sizeof(CARD_TRANSACTION),
		(LPVOID)&ct, sizeof(CARD_TRANSACTION), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

//
// returns 0 if timed-out, 1 if ok
//
char api_base::SetWaitOppFlag(WORD offset)
{
	DbgPrint1(DBG_API_BASE, "char api_base::SetWaitOppFlag(WORD offset)\n");
	
	DWORD count = 0;
	USHORT value = 1;
	
	//byte 0 of data buffer will contain 1
	*data = 0x01;//this will set opp flag on the card to 1
	API_BASE_WriteBlock(offset, 1, data);
	
	// loop until opp flag is zeroed or timeout
	do
	{
		my_sleep(DELAY_TIME);// 10000 - sleep for 1/200 of a second
		
		//maximum 1000 times
		if (count++ > MAX_RETRY_COUNT)
		{	return(0);//timeout
		}
		
		value = API_BASE_ReadBlock(offset, 1, data);
	} while (value);
	
	return(1);
}

#elif defined(WIN32)

// runs inside fapicontrol_mutex
char api_base::SetWaitOppFlag(WORD offset)
/* returns 0 if timed-out, 1 if ok */
{
	DWORD t1 = GetTickCount();
	UCHAR opp_flag = 1;
	
	/* set opp_flag to 1 */
	API_BASE_WriteBlock(offset, 1, &opp_flag);
	
	/* loop until opp_flag is reset to 0, or until timeout */
	do {
		if (GetTickCount() > (t1+timeout_ticks))
		{	DbgPrint2(DBG_API_BASE,
		"Card %u is not responding to mailbox command!\n", cardnumber);
		return(0);
		}
		
		my_sleep(10);
		
		API_BASE_ReadBlock(offset, 1, &opp_flag);
		
	}while (opp_flag);
	
	return(1);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

void api_base::API_BASE_StartS514(BYTE cardnumber)
{
	ctrl->ioctl = IOCTL_START_CARD;
	ctrl->buffer_length = 0x00;
	ctrl->offset = 0x00;
	DoIoctl();
}

void api_base::API_BASE_StopS514(BYTE cardnumber)
{
	ctrl->ioctl = IOCTL_STOP_CARD;
	ctrl->buffer_length = 0x00;
	ctrl->offset = 0x00;
	DoIoctl();
}

#elif defined(WIN32)

void api_base::API_BASE_StartS514(BYTE cardnumber)
{
	XFER_BYTE_CONTROL xbc;
	
	xbc.cardnumber = cardnumber;
	xbc.offset = 0;//offset;
	xbc.value = 0;//data;
	DeviceIoControl(frdrv, IOCTL_START_S514, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL),
		(LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
}

void api_base::API_BASE_StopS514(BYTE cardnumber)
{
	XFER_BYTE_CONTROL xbc;
	
	xbc.cardnumber = cardnumber;
	xbc.offset = 0;//offset;
	xbc.value = 0;//data;
	DeviceIoControl(frdrv, IOCTL_STOP_S514, (LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL),
		(LPVOID)&xbc, sizeof(XFER_BYTE_CONTROL), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

int api_base::my_sleep(int usecs)
{
	timeval tv;
	
	tv.tv_sec = 0;
	tv.tv_usec = usecs;
	
	return select(0, NULL, NULL, NULL, &tv);
}

#elif defined(WIN32)


//delays execution of calling thread for time 'milliseconds'
int api_base::my_sleep(int milliseconds)
{
	int rc;
	
	HANDLE hHandle;
	
	hHandle = CreateEvent( NULL, FALSE, FALSE, "atm_api_wait_event");
	rc = WaitForSingleObject( hHandle,  milliseconds);
	
	return rc;
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////

#if defined(BUILD_FOR_UNIX)

int api_base::API_BASE_GetCards(void)
{
	DbgPrint1(DBG_API_BASE, "int api_base::API_BASE_GetCards(void)\n");
	
	return(1);
}

#elif defined(WIN32)

/*
returns a bit record (32-bit) of cards registered for use
and cards that exist:
[ E15 E14 ... E3 E2 E1 E0 R15 R14 ... R3 R2 R1 R0 ]
For example, if bit R2 is set, then card 2 is registered
if bit E2 is set, then card 2 exists
(i.e. a configuration entry exists
in SDLA.INI)
*/
int api_base::API_BASE_GetCards(void)
{
	LRESULT buffer;
	DWORD mresult;
	
	buffer = 0L;
	mresult = WaitForSingleObject(cardoperation_mutex, timeout_ticks);
	
	DbgPrint1(DBG_API_BASE, "api_base::API_BASE_GetCards()\n");
	
	if (mresult != WAIT_TIMEOUT && mresult != WAIT_FAILED) {
		
		DeviceIoControl(frdrv, IOCTL_SDLA_GET_CARDS, (LPVOID)&buffer, sizeof(LRESULT),
			(LPVOID)&buffer, sizeof(LRESULT), (LPDWORD)(&ln), (LPOVERLAPPED)NULL);
		
		ReleaseMutex(cardoperation_mutex);
	}
	return(buffer);
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////
