// atm_api.cpp: implementation of the atm_api class.
//
////////////////////////////////////////////////////////////////////////////////////

#include "atm_api.h"

#define DBG_ATM_API			0
#define DBG_ATM_API_TX		0
#define DBG_ATM_API_RX		0
#define DBG_ATM_LINE_TRACE	0

////////////////////////////////////////////////////////////////////////////////////
atm_api::atm_api()
{
	DbgPrint1(DBG_ATM_API, "atm_api::atm_api()\n");

	ready_to_recieve = false;
	ready_to_transmit = false;
	ready_to_read_trace_data = false;
}

////////////////////////////////////////////////////////////////////////////////////

atm_api::~atm_api()
{
	DbgPrint1(DBG_ATM_API, "atm_api::~atm_api()\n");

	API_BASE_Cleanup();
}

////////////////////////////////////////////////////////////////////////////////////

/*	Perform a control call with control block as input
	returns return_code value in structure.
	('cardnumber' is ignored on UNIX-based systems)
*/
char atm_api::ATMAPIControl(ATM_MAILBOX_STRUCT * block, BYTE cardnumber)
{
  if(!(internal_state & CONNECTED_TO_DRIVER) || !(internal_state & FIRMWARE_LOADED))
  {
    DbgPrint1(DBG_ATM_API, "atm_api::ATMAPIControl() : not connected!!!\n");
    block->return_code = Z80_TIMEOUT_ERROR;
    return block->return_code;
  }

	InternalDoControl(block);
	return block->return_code;
}

////////////////////////////////////////////////////////////////////////////////////
	
/** prototypes of functions which should not be used directly by an application **/

char atm_api::InternalDoControl(ATM_MAILBOX_STRUCT * block)
{
	char flag;
	DWORD mresult;

//	DbgPrint2(DBG_ATM_API, "InternalDoControl():cardnumber :            0x%x.\n", cardnumber);
//	DbgPrint2(DBG_ATM_API, "InternalDoControl():block->command			0x%x.\n", block->command);

//	DbgPrint2(DBG_ATM_API, "InternalDoControl():block->buffer_length	0x%x.\n", block->buffer_length);

#if defined(BUILD_FOR_UNIX)

#elif defined(WIN32)
	// start system-wide critical section
	// release mutex in SayReturnResult!!!
	mresult = WaitForSingleObject(api_base_control_mutex, timeout_ticks);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED) {
		block->return_code = Z80_TIMEOUT_ERROR;
		return block->return_code;
	}
#endif

	/* reject block if buffer_length is too high, no matter what */
  if (block->buffer_length > ATM_MDATALEN)
	{	block->return_code = ATM_DATA_LENGTH_TOO_BIG;

		// someone else can do a mail box call now
#if defined(BUILD_FOR_UNIX)

#elif defined(WIN32)
    ReleaseMutex(api_base_control_mutex);
#endif
		return block->return_code;
	}

	/* submit to appropriate mailbox via driver */
	API_BASE_WriteBlock(	BASE_ADDR_MB_STRUCT,				//offset on the card
						block->buffer_length + ATM_CB_HEADERLEN,//length
						block								//source_ptr
						);


	/* submitted to mailbox... now set flag and wait to clear */
	flag = SetWaitOppFlag(BASE_ADDR_MB_STRUCT);

	if (!flag)

		block->return_code = Z80_TIMEOUT_ERROR;
	else{
		/* give application a copy of the mailbox after the transaction */

		//first read the mail box
		API_BASE_ReadBlock(	BASE_ADDR_MB_STRUCT,	//offset on the card
							ATM_CB_HEADERLEN,			//length
							block					//destination_ptr
							);

		//if some data in was returned, read it too
		if(block->buffer_length)
		{
			API_BASE_ReadBlock(	BASE_ADDR_MB_STRUCT + ATM_CB_HEADERLEN,	//offset on the card
								block->buffer_length,				//length
								block->data							//destination_ptr
								);
		}
	}

	DbgPrint2(DBG_ATM_API, "InternalDoControl():block->return_code	0x%x.\n",  
		block->return_code);

#if defined(BUILD_FOR_UNIX)

#elif defined(WIN32)
	// someone else can do a mail box call now
  ReleaseMutex(api_base_control_mutex);
#endif

	//on some commands additional work should be done
	switch(block->command)
	{
	case ATMAL_SET_CONFIGURATION:
		if(block->return_code == 0)
		{	ATMAPIReadTxConfiguration();
			ATMAPIReadRxConfiguration();
		}

		break;

	}

	return block->return_code;
}

////////////////////////////////////////////////////////////////////////////////////

/*	Call ATMAPIReadTxConfiguration() only once,
	before transmitting a frame or a number of frames.
*/
int atm_api::ATMAPIReadTxConfiguration()
{
	ATMAL_CONFIGURATION_STRUCT * cfg_ptr = (ATMAL_CONFIGURATION_STRUCT *)cb.data;

	cb.command = ATMAL_READ_CONFIGURATION;
	cb.buffer_length = sizeof(ATMAL_CONFIGURATION_STRUCT);
	cb.return_code = Z80_TIMEOUT_ERROR;

	ATMAPIControl(&cb, cardnumber);

	if(!cb.return_code)
	{	ptr_AAL_Tx_control_struct = cfg_ptr->ptr_AAL_Tx_control_struct;
		ready_to_transmit = true;

#if DBG_ATM_API_TX

		API_BASE_ReadBlock(	ptr_AAL_Tx_control_struct,		//offset on the card
							sizeof(aal_tx_control_struct),	//length

							&aal_tx_control_struct			//destination_ptr
							);

		DbgPrint2(DBG_ATM_API_TX, "number_Tx_status_els %d.\n",  
			aal_tx_control_struct.number_Tx_status_els
			);

		DbgPrint2(DBG_ATM_API_TX, "opp_flag 0x%x.\n",  
			aal_tx_control_struct.opp_flag
			);

		DbgPrint2(DBG_ATM_API_TX, "ptr_Tx_status_el 0x%x.\n",  
			aal_tx_control_struct.ptr_Tx_status_el
			);
#endif //DBG_ATM_API_TX

	}else
	{	ptr_AAL_Tx_control_struct = 0;
		ready_to_transmit = false;
	}

	return cb.return_code;
}

////////////////////////////////////////////////////////////////////////////////////

int atm_api::ATMAPITransmitFrame(	AAL_TX_STATUS_EL_STRUCT * p_tx_el,
									BYTE cardnumber
									)
{
	AAL_TX_STATUS_EL_STRUCT tx_el;
	unsigned char opp_flag;

	if(ready_to_transmit == false)
	{	
		DbgPrint1(DBG_ATM_API_TX, 
			"ATMAPITransmitFrame(): ATMAL_CONFIGURATION_NOT_SET !!\n");
		return ATMAL_CONFIGURATION_NOT_SET;
	}

	if(	p_tx_el->user_info_length < AAL_MIN_LENGTH_TX_USER_INFO ||
		p_tx_el->user_info_length > AAL_MAX_LENGTH_TX_USER_INFO)
	{
		DbgPrint2(DBG_ATM_API_TX, "p_tx_el->user_info_length %d.\n",  
			p_tx_el->user_info_length
			);

		return ATMAL_TX_INVALID_DATA_LENGTH;
	}

	API_BASE_ReadBlock(	ptr_AAL_Tx_control_struct,		//offset on the card
						sizeof(aal_tx_control_struct),	//length
						&aal_tx_control_struct			//destination_ptr
						);

	if(!aal_tx_control_struct.opp_flag)
	{
		aal_tx_control_struct.opp_flag = ATMAL_TX_OK;

		//read current tx element from the card
		API_BASE_ReadBlock(aal_tx_control_struct.ptr_Tx_status_el,	//offset on the card
						sizeof(tx_el),							//length
						&tx_el									//destination_ptr
						);

		//update it with user data
		tx_el.user_info_length = p_tx_el->user_info_length;
//		tx_el.misc_Tx_bits = p_tx_el->misc_Tx_bits;
		tx_el.GFC_VPI = p_tx_el->GFC_VPI;
		tx_el.VCI = p_tx_el->VCI;
//		tx_el.PT = p_tx_el->PT;
//		tx_el.CLP = p_tx_el->CLP;

		//write it back to the card
		API_BASE_WriteBlock(
			aal_tx_control_struct.ptr_Tx_status_el,	//offset on the card
			sizeof(tx_el),							//length
			&tx_el									//source ptr
			);

		//write user data
		API_BASE_WriteBlock(
			tx_el.ptr_data_bfr,			//offset on the card
			p_tx_el->user_info_length,	//length
			(void *)p_tx_el->ptr_data_bfr		//source ptr
			);

		//set opp flag to start transmission
		opp_flag = 1;
		API_BASE_WriteBlock(
			ptr_AAL_Tx_control_struct + ATM_TX_OPP_FLAG_OFFSET,	//offset on the card
			sizeof(opp_flag),	//length
			&opp_flag			//source ptr
			);

	}else
	{
		aal_tx_control_struct.opp_flag = ATMAL_TRANSMITTER_BUSY;
	}

	return aal_tx_control_struct.opp_flag;
}

////////////////////////////////////////////////////////////////////////////////////

int atm_api::ATMAPISetTraceConfiguration(unsigned char config)
{
	LINE_TRACE_CONFIG_STRUCT * trace_config_struct =
		(LINE_TRACE_CONFIG_STRUCT *)cb.data;

	trace_config_struct->trace_config = config;

	cb.command = SET_TRACE_CONFIGURATION;
	cb.buffer_length = sizeof(LINE_TRACE_CONFIG_STRUCT);
	cb.return_code = Z80_TIMEOUT_ERROR;

	ATMAPIControl(&cb, cardnumber);

	return cb.return_code;
}

////////////////////////////////////////////////////////////////////////////////////

int atm_api::ATMAPIReadTraceConfiguration()
{
	LINE_TRACE_CONFIG_STRUCT * ptr_trace_config_struct;
	TRACE_STATUS_EL_CFG_STRUCT line_trace_info_struct;

	cb.command = READ_TRACE_CONFIGURATION;
	cb.buffer_length = 0;
	cb.return_code = Z80_TIMEOUT_ERROR;

	ATMAPIControl(&cb, cardnumber);
	
	if(cb.return_code)
	{	ready_to_read_trace_data = false;
		return cb.return_code;
	}
	
	ptr_trace_config_struct = (LINE_TRACE_CONFIG_STRUCT * )cb.data;

	DbgPrint3(DBG_ATM_LINE_TRACE,
		"trace_config = 0x%02X, ptr_trace_stat_el_cfg_struct = 0x%X",
		ptr_trace_config_struct->trace_config,
		ptr_trace_config_struct->ptr_trace_stat_el_cfg_struct
		);

	API_BASE_ReadBlock(
		(unsigned long)ptr_trace_config_struct->ptr_trace_stat_el_cfg_struct,
		sizeof(TRACE_STATUS_EL_CFG_STRUCT),
		&line_trace_info_struct
		);
	
	// read the trace status element configuration information
	number_trc_stat_els = line_trace_info_struct.number_trace_status_els;

	base_addr_trc_stat_el_list = 
		(TRACE_STATUS_ELEMENT_STRUCT *)line_trace_info_struct.base_addr_trace_status_els;
	ptr_next_trc_stat_el_to_use = 
		(TRACE_STATUS_ELEMENT_STRUCT *)line_trace_info_struct.next_trace_el_to_use;
	
	// calculate a pointer to the last element in the trace status element list
	end_addr_trc_stat_el_list =
		(TRACE_STATUS_ELEMENT_STRUCT *)((unsigned long)base_addr_trc_stat_el_list + 
			((number_trc_stat_els - 1) * sizeof(TRACE_STATUS_ELEMENT_STRUCT)));
	
	DbgPrint2(DBG_ATM_LINE_TRACE,
		"\nNumber of trace status elements allocated          : %u",
		number_trc_stat_els);

	DbgPrint2(DBG_ATM_LINE_TRACE,
		"\nBase address of the trace status element list      : 0x%lX",
		base_addr_trc_stat_el_list);

	DbgPrint2(DBG_ATM_LINE_TRACE,
		"\nEnd address of the trace status element list       : 0x%lX",
		end_addr_trc_stat_el_list);

	DbgPrint2(DBG_ATM_LINE_TRACE,
		"\nPointer to the next trace status element to be used: 0x%lX",
		ptr_next_trc_stat_el_to_use);

	ready_to_read_trace_data = true;

	return 0;
}

////////////////////////////////////////////////////////////////////////////////////

int atm_api::ATMAPIReadTraceData(TRACE_STATUS_ELEMENT_STRUCT * trace_element_struct,
								 unsigned char * trc_data_bfr
								 )
{
	bool rc;

	if(	ready_to_read_trace_data == false)
	{	DbgPrint1(DBG_ATM_LINE_TRACE, "ready_to_read_trace_data == false\n");
		rc = false;
		return rc;
	}

	API_BASE_ReadBlock(
		(unsigned long)ptr_next_trc_stat_el_to_use,
		sizeof(TRACE_STATUS_ELEMENT_STRUCT),
		trace_element_struct
		);

	if(trace_element_struct->opp_flag != 0x01)

	{	//there is nothing in this trace buffer
		//DbgPrint1(DBG_ATM_LINE_TRACE, "trace_element_struct.opp_flag != 0x01\n",);
		rc = false;
		return rc;
	}

	/* read the data if it was stored in the trace buffer */
	if(trace_element_struct->ptr_data_bfr) 
	{
		if(trace_element_struct->trace_length)
		{
			API_BASE_ReadBlock(
				trace_element_struct->ptr_data_bfr,
				trace_element_struct->trace_length,
				trc_data_bfr
				);
		}
		rc = true;
	}else
	{
		rc = false;
	}

	/* indicate to the adapter that we have processed this incomming data buffer */
	trace_element_struct->opp_flag = 0x00;

	API_BASE_WriteBlock(
		(unsigned long)ptr_next_trc_stat_el_to_use,
		sizeof(TRACE_STATUS_ELEMENT_STRUCT),
		trace_element_struct
		);

	/* point to the next sequential trace status element to be processed */ 
	ptr_next_trc_stat_el_to_use =
		(ptr_next_trc_stat_el_to_use == end_addr_trc_stat_el_list) ?
			base_addr_trc_stat_el_list :
			(++ ptr_next_trc_stat_el_to_use);

	return rc;
}

//////////////////////////////////////////////////////////////////////////////////////////////

int atm_api::ATMAPIReadRxConfiguration()
{
	ATMAL_CONFIGURATION_STRUCT * cfg_ptr = (ATMAL_CONFIGURATION_STRUCT *)cb.data;

	cb.command = ATMAL_READ_CONFIGURATION;
	cb.buffer_length = sizeof(ATMAL_CONFIGURATION_STRUCT);
	cb.return_code = Z80_TIMEOUT_ERROR;

	ATMAPIControl(&cb, cardnumber);

	if(!cb.return_code)
	{	ptr_AAL_Rx_control_struct = cfg_ptr->ptr_AAL_Rx_control_struct;
		ready_to_recieve = true;

#if DBG_ATM_API_RX

		API_BASE_ReadBlock(	ptr_AAL_Rx_control_struct,		//offset on the card
							sizeof(aal_rx_control_struct),	//length
							&aal_rx_control_struct			//destination_ptr
							);

		DbgPrint2(DBG_ATM_API_RX, "number_Rx_status_els %d.\n",  
			aal_rx_control_struct.number_Rx_status_els
			);

		DbgPrint2(DBG_ATM_API_RX, "opp_flag 0x%x.\n",  
			aal_rx_control_struct.opp_flag
			);

		DbgPrint2(DBG_ATM_API_RX, "ptr_Rx_status_el 0x%x.\n",  
			aal_rx_control_struct.ptr_Rx_status_el
			);
#endif //DBG_ATM_API_RX

	}else
	{	ptr_AAL_Rx_control_struct = 0;
		ready_to_recieve = false;
	}

	return cb.return_code;
}

//////////////////////////////////////////////////////////////////////////////////////////////

int atm_api::ATMAPIReadData(	AAL_RX_STATUS_EL_STRUCT * p_rx_el,
								BYTE cardnumber
								)
{
	unsigned char opp_flag;
	unsigned long ptr_user_data_buffer = p_rx_el->ptr_data_bfr;

	if(ready_to_recieve == false)
	{	
		DbgPrint1(DBG_ATM_API_RX, 
			"ATMAPITransmitFrame(): ATMAL_CONFIGURATION_NOT_SET !!\n");
		return ATMAL_CONFIGURATION_NOT_SET;
	}

	API_BASE_ReadBlock(	ptr_AAL_Rx_control_struct,		//offset on the card
						sizeof(aal_rx_control_struct),	//length
						&aal_rx_control_struct			//destination_ptr
						);

	if(aal_rx_control_struct.opp_flag)
	{
		//read current rx element from the card
		API_BASE_ReadBlock(aal_rx_control_struct.ptr_Rx_status_el,	//offset on the card
						sizeof(AAL_RX_STATUS_EL_STRUCT),		//length
						p_rx_el									//destination_ptr
						);

		if(	p_rx_el->user_info_length < AAL_MIN_LENGTH_RX_USER_INFO ||
			p_rx_el->user_info_length > AAL_MAX_LENGTH_RX_USER_INFO)
		{

			DbgPrint2(DBG_ATM_API_RX, "p_rx_el->user_info_length %d.\n",  
				p_rx_el->user_info_length
				);

			return ATMAL_RX_INVALID_DATA_LENGTH;
		}

		//read user data into user's buffer
		API_BASE_ReadBlock(
			p_rx_el->ptr_data_bfr,			//offset on the card
			p_rx_el->user_info_length,		//length
			(void *)ptr_user_data_buffer	//destination ptr
			);

		//note that original user's rx element was overwritten,
		//restore the pointer to data area
		p_rx_el->ptr_data_bfr = ptr_user_data_buffer;



		//reset opp flag to realease the element
		opp_flag = 0;
		API_BASE_WriteBlock(
			ptr_AAL_Rx_control_struct + ATM_RX_OPP_FLAG_OFFSET,	//offset on the card
			sizeof(opp_flag),	//length
			&opp_flag			//source ptr
			);

		aal_tx_control_struct.opp_flag = 0;
	}
	else
	{
		aal_tx_control_struct.opp_flag = ATMAL_NO_RX_DATA_AVAILABLE;
	}

	return aal_tx_control_struct.opp_flag;
}

//////////////////////////////////////////////////////////////////////////////////////////////
