// atm_api.h: interface for the atm_api class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(ATM_API_H)
#define ATM_API_H

#include "api_base.h"

#include "phyapi.h"
#include "atmalapi.h"

////////////////////////////////////////////////////////////////////////

#define	ATM_MDATALEN		SIZEOF_MB_DATA_BFR
#define	ATM_CB_HEADERLEN	16

#define ATM_DATA_LENGTH_TOO_BIG		0x05

#define MIN_VPI_NUMBER 0
#define MAX_VPI_NUMBER 255

#define MIN_VCI_NUMBER 0
#define MAX_VCI_NUMBER 65535

#define ATMAL_TX_OK						        0
#define ATMAL_TRANSMITTER_BUSY			  2
#define ATMAL_CONFIGURATION_NOT_SET	  3
#define ATMAL_TX_INVALID_DATA_LENGTH	4
#define ATMAL_RX_INVALID_DATA_LENGTH	ATMAL_TX_INVALID_DATA_LENGTH
#define ATMAL_NO_RX_DATA_AVAILABLE		5

#define ATM_TX_OPP_FLAG_OFFSET	16
#define ATM_RX_OPP_FLAG_OFFSET	16

//types of ATM cells
#define ATM_UNKNOWN_CELL					0
#define ATM_IDLE_CELL							1

#define ATM_PHYSICAL_LAYER_OAM_CELL				    2
#define ATM_RESERVED_FOR_PHYSICAL_LAYER_CELL	3
#define ATM_UNASSIGNED_CELL						        4
#define ATM_USER_DATA_CELL						        5
#define ATM_OAM_F5_CELL							          6

class atm_api : public api_base
{
public:
	atm_api();
	~atm_api();

  char ATMAPIControl(ATM_MAILBOX_STRUCT * block, BYTE cardnumber);	

	int ATMAPITransmitFrame(AAL_TX_STATUS_EL_STRUCT * tx_el,
							BYTE cardnumber
							);

	int ATMAPIReadData(	AAL_RX_STATUS_EL_STRUCT * p_rx_el,
						  BYTE cardnumber
						  );

	int ATMAPISetTraceConfiguration(unsigned char config);
	int ATMAPIReadTraceConfiguration();
	int ATMAPIReadTraceData(TRACE_STATUS_ELEMENT_STRUCT * trace_element_struct,
							unsigned char * trc_data_bfr
							);

	ATM_MAILBOX_STRUCT atm_cb;

private:

	ATM_MAILBOX_STRUCT cb;

	/** prototypes of functions which SHOULD NOT be used directly by an application **/

	char InternalDoControl(ATM_MAILBOX_STRUCT * block);

	/*	Call ATMAPIReadTxConfiguration() each time, the AAL configuration
		is successfully set. It will read Tx configuration.
	*/
	int ATMAPIReadTxConfiguration();
	int ATMAPIReadRxConfiguration();
	
	//transmission related variables
	unsigned long ptr_AAL_Tx_control_struct;
	AAL_TX_CONTROL_STRUCT aal_tx_control_struct;
	//The "ATMAPITransmitFrame()" command is NOT allowed
	//if 'ready_to_transmit' is set to 'false'.
	bool ready_to_transmit;

	//reception related variables
	unsigned long ptr_AAL_Rx_control_struct;
	AAL_RX_CONTROL_STRUCT aal_rx_control_struct;
	//The "ATMAPIReadData()" command is NOT allowed
	//if 'ready_to_recieve' is set to 'false'.
	bool ready_to_recieve;

	//trace related variables
	unsigned short number_trc_stat_els;
	TRACE_STATUS_ELEMENT_STRUCT * base_addr_trc_stat_el_list;
	TRACE_STATUS_ELEMENT_STRUCT * end_addr_trc_stat_el_list;
	TRACE_STATUS_ELEMENT_STRUCT * ptr_next_trc_stat_el_to_use;

	//The "ATMAPIReadTraceData()" command is NOT allowed
	//if 'ready_to_read_trace_data' is set to 'false'.
	bool ready_to_read_trace_data;

};

#endif // !defined(ATM_API_H)
