// atm_line_trace_impl.cpp: implementation of the atm_line_trace_impl class.
//
//////////////////////////////////////////////////////////////////////

#include "MainWizardImpl.h"

extern TesterWizardImpl * globalTesterWizPtr;

#include "atmutils.h"

#include "oam.h"	//for decoding OAM cells in the trace

typedef struct{

	unsigned char UU;
	unsigned char CPI;
	unsigned short length;
	unsigned long CRC;

}AAL5_CPCS_PDU_TRAILER;

#define DBG_ATM_LINE_TRACE 0

//Common columns
#define DIR_COL			0
#define LEN_COL			1
#define TIME_COL		2

#define CELL_TYPE_COL	3
#define GFC_COL			4
#define VPI_COL			5
#define VCI_COL			6
#define PT_COL			7
#define CLP_COL			8
#define HEC_COL			9

//AAL level trace columns
#define AAL_DATA_COL_RAW			3
#define AAL_DATA_COL_INTERPRETED	11

#define AAL_OAM_TYPE_COL		9
#define AAL_OAM_FUNC_TYPE_COL	10

//PHY level trace columns
#define PHY_DATA_COL_RAW			3
#define PHY_DATA_COL_INTERPRETED	10

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

atm_line_trace_impl::atm_line_trace_impl()
{
	window_closed = true;
	set_widget_caption(this);

	widget_caption = caption();

	trace_to_file_flag = false;
	
	show_interpreted = TRUE;

	on_ShowInterpretedTrace();

	show_atmal_cell = false;
	show_phy_cell = false;
}

atm_line_trace_impl::~atm_line_trace_impl()
{
	if(trace_to_file_flag == true)
	{	Stop_Trace_To_File();
	}
}

void atm_line_trace_impl::on_Close_Trace_Window()
{
	close();	
}

void atm_line_trace_impl::showEvent(QShowEvent * ev)
{
	set_widget_position_on_screen(this);
}

bool atm_line_trace_impl::initialize()
{
	DbgPrint1(DBG_ATM_LINE_TRACE, "atm_line_trace_impl::initialize()\n");

	window_closed = false;

	ListView_AAL_Level->clear();
	ListView_PHY_Level->clear();

	show_interpreted = TRUE;

	chk_PHY_StopAutoScroll->setChecked(FALSE);
	checkBox_AAL_StopAutoScroll->setChecked(FALSE);

	checkBox_Show_User_Data_cells->setChecked(true);
	checkBox_Show_OAM_F5_cells->setChecked(true);

	checkBox_show_idle_cells->setChecked(true);
	checkBox_show_phy_layer_cells->setChecked(true);
	checkBox_show_unassigned_cells->setChecked(true);

	if(globalTesterWizPtr->trace_file == NULL)
	{
		IDC_STOP_TO_FILE->setEnabled(false);
		START_TO_FILE->setEnabled(false);
	}else
	{
		IDC_STOP_TO_FILE->setEnabled(false);
		START_TO_FILE->setEnabled(true);
	}

	//disable so file name will not be changed while we are here
	//globalTesterWizPtr->atm_phy_level_trace_menu->
	//	setItemEnabled (globalTesterWizPtr->atm_phy_level_trace_menu->idAt(1), FALSE);

	return (BOOL)EnableTracing();
}

void atm_line_trace_impl::closeEvent(QCloseEvent * e)
{
	window_closed = true;

	DisableTracing();

	globalTesterWizPtr->atm_phy_level_trace_menu->
		setItemChecked (globalTesterWizPtr->atm_phy_level_trace_menu->idAt(0), FALSE );
	
	//enable so file name can be changed while we are NOT here
	//globalTesterWizPtr->atm_phy_level_trace_menu->
	//	setItemEnabled (globalTesterWizPtr->atm_phy_level_trace_menu->idAt(1), TRUE );

	e->accept();
}

//this is over ride of QObject's timer function
void atm_line_trace_impl::timerEvent(QTimerEvent * e)
{
	int num_of_traces_read;

	if(window_closed == true)
	{	return;
	}

	num_of_traces_read = 0;
	while(read_trace_data() == (int)true && num_of_traces_read++ < 3)
		;
}

void atm_line_trace_impl::on_Flush_Trace_Window()
{
	ListView_PHY_Level->clear();
	ListView_AAL_Level->clear();
}

void atm_line_trace_impl::on_Start_Trace_To_File()
{
	IDC_STOP_TO_FILE->setEnabled(true);
	START_TO_FILE->setEnabled(false);

	trace_to_file_flag = true;

	QTime new_time = QTime::currentTime();      // get the current time
	QString str_new_time = new_time.toString ( QString("hh:mm:ss.zzz") );

	qstr_trace_file_data = "****** Trace to file started at: " + str_new_time + ".\n\n";

	fprintf	(globalTesterWizPtr->trace_file,(const char *)qstr_trace_file_data);
	fflush	(globalTesterWizPtr->trace_file);
}

void atm_line_trace_impl::Stop_Trace_To_File()
{
	IDC_STOP_TO_FILE->setEnabled(false);
	START_TO_FILE->setEnabled(true);

	trace_to_file_flag = false;

	QTime new_time = QTime::currentTime();      // get the current time
	QString str_new_time = new_time.toString ( QString("hh:mm:ss.zzz") );

	qstr_trace_file_data = "****** Trace to file stopped at: " + str_new_time + ".\n\n";

	fprintf	(globalTesterWizPtr->trace_file,(const char *)qstr_trace_file_data);
	fflush	(globalTesterWizPtr->trace_file);

}

bool atm_line_trace_impl::EnableTracing(void)
{
  int rc = set_trc_config(
		        TRACE_ACTIVE | TRACE_PHY_UNASSIGNED_CELLS | TRACE_PHY_IDLE_CELLS |
		        TRACE_PHY_PHYS_LAYER_CELLS |
		        TRACE_PHY_NON_UNAS_PHYS_CELLS); // | TRACE_LIMIT_REPEAT_CELLS);

	if(rc)
	{	
		qstr_err_message.sprintf(
			"ATMAPISetTraceConfiguration() failed!!. Return code 0x%02X",
			rc
			);
		QMessageBox::warning(this, "Tracing Error", qstr_err_message );
		return false;
	}

	return read_trace_config();
}

BOOL atm_line_trace_impl::DisableTracing(void)
{
	return (BOOL)set_trc_config(0x00);
}

void atm_line_trace_impl::on_ShowRawTrace()
{
	show_interpreted = FALSE;
  int i;

	///////////////////////////////////////////////////////////
	//PHY level trace window
	//remove all columns first
	int column_count = ListView_PHY_Level->columns();
	
	for(i = column_count - 1; i >= 0; i--)
	{	ListView_PHY_Level->removeColumn(i);
	}

	//add common columns
	ListView_PHY_Level->setSorting(-1);
	ListView_PHY_Level->addColumn("Dir");	//0
	ListView_PHY_Level->setColumnAlignment(DIR_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("Len");		//1
	ListView_PHY_Level->setColumnAlignment(LEN_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("Time");		//2
	ListView_PHY_Level->setColumnAlignment(TIME_COL, Qt::AlignHCenter);

	//add Data column
	ListView_PHY_Level->addColumn
		("                                                                                                                   ");//9

	ListView_PHY_Level->clear();
	
	buttonGroup1->setTitle("PHY Level Trace - Raw Hex Mode");

	///////////////////////////////////////////////////////////
	//ATM/AAL level trace window
	//remove all columns first
	column_count = ListView_AAL_Level->columns();
	
	for(i = column_count - 1; i >= 0; i--)
	{	ListView_AAL_Level->removeColumn(i);
	}

	//add common columns
	ListView_AAL_Level->setSorting(-1);
	ListView_AAL_Level->addColumn("Dir");	//0
	ListView_AAL_Level->setColumnAlignment(DIR_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("Len");		//1
	ListView_AAL_Level->setColumnAlignment(LEN_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("Time");		//2
	ListView_AAL_Level->setColumnAlignment(TIME_COL, Qt::AlignHCenter);

	//add Data column
	ListView_AAL_Level->addColumn
		("                                                                                                                   ");//9

	ListView_AAL_Level->clear();

	buttonGroup2->setTitle("ATM/AAL Level Trace - Raw Hex Mode");	
}

void atm_line_trace_impl::on_ShowInterpretedTrace()
{
	show_interpreted = TRUE;

	///////////////////////////////////////////////////////////
	//PHY level trace window
	//remove all columns first
	int column_count = ListView_PHY_Level->columns();
  int i;
	
	for(i = column_count - 1; i >= 0; i--)
	{	ListView_PHY_Level->removeColumn(i);
	}
	
	//add common columns
	ListView_PHY_Level->setSorting(-1);
	ListView_PHY_Level->addColumn("Dir");	//0
	ListView_PHY_Level->setColumnAlignment(DIR_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("Len");		//1
	ListView_PHY_Level->setColumnAlignment(LEN_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("Time");		//2
	ListView_PHY_Level->setColumnAlignment(TIME_COL, Qt::AlignHCenter);

	//add columns for interpreted trace
	ListView_PHY_Level->addColumn("Cell Type");	//3
	ListView_PHY_Level->setColumnAlignment(CELL_TYPE_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("GFC");		//4
	ListView_PHY_Level->setColumnAlignment(GFC_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("VPI", -1);		//5
	ListView_PHY_Level->setColumnAlignment(VPI_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("VCI");		//6
	ListView_PHY_Level->setColumnAlignment(VCI_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("PT");		//7
	ListView_PHY_Level->setColumnAlignment(PT_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("CLP");		//8
	ListView_PHY_Level->setColumnAlignment(CLP_COL, Qt::AlignHCenter);
	ListView_PHY_Level->addColumn("HEC");		//9
	ListView_PHY_Level->setColumnAlignment(HEC_COL, Qt::AlignHCenter);

	//add Data column
	ListView_PHY_Level->addColumn
		("Payload                                                                                                                   ");//9

	ListView_PHY_Level->clear();

	buttonGroup1->setTitle("PHY Level Trace - Interpreted Mode");
	///////////////////////////////////////////////////////////
	//ATM/AAL level trace window
	//remove all columns first
	column_count = ListView_AAL_Level->columns();
	
	for(i = column_count - 1; i >= 0; i--)
	{	ListView_AAL_Level->removeColumn(i);
	}
	
	//add common columns
	ListView_AAL_Level->setSorting(-1);
	ListView_AAL_Level->addColumn("Dir");	//0
	ListView_AAL_Level->setColumnAlignment(DIR_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("Len");		//1
	ListView_AAL_Level->setColumnAlignment(LEN_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("Time");		//2
	ListView_AAL_Level->setColumnAlignment(TIME_COL, Qt::AlignHCenter);

	//add columns for interpreted trace
	ListView_AAL_Level->addColumn("Cell Type");	//3
	ListView_AAL_Level->setColumnAlignment(CELL_TYPE_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("GFC");		//4
	ListView_AAL_Level->setColumnAlignment(GFC_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("VPI", -1);	//5
	ListView_AAL_Level->setColumnAlignment(VPI_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("VCI");		//6
	ListView_AAL_Level->setColumnAlignment(VCI_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("PT");		//7
	ListView_AAL_Level->setColumnAlignment(PT_COL, Qt::AlignHCenter);

	ListView_AAL_Level->addColumn("CLP");		//8
	ListView_AAL_Level->setColumnAlignment(CLP_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("OAM Type");		//9
	ListView_AAL_Level->setColumnAlignment(AAL_OAM_TYPE_COL, Qt::AlignHCenter);
	ListView_AAL_Level->addColumn("OAM Funcn Type");	//10
	ListView_AAL_Level->setColumnAlignment(AAL_OAM_FUNC_TYPE_COL, Qt::AlignHCenter);

	//add Data column
	ListView_AAL_Level->addColumn
		("Payload                                                                                                                   ");//9

	ListView_AAL_Level->clear();

	buttonGroup2->setTitle("ATM/AAL Level Trace - Interpreted Mode");
}


int atm_line_trace_impl::read_trace_config(void)
{
	int rc = globalTesterWizPtr->atm_api_obj.ATMAPIReadTraceConfiguration();

	if(rc)
	{	
		qstr_err_message.sprintf(
			"ATMAPIReadTraceConfiguration()!!. Return code 0x%02X",
			rc
			);
		QMessageBox::warning(this, "Tracing Error", qstr_err_message );

		return false;
	}
	return true;
}

int atm_line_trace_impl::set_trc_config(unsigned char config)
{
	return globalTesterWizPtr->atm_api_obj.ATMAPISetTraceConfiguration(config);
}

int atm_line_trace_impl::read_trace_data(void)
{
	unsigned short i;
	ATM_CELL_STRUCT * ATM_cell_ptr = (ATM_CELL_STRUCT * )trc_data_bfr;
	TRACE_STATUS_ELEMENT_STRUCT trace_element_struct;

	int rc = globalTesterWizPtr->
			atm_api_obj.ATMAPIReadTraceData(&trace_element_struct,
											trc_data_bfr
											);

	if(rc == false)
		return rc;

	qstr_direction = (trace_element_struct.trace_type  == 0x00) ? "Rx" : "Tx";
	qstr_length.sprintf("%u", trace_element_struct.trace_length);
	qstr_time_stamp.sprintf("%u", trace_element_struct.trace_time_stamp);

	qstr_cell_type = "";
	qstr_gfc = "";
	qstr_vpi = "";
	qstr_vci = "";

	qstr_pt = "";
	qstr_clp= "";
	qstr_hec= "";

	int cell_type = get_cell_type();

	if(show_interpreted == TRUE)
	{
		qstr_gfc = "--";

		qstr_vpi.sprintf("%4u", GFC_VPI);

		qstr_vci.sprintf("%4u", VCI);

		qstr_pt.sprintf("0x%1X", PTI);

		qstr_clp.sprintf("%1d", CLP);

		qstr_hec.sprintf("0x%02X", HEC);

		/////////////////////////////////////////////////////////////
		qstr_phy_level_data = "";
		for(i = 0; i < 48; i ++) {
			qstr_tmp.sprintf("%02X ", ATM_cell_ptr->ATM_payload[i]);
			qstr_phy_level_data += qstr_tmp;
		}
		qstr_atmal_level_data = qstr_phy_level_data;
		/////////////////////////////////////////////////////////////

	}else 
	{
		qstr_phy_level_data = "";
		for(i = 0; i < 53; i ++) {
			qstr_tmp.sprintf("%02X ", trc_data_bfr[i]);
			qstr_phy_level_data += qstr_tmp;
		}

		qstr_atmal_level_data = qstr_phy_level_data;
	}

	if(	trace_to_file_flag == true)
	{	trace_to_file_phy_level_cell();
	}

	set_show_flags(cell_type);

	display_phy_level_cell(cell_type);

	display_atmal_level_cell(cell_type);

	return rc;
}


typedef struct
{
	unsigned char byte0;
	unsigned char byte1;
	unsigned char byte2;
	unsigned char byte3;
	unsigned char byte4;

}ATM_CELL_HEADER;

int atm_line_trace_impl::get_cell_type()
{

	ATM_CELL_HEADER * atm_hdr_ptr;
	ATM_CELL_STRUCT * ATM_cell_ptr;

	atm_hdr_ptr = (ATM_CELL_HEADER * )trc_data_bfr;
	ATM_cell_ptr = (ATM_CELL_STRUCT * )trc_data_bfr;
/*
	DbgPrint1(DBG_ATM_LINE_TRACE, "header bytes :\n");
	for(int i = 0; i < sizeof(ATM_HEADER_STRUCT); i++)
	{
		DbgPrint2(DBG_ATM_LINE_TRACE, "0x%02X ", trc_data_bfr[i]);
	}

	DbgPrint2(DBG_ATM_LINE_TRACE, "atm_hdr_ptr->byte0 : 0x%02X\n", atm_hdr_ptr->byte0);
	DbgPrint2(DBG_ATM_LINE_TRACE, "atm_hdr_ptr->byte1 : 0x%02X\n", atm_hdr_ptr->byte1);
	DbgPrint2(DBG_ATM_LINE_TRACE, "atm_hdr_ptr->byte2 : 0x%02X\n", atm_hdr_ptr->byte2);
	DbgPrint2(DBG_ATM_LINE_TRACE, "atm_hdr_ptr->byte3 : 0x%02X\n", atm_hdr_ptr->byte3);
	DbgPrint2(DBG_ATM_LINE_TRACE, "atm_hdr_ptr->byte4 : 0x%02X\n", atm_hdr_ptr->byte4);
*/
	GFC_VPI = ((ATM_cell_ptr->ATM_header.GFC_VPI << 4) & 0x0FF0) |
				((ATM_cell_ptr->ATM_header.VPI_VCI >> 4) & 0x000F);

	VCI = ((ATM_cell_ptr->ATM_header.VPI_VCI << 12) & 0xF000) |
				((ATM_cell_ptr->ATM_header.VCI << 4) & 0x0FF0) |
					((ATM_cell_ptr->ATM_header.VCI_PT_CLP >> 4) & 0x000F);

	PTI = ((ATM_cell_ptr->ATM_header.VCI_PT_CLP >> 1) & 0x07);

	CLP = ATM_cell_ptr->ATM_header.VCI_PT_CLP & 0x01;

	HEC = ATM_cell_ptr->ATM_header.HEC;

	if(!(GFC_VPI & 0x00FF) && !VCI)
	{
		if(	!atm_hdr_ptr->byte0 &&
			!atm_hdr_ptr->byte1 && 
			!atm_hdr_ptr->byte2 &&
			!(atm_hdr_ptr->byte3 & 0xFE) &&
			(CLP & 0x01)
		  )
		{
			return ATM_IDLE_CELL;
		}

		if(	!atm_hdr_ptr->byte0 &&
			!atm_hdr_ptr->byte1 && 
			!atm_hdr_ptr->byte2 &&
			!(atm_hdr_ptr->byte3 & 0xFC) &&
			atm_hdr_ptr->byte3 & 0x03
		  )
		{
			return ATM_PHYSICAL_LAYER_OAM_CELL;
		}


		if(	!(atm_hdr_ptr->byte0 & 0x0F)	&&
			!atm_hdr_ptr->byte1				&& 
			!atm_hdr_ptr->byte2				&&
			!(atm_hdr_ptr->byte3 & 0xF0)	&&
			atm_hdr_ptr->byte3 & 0x01
		  )
		{
			return ATM_RESERVED_FOR_PHYSICAL_LAYER_CELL;
		}

		if(	!(atm_hdr_ptr->byte0 & 0x0F)	&&
			!atm_hdr_ptr->byte1				&& 
			!atm_hdr_ptr->byte2				&&
			!(atm_hdr_ptr->byte3 & 0xF0)	&&
			!atm_hdr_ptr->byte3 & 0x01
		  )
		{
			return ATM_UNASSIGNED_CELL;
		}

	}else
	{
		if(!(PTI & 0x04))
		{
			return ATM_USER_DATA_CELL;

		}else if(PTI & 0x04)
		{
			return ATM_OAM_F5_CELL;

		}
	}

	return ATM_UNKNOWN_CELL;
}

void atm_line_trace_impl::display_phy_level_cell(int cell_type)
{
	if(chk_PHY_StopAutoScroll->isChecked() == true)
		return;

	//set_show_flags(cell_type);

	if(	show_phy_cell == false && show_atmal_cell == false)
		return;

	list_view_item_phy_level = 
		new QListViewItem(ListView_PHY_Level, ListView_PHY_Level->lastItem());

	list_view_item_phy_level->setText(DIR_COL, qstr_direction);

	list_view_item_phy_level->setText(LEN_COL, qstr_length);

	list_view_item_phy_level->setText(TIME_COL, qstr_time_stamp);

	if(	show_interpreted == TRUE)
	{
		list_view_item_phy_level->setText(CELL_TYPE_COL, qstr_cell_type);
		list_view_item_phy_level->setText(GFC_COL, qstr_gfc);
		list_view_item_phy_level->setText(VPI_COL, qstr_vpi);
		list_view_item_phy_level->setText(VCI_COL, qstr_vci);

		list_view_item_phy_level->setText(PT_COL, qstr_pt);
		list_view_item_phy_level->setText(CLP_COL, qstr_clp);
		list_view_item_phy_level->setText(HEC_COL, qstr_hec);

		list_view_item_phy_level->setText(PHY_DATA_COL_INTERPRETED, qstr_phy_level_data);

	}else
	{
		list_view_item_phy_level->setText(PHY_DATA_COL_RAW, qstr_phy_level_data);
	}

	LIST_VIEW_ADDSTR(ListView_PHY_Level, NULL);
}

void atm_line_trace_impl::display_atmal_level_cell(int cell_type)
{
	if(checkBox_AAL_StopAutoScroll->isChecked() == true)
		return;

//	if(show_interpreted == FALSE)
//		return;

	//set_show_flags(cell_type);

	if(	show_atmal_cell == false)
		return;

	list_view_item_aal_level = 
		new QListViewItem(ListView_AAL_Level, ListView_AAL_Level->lastItem());
	
	list_view_item_aal_level->setText(DIR_COL, qstr_direction);
	
	list_view_item_aal_level->setText(LEN_COL, qstr_length);
	
	list_view_item_aal_level->setText(TIME_COL, qstr_time_stamp);
	
	if(show_interpreted == TRUE)
	{
		list_view_item_aal_level->setText(CELL_TYPE_COL, qstr_cell_type);
		list_view_item_aal_level->setText(GFC_COL, qstr_gfc);
		list_view_item_aal_level->setText(VPI_COL, qstr_vpi);
		list_view_item_aal_level->setText(VCI_COL, qstr_vci);
		
		list_view_item_aal_level->setText(PT_COL, qstr_pt);
		list_view_item_aal_level->setText(CLP_COL, qstr_clp);

		list_view_item_aal_level->setText(AAL_OAM_TYPE_COL, qstr_OAM_Cell_Type);
		list_view_item_aal_level->setText(AAL_OAM_FUNC_TYPE_COL, qstr_OAM_Function_Type);

		list_view_item_aal_level->setText(AAL_DATA_COL_INTERPRETED, qstr_atmal_level_data);
		
	}else
	{
		list_view_item_aal_level->setText(AAL_DATA_COL_RAW, qstr_atmal_level_data);
	}
	
	LIST_VIEW_ADDSTR(ListView_AAL_Level, NULL);
}

int atm_line_trace_impl::set_show_flags(int cell_type)
{
	show_atmal_cell = false;
	show_phy_cell = false;

	switch(cell_type)
	{
	case ATM_UNKNOWN_CELL:
		qstr_cell_type = "?";
		break;

	case ATM_IDLE_CELL:
		qstr_cell_type = "Idle";
		if(checkBox_show_idle_cells->isChecked() == false)
		{	show_phy_cell = false;
		}else
		{	show_phy_cell = true;
		}
		break;

	case ATM_PHYSICAL_LAYER_OAM_CELL:
	case ATM_RESERVED_FOR_PHYSICAL_LAYER_CELL:
		qstr_cell_type = "PHY Layer.";
		if(checkBox_show_phy_layer_cells->isChecked() == false)
		{	show_phy_cell = false;
		}else
		{	show_phy_cell = true;
		}

		break;

	case ATM_UNASSIGNED_CELL:
		qstr_cell_type = "Unassigned";
		if(checkBox_show_unassigned_cells->isChecked() == false)
		{	show_phy_cell = false;
		}else
		{	show_phy_cell = true;
		}

		break;

	case ATM_USER_DATA_CELL:
		qstr_cell_type = "User Data";
		if(checkBox_Show_User_Data_cells->isChecked() == false)
		{	show_atmal_cell = false;
		}else
		{	show_atmal_cell = true;

			decode_user_data_cell();
		}
		break;

	case ATM_OAM_F5_CELL:
		qstr_cell_type = "OAM";
		if(checkBox_Show_OAM_F5_cells->isChecked() == false)
		{	show_atmal_cell = false;
		}else
		{	show_atmal_cell = true;

			decode_oam_cell();
		}
		break;
	}

	return 0;
}

void atm_line_trace_impl::decode_user_data_cell()
{
	AAL5_CPCS_PDU_TRAILER * trailer_ptr;
	int i;

	qstr_OAM_Cell_Type = "---";
	qstr_OAM_Function_Type = "---";
	
	ATM_CELL_STRUCT * ATM_cell_ptr = (ATM_CELL_STRUCT * )trc_data_bfr;

	PTI = ((ATM_cell_ptr->ATM_header.VCI_PT_CLP >> 1) & 0x07);

	//DbgPrint1(DBG_ATM_LINE_TRACE, "decode_user_data_cell()\n");

	if(PTI & 0x01)
	{
		trailer_ptr = (AAL5_CPCS_PDU_TRAILER * )&trc_data_bfr[45];

		qstr_atmal_level_data = "";

		/////////////////////////////////////////////////////////////
		//qstr_atmal_level_data += ", Data: ";
		for(i = 0; i < 40; i ++) {
			qstr_tmp.sprintf("%02X ", ATM_cell_ptr->ATM_payload[i]);
			qstr_atmal_level_data += qstr_tmp;
		}
		/////////////////////////////////////////////////////////////

		qstr_tmp.sprintf("  UU: 0x%02X,", trailer_ptr->UU);
		qstr_atmal_level_data += qstr_tmp;

		qstr_tmp.sprintf(" CPI: 0x%02X,", trailer_ptr->CPI);
		qstr_atmal_level_data += qstr_tmp;

		qstr_tmp.sprintf(" length: %u,", ntohs(trailer_ptr->length));
		qstr_atmal_level_data += qstr_tmp;

		qstr_tmp.sprintf(" CRC: 0x%04X", trailer_ptr->CRC);
		qstr_atmal_level_data += qstr_tmp;

	}
}


void atm_line_trace_impl::decode_oam_cell()
{
	bool show_default_payload = true;

	qstr_OAM_Cell_Type = "?";
	qstr_OAM_Function_Type = "?";
	
	ATM_CELL_STRUCT * ATM_cell_ptr = (ATM_CELL_STRUCT * )trc_data_bfr;

	OAM_PAYLOAD_STRUCT * oam_payload_ptr = 
		(OAM_PAYLOAD_STRUCT * )ATM_cell_ptr->ATM_payload;

	unsigned char OAM_Cell_Type = oam_payload_ptr->OAM_type_func_type >> 4;
	unsigned char OAM_Function_Type = oam_payload_ptr->OAM_type_func_type & 0x0F;

	DbgPrint3(DBG_ATM_LINE_TRACE,
		"OAM_Cell_Type : 0x%02X, OAM_Function_Type : 0x%02X\n",
		OAM_Cell_Type, OAM_Function_Type
		);

	switch(OAM_Cell_Type)
	{
	case OAM_TYPE_FAULT_MGMT:
		DbgPrint1(DBG_ATM_LINE_TRACE, "OAM_TYPE_FAULT_MGMT\n");

		qstr_OAM_Cell_Type = "Fault Mgmt.";

		switch(oam_payload_ptr->OAM_type_func_type)
		{
		case OAM_FAULT_MGMT_AIS:
			DbgPrint1(DBG_ATM_LINE_TRACE, "OAM_FAULT_MGMT_AIS\n");

			qstr_OAM_Function_Type = "AIS";
			form_ais_rdi_payload_sring();
			show_default_payload = false;
			break;

		case OAM_FAULT_MGMT_RDI:
			DbgPrint1(DBG_ATM_LINE_TRACE, "OAM_FAULT_MGMT_RDI\n");

			qstr_OAM_Function_Type = "RDI";
			form_ais_rdi_payload_sring();
			show_default_payload = false;
			break;

		case OAM_FAULT_MGMT_CONTINUITY:
			DbgPrint1(DBG_ATM_LINE_TRACE, "OAM_FAULT_MGMT_CONTINUITY\n");

			qstr_OAM_Function_Type = "Continuity Check";
			form_continuity_check_payload_sring();
			show_default_payload = false;
			break;

		case OAM_FAULT_MGMT_LOOPBACK:
			DbgPrint1(DBG_ATM_LINE_TRACE, "OAM_FAULT_MGMT_LOOPBACK\n");

			qstr_OAM_Function_Type = "Loopback";
			form_loop_back_payload_sring();
			show_default_payload = false;
			break;

		case OAM_ACT_DEACT_CONTINUITY:
			DbgPrint1(DBG_ATM_LINE_TRACE, "OAM_ACT_DEACT_CONTINUITY\n");
			qstr_OAM_Function_Type = "Deact. Continuity Check";
			break;
		default:
			DbgPrint1(DBG_ATM_LINE_TRACE, "?\n");
			qstr_OAM_Function_Type = "?";
			break;
		}

		break;

	case OAM_TYPE_PREFORMANCE_MGMT:
		DbgPrint1(DBG_ATM_LINE_TRACE, "OAM_TYPE_PREFORMANCE_MGMT\n");
		qstr_OAM_Cell_Type = "Performance Mgmt.";
		break;

	case OAM_TYPE_ACT_DEACT:
		DbgPrint1(DBG_ATM_LINE_TRACE, "OAM_TYPE_ACT_DEACT\n");

		qstr_OAM_Cell_Type = "Act./Deact.";

		if(OAM_Function_Type == 0x01)
		{	qstr_OAM_Function_Type = "Cont Check";
			form_act_deact_continuity_check_payload_sring();
		}

		
		show_default_payload = false;
		break;

	case OAM_TYPE_SYSTEM_MGMT:
		DbgPrint1(DBG_ATM_LINE_TRACE, "OAM_TYPE_SYSTEM_MGMT\n");

		qstr_OAM_Cell_Type = "System Mgmt.";
		break;
	}

	if(show_interpreted == FALSE)
	{	show_default_payload = true;
	}

	if(show_default_payload == true)
	{	qstr_atmal_level_data = "";
		for(int i = 0; i < 53; i ++) {
			qstr_tmp.sprintf("%02X ", trc_data_bfr[i]);
			qstr_atmal_level_data += qstr_tmp;
		}
	}
}

#define MESSAGE_ID_ACT_REQ					0X01	//000001
#define MESSAGE_ID_ACT_DEACT_REQ_CONFIRMED	0X02	//000010
#define MESSAGE_ID_ACT_DEACT_REQ_DENIED		0X03	//000011
#define MESSAGE_ID_DEACT_REQ				0X05	//000101

void atm_line_trace_impl::form_act_deact_continuity_check_payload_sring()
{
	int i;
	QString qstr_tmp;
	
	ATM_CELL_STRUCT * ATM_cell_ptr = (ATM_CELL_STRUCT * )trc_data_bfr;

	OAM_PAYLOAD_STRUCT * oam_payload_ptr = 
		(OAM_PAYLOAD_STRUCT * )ATM_cell_ptr->ATM_payload;

	unsigned char message_id = trc_data_bfr[6] >> 2;

	unsigned char direction_of_activation = trc_data_bfr[6] & 0x3;

	DbgPrint3(DBG_ATM_LINE_TRACE, "message_id : 0x%X, direction_of_activation : 0x%X\n",
		message_id, direction_of_activation
		);

	unsigned char correlation_tag = trc_data_bfr[7];

	unsigned char pm_block_AB = trc_data_bfr[8] >> 4;
	unsigned char pm_block_BA = trc_data_bfr[8] & 0x0F;

	qstr_atmal_level_data = "";
	switch(message_id)
	{
	case MESSAGE_ID_ACT_REQ:
		qstr_atmal_level_data = "Act Req";
		break;

	case MESSAGE_ID_ACT_DEACT_REQ_CONFIRMED:
		qstr_atmal_level_data = "Act/Deact Req Confirmed";
		break;

	case MESSAGE_ID_ACT_DEACT_REQ_DENIED:
		qstr_atmal_level_data = "Act/Deact Req Denied";
		break;

	case MESSAGE_ID_DEACT_REQ:
		qstr_atmal_level_data = "Deact Req";
		break;
	}

	qstr_atmal_level_data += ", ";

	if(direction_of_activation == 0x02)
	{
		qstr_atmal_level_data += " A->B (From Activator)";

	}else if(direction_of_activation == 0x01)
	{
		qstr_atmal_level_data += " B->A (To Activator)";
	}else
	{
		qstr_atmal_level_data += " Direction Unknown";
	}

	//////////////////////////////////////////////////////
	qstr_tmp.sprintf(" Corr Tag: 0x%04X", correlation_tag);
	qstr_atmal_level_data += qstr_tmp;
	//////////////////////////////////////////////////////

	qstr_tmp.sprintf(", PM block AB: 0x%02X, PM block BA: 0x%02X, ",
		pm_block_AB, pm_block_BA
		);
	qstr_atmal_level_data += qstr_tmp;
	//////////////////////////////////////////////////////
	qstr_atmal_level_data += "Unused: ";
	for(i = 9; i < 9+3; i ++) 
	{	qstr_tmp.sprintf("%02X ", ATM_cell_ptr->ATM_payload[i]);
		qstr_atmal_level_data += qstr_tmp;
	}
	qstr_atmal_level_data += "... ";
	//////////////////////////////////////////////////////
	qstr_tmp.sprintf(", CRC10: 0x%04X", oam_payload_ptr->CRC_10);
	qstr_atmal_level_data += qstr_tmp;

}

void atm_line_trace_impl::form_continuity_check_payload_sring()
{
	int i;

	ATM_CELL_STRUCT * ATM_cell_ptr = (ATM_CELL_STRUCT * )trc_data_bfr;

	OAM_PAYLOAD_STRUCT * oam_payload_ptr = 
		(OAM_PAYLOAD_STRUCT * )ATM_cell_ptr->ATM_payload;

	qstr_atmal_level_data = "Unused: ";
	for(i = 1; i < 4; i ++) 
	{	qstr_tmp.sprintf("%02X ", ATM_cell_ptr->ATM_payload[i]);
		qstr_atmal_level_data += qstr_tmp;
	}
	qstr_atmal_level_data += "... ";

	////////////////////////////////////////////////////////////////////////

	qstr_tmp.sprintf(" CRC10: 0x%04X", oam_payload_ptr->CRC_10);
	qstr_atmal_level_data += qstr_tmp;

	////////////////////////////////////////////////////////////////////////

}

void atm_line_trace_impl::form_ais_rdi_payload_sring()
{
	int i;
	QString qstr_tmp;

	ATM_CELL_STRUCT * ATM_cell_ptr = (ATM_CELL_STRUCT * )trc_data_bfr;

	OAM_PAYLOAD_STRUCT * oam_payload_ptr = 
		(OAM_PAYLOAD_STRUCT * )ATM_cell_ptr->ATM_payload;

	OAM_AIS_RDI_STRUCT * ais_rdi_ptr = 
		(OAM_AIS_RDI_STRUCT * )&oam_payload_ptr->FUNC_SPEC.AIS_RDI_STR;

	////////////////////////////////////////////////////////////////////////
	qstr_tmp.sprintf("Defect type : 0x%02X, ", ais_rdi_ptr->defect_type);
	qstr_atmal_level_data = qstr_tmp;

	////////////////////////////////////////////////////////////////////////
#define NUM_OF_DEFECT_LOCATION_BYTES	3 //LGTH_OAM_AIS_RDI_DEFECT_LOC

	qstr_atmal_level_data += "Defect location:";

	for(i=0; i < NUM_OF_DEFECT_LOCATION_BYTES; i++)
	{
		qstr_tmp.sprintf(" 0x%02X", ais_rdi_ptr->defect_location[i]);
		qstr_atmal_level_data += qstr_tmp;
	}
	qstr_atmal_level_data += "... ";
	////////////////////////////////////////////////////////////////////////

#define NUM_OF_UNUSED_BYTES		3 //LGTH_OAM_AIS_RDI_UNUSED

	qstr_atmal_level_data += "Unused:";

	for(i=0; i < NUM_OF_UNUSED_BYTES; i++)
	{
		qstr_tmp.sprintf(" 0x%02X", ais_rdi_ptr->unused[i]);
		qstr_atmal_level_data += qstr_tmp;
	}
	qstr_atmal_level_data += "... ";
	////////////////////////////////////////////////////////////////////////

	qstr_tmp.sprintf(" CRC10: 0x%04X", oam_payload_ptr->CRC_10);
	qstr_atmal_level_data += qstr_tmp;

	////////////////////////////////////////////////////////////////////////
}

void atm_line_trace_impl::form_loop_back_payload_sring()
{
	QString qstr_tmp;
	int i;

	ATM_CELL_STRUCT * ATM_cell_ptr = (ATM_CELL_STRUCT * )trc_data_bfr;

	OAM_PAYLOAD_STRUCT * oam_payload_ptr = 
		(OAM_PAYLOAD_STRUCT * )ATM_cell_ptr->ATM_payload;

	OAM_LOOPBACK_STRUCT * loop_back_ptr = 
		(OAM_LOOPBACK_STRUCT * )&oam_payload_ptr->FUNC_SPEC.LPBK_STR;

	////////////////////////////////////////////////////////////////////////
	qstr_tmp.sprintf("Lpbk Ind: 0x%02X, ",
		loop_back_ptr->loopback_indication
		);
	qstr_atmal_level_data = qstr_tmp;

	////////////////////////////////////////////////////////////////////////
	qstr_tmp.sprintf(" Corr Tag: 0x%04X, ", 
		ntohl(loop_back_ptr->correlation_tag)
		);
	qstr_atmal_level_data += qstr_tmp;

	////////////////////////////////////////////////////////////////////////
	qstr_atmal_level_data += " Lpbk Loc Id:";

	for(i=0; i < LGTH_OAM_LPBK_LPBK_LOC_ID; i++)
	{
		qstr_tmp.sprintf(" 0x%02X", loop_back_ptr->loopback_location_ID[i]);
		qstr_atmal_level_data += qstr_tmp;
	}
	qstr_atmal_level_data += ",";
	////////////////////////////////////////////////////////////////////////
	qstr_atmal_level_data += " Src Loc Id:";

	for(i=0; i < LGTH_OAM_LPBK_SRC_ID; i++)
	{
		qstr_tmp.sprintf(" 0x%02X", loop_back_ptr->source_ID[i]);
		qstr_atmal_level_data += qstr_tmp;
	}
	//qstr_atmal_level_data += ", ";
	////////////////////////////////////////////////////////////////////////
	qstr_tmp.sprintf("  CRC10: 0x%04X",
		oam_payload_ptr->CRC_10);//()loop_back_ptr->trc_data_bfr[51]);
	qstr_atmal_level_data += qstr_tmp;
}

void atm_line_trace_impl::trace_to_file_atmal_level_cell()
{

}

//will trace ALL cells to the file
void atm_line_trace_impl::trace_to_file_phy_level_cell()
{
	//DbgPrint1(1, "atm_line_trace_impl::trace_to_file_phy_level_cell()\n");

	qstr_trace_file_data = qstr_direction	+ "\t";
	qstr_trace_file_data += qstr_length		+ "\t";
	qstr_trace_file_data += qstr_time_stamp + "\t";

	for(int i = 0; i < 53; i ++) 
	{	qstr_tmp.sprintf("%02X ", trc_data_bfr[i]);
		qstr_trace_file_data += qstr_tmp;
	}

	qstr_trace_file_data += "\n\n";

	fprintf	(globalTesterWizPtr->trace_file, (const char *)qstr_trace_file_data);
	fflush	(globalTesterWizPtr->trace_file);

	qstr_tmp.sprintf(" (Trace File size: %u bytes).",
		ftell(globalTesterWizPtr->trace_file));
	setCaption( widget_caption + qstr_tmp );
}

