// atm_send_info_frames_impl.cpp: implementation of the atm_send_info_frames_impl class.
//
//////////////////////////////////////////////////////////////////////

#include "MainWizardImpl.h"

extern TesterWizardImpl * globalTesterWizPtr;

#define DBG_ATM_SEND_INFO 0

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

atm_send_info_frames_impl::atm_send_info_frames_impl()
{
	set_widget_caption(this);
	initialize();
	window_closed = true;
}

atm_send_info_frames_impl::~atm_send_info_frames_impl()
{

}

void atm_send_info_frames_impl::showEvent(QShowEvent * ev)
{
	set_widget_position_on_screen(this);
}

void atm_send_info_frames_impl::closeEvent(QCloseEvent * e)
{
	if(timer_id)
	{	killTimer(timer_id);
		timer_id = 0;
	}
	initialize();

	window_closed = true;

	e->accept();
}

void atm_send_info_frames_impl::timerEvent(QTimerEvent * e)
{
	//Set values to be transmitted, just in case were overwritten
	//by 'Send One' frame button.
	atm_init_multiple_tx_buffer();

	send_frame(	&tx_el, globalTesterWizPtr->m_cCard);

	if(num_of_frames_to_transmit == tx_ok_counter)
	{	//this will stop sending many
		on_send_many_frames();
	}
}

int atm_send_info_frames_impl::initialize()
{
	//some default data to send
	qstr_temp = "01 02 03";
	lineEdit_Tx_data->setText(qstr_temp);

	vpi = 0;
	vci = 35;

	timer_id = 0;

	//default VPI
	qstr_temp.sprintf("%d", vpi);
	lineEdit_VPI->setText(qstr_temp);
	
	//default VCI
	qstr_temp.sprintf("%d", vci);
	lineEdit_VCI->setText(qstr_temp);
	
	//length of data when sending many
	tx_length = 100;

	qstr_temp.sprintf("%d", tx_length);
	edt_send_many_length->setText(qstr_temp);
	///////////////////////////////////////////
	num_of_frames_to_transmit = 1;

	qstr_temp.sprintf("%d", num_of_frames_to_transmit);
	lineEdit_number_of_frames_to_send->setText(qstr_temp);
	///////////////////////////////////////////

	tx_ok_counter = 0;

	qstr_temp.sprintf("%d", tx_ok_counter);
	lbl_number_of_frames_transmitted->setText(qstr_temp);

	sending_many = false;

	qstr_temp = "Start";
	btn_start_stop_send_many->setText(qstr_temp);

	textLabel_transmission_status->setText("");	

	return 0;
}

void atm_send_info_frames_impl::on_send_many_frames()
{
	int i_temp;

	if(sending_many == false)
	{	//user wants to start sending many
		///////////////////////////////////////////////////////////////
		tx_ok_counter = 0;
		qstr_temp.sprintf("%d", tx_ok_counter);
		lbl_number_of_frames_transmitted->setText(qstr_temp);

		///////////////////////////////////////////////////////////////
		if(get_vpi_vci())
			return;
		////////////////////////////////////////////////////////////////

		i_temp = get_edit_box_value(edt_send_many_length, &ok);
		if(	ok == false								||
			i_temp < AAL_MIN_LENGTH_TX_USER_INFO	||
			i_temp > AAL_MAX_LENGTH_TX_USER_INFO
		   )
		{	
			qstr_messages.sprintf(
				"Data length should be between %u and %u bytes long.",
				AAL_MIN_LENGTH_TX_USER_INFO,
				AAL_MAX_LENGTH_TX_USER_INFO
				);
			QMessageBox::warning( 0, "Invalid Input.", qstr_messages);
			return;
		}else
		{	tx_length = i_temp;
		}
		////////////////////////////////////////////////////////////////

		i_temp = get_edit_box_value(lineEdit_number_of_frames_to_send, &ok);
		if(	ok == false )
		{	
			qstr_messages.sprintf("Number of frames to transmit is invalid.");
			QMessageBox::warning( 0, "Invalid Input.", qstr_messages);
			return;
		}else
		{	num_of_frames_to_transmit = i_temp;
		}

		//////////////////////////////////////////////////////////////////
		//start the transmission timer.
		if((timer_id = startTimer(TX_DELAY)) == 0)
		{	QMessageBox::warning( 0, "Error.", "Failed to start transmission timer!!");
			return;
		}else
		{
			qstr_temp = "Stop";
			btn_start_stop_send_many->setText(qstr_temp);

			sending_many = true;

			//set values to be transmitted
			atm_init_multiple_tx_buffer();

			//disable 'Send One' button
			btn_send_one_frame->setEnabled(false);
		}

	}else
	{	//user wants to stop sending many
		sending_many = false;

		qstr_temp = "Start";
		btn_start_stop_send_many->setText(qstr_temp);

		if(timer_id)
		{	killTimer(timer_id);
			timer_id = 0;
		}

		//enable 'Send One' button
		btn_send_one_frame->setEnabled(true);
	}
}

void atm_send_info_frames_impl::on_send_one_frame()
{
	unsigned char src[200];

	if(get_vpi_vci())
		return;

	qstr_temp = lineEdit_Tx_data->text();	
	strcpy((char *)src, (const char *)qstr_temp);

	tx_length = ParseTextToHex(	strlen((const char *)src),
								src,
								tx_buff
								);

	tx_el.user_info_length = tx_length;
	tx_el.misc_Tx_bits = 0;
	tx_el.GFC_VPI = vpi;
	tx_el.VCI = vci;
	tx_el.PT = 0;
	tx_el.CLP = 0;
	tx_el.ptr_data_bfr = (unsigned long)tx_buff;

	send_frame(	&tx_el, globalTesterWizPtr->m_cCard);

}

bool atm_send_info_frames_impl::my_isVisible()
{
	return (window_closed == false) ? true : false;
}

bool atm_send_info_frames_impl::my_show()
{
	window_closed = false;
	show();
	return true;
}

int atm_send_info_frames_impl::send_frame(	AAL_TX_STATUS_EL_STRUCT * p_tx_el,
											BYTE cardnumber
											)
{
	int rc = globalTesterWizPtr->
				atm_api_obj.ATMAPITransmitFrame(p_tx_el,
												globalTesterWizPtr->m_cCard
												);
	switch(rc)
	{
	case ATMAL_TX_OK:
		tx_ok_counter++;
		qstr_temp.sprintf("%u", tx_ok_counter);
		lbl_number_of_frames_transmitted->setText(qstr_temp);

		qstr_temp = "Transmitted successfully";
		break;

	case ATMAL_TRANSMITTER_BUSY:
		qstr_temp = "Transmitter busy!";
		break;

	case ATMAL_CONFIGURATION_NOT_SET:
		qstr_temp = "ATMAL Configuration Not set!!";
		break;

	case ATMAL_TX_INVALID_DATA_LENGTH:
		qstr_temp = "Invalid Tx data length!";
		break;

	default:
		qstr_temp = "Unknown Tx return code!";
	}

	textLabel_transmission_status->setText(qstr_temp);

	return rc;
}


void atm_send_info_frames_impl::atm_init_multiple_tx_buffer()
{
	int i;
	unsigned char increment_char = 0x00;

	for(i = 0;
		i < tx_length && i < AAL_MAX_LENGTH_TX_USER_INFO;
		i++)
	{
		tx_buff[i] = increment_char++;
	}

	tx_el.user_info_length = tx_length;
	tx_el.misc_Tx_bits = 0;
	tx_el.GFC_VPI = vpi;
	tx_el.VCI = vci;
	tx_el.PT = 0;
	tx_el.CLP = 0;
	tx_el.ptr_data_bfr = (unsigned long)tx_buff;

}

int atm_send_info_frames_impl::get_vpi_vci()
{
	int i_temp = get_edit_box_value(lineEdit_VPI, &ok);

	if(ok == false || i_temp < MIN_VPI_NUMBER || i_temp > MAX_VPI_NUMBER)
	{	
		qstr_messages.sprintf(
			"VPI should be between %u and %u.",
			MIN_VPI_NUMBER,
			MAX_VPI_NUMBER
			);
		QMessageBox::warning( 0, "Invalid Input.", qstr_messages);
		return 1;
	}else
	{	vpi = i_temp;
	}
	////////////////////////////////////////////////////////////////
	i_temp = get_edit_box_value(lineEdit_VCI, &ok);
	if(ok == false || i_temp < MIN_VCI_NUMBER || i_temp > MAX_VCI_NUMBER)
	{	
		qstr_messages.sprintf(
			"VCI should be between %u and %u.",
			MIN_VPI_NUMBER,
			MAX_VPI_NUMBER
			);
		QMessageBox::warning( 0, "Invalid Input.", qstr_messages);
		return 1;
	}else
	{	vci = i_temp;
	}

	return 0;
}
