// atm_set_atm_aal_configuration_impl.cpp: implementation of the atm_set_atm_aal_configuration_impl class.
//
//////////////////////////////////////////////////////////////////////

#include "TesterDefines.h"					//debug macros

#include "MainWizardImpl.h"		//for declarations of global functions

#define DBG_ATM_AAL_CFG 0


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

atm_set_atm_aal_configuration_impl::atm_set_atm_aal_configuration_impl(
									QWidget* parent,
									const char* name,
									bool modal, WFlags fl
									):atm_set_atm_aal_configuration(
															parent,
															name,
															modal,
															fl)
{
	DbgPrint1(DBG_ATM_AAL_CFG,
		"atm_set_atm_aal_configuration_impl::atm_set_atm_aal_configuration_impl()\n");

	set_widget_caption(this);

	memset(&atm_aal_cfg_old, 0x00, sizeof(atm_aal_cfg_old));
	memset(&atm_aal_cfg, 0x00, sizeof(atm_aal_cfg));

	atm_aal_cfg.API_options = 0x0000;
	atm_aal_cfg.protocol_options = 0x0000;			
	atm_aal_cfg.buffer_options = 0;				
	atm_aal_cfg.max_length_Tx_user_info = 4096;	
	atm_aal_cfg.max_length_Rx_user_info = 4096;	
	atm_aal_cfg.AAL_PAD_char = 0x6A;				
	atm_aal_cfg.statistics_options = 0x0000;			
	/*
	atm_aal_cfg.ptr_shared_mem_info_struct;	
	atm_aal_cfg.ptr_AAL_Tx_control_struct;	
	atm_aal_cfg.ptr_AAL_Rx_control_struct;	
	*/

}

atm_set_atm_aal_configuration_impl::~atm_set_atm_aal_configuration_impl()
{

}

void atm_set_atm_aal_configuration_impl::OnOkClicked()
{
	DbgPrint1(DBG_ATM_AAL_CFG, "atm_set_atm_aal_configuration_impl::OnOkClicked()\n");
	
	ATMAL_CONFIGURATION_STRUCT atm_aal_cfg_back;

	unsigned short i_temp;
	bool ok;

	//create a backup copy and do all the temporary changes in it.
	//if all checks passed, copy the new configuration into
	//the real congig structure.
	memcpy(&atm_aal_cfg_back, &atm_aal_cfg,	sizeof(ATMAL_CONFIGURATION_STRUCT));

	//
	// copy values from visual controls to cfg structure
	//

	/////////////////////////////////////////////////////////////
	// Tx length
	i_temp = (unsigned short)get_edit_box_value(lineEdit_max_length_Tx_user_info, &ok);

	if(	ok == false ||
		i_temp < AAL_MIN_LENGTH_TX_USER_INFO ||
		i_temp > AAL_MAX_LENGTH_TX_USER_INFO
	  )
	{	
		qsr_tmp.sprintf("Max length Tx user info should be between %u and %u bytes long.",
			AAL_MIN_LENGTH_TX_USER_INFO,
			AAL_MAX_LENGTH_TX_USER_INFO
			);
		QMessageBox::warning(0,	"Invalid Configuration Parameter", qsr_tmp.latin1());
		return;
	}else
		atm_aal_cfg_back.max_length_Tx_user_info = i_temp;

	/////////////////////////////////////////////////////////////
	// Rx length
	i_temp = (unsigned short)get_edit_box_value(lineEdit_max_length_Rx_user_info, &ok);

	if(	ok == false ||
		i_temp < AAL_MIN_LENGTH_RX_USER_INFO ||
		i_temp > AAL_MAX_LENGTH_RX_USER_INFO
	  )
	{	
		qsr_tmp.sprintf("Max length Rx user info should be between %u and %u bytes long.",
			AAL_MIN_LENGTH_RX_USER_INFO,
			AAL_MAX_LENGTH_RX_USER_INFO
			);
		QMessageBox::warning(0,	"Invalid Configuration Parameter", qsr_tmp.latin1());
		return;
	}else
		atm_aal_cfg_back.max_length_Tx_user_info = i_temp;

	/////////////////////////////////////////////////////////////
	i_temp = (unsigned short)get_int_hex_value_from_line_edit_box(
										lineEdit_AAL_PAD_char, 
										&ok);
	if(ok == false)
	{	QMessageBox::warning(0, "Invalid Configuration Parameter", "AAL_PAD_char.");
		return;
	}else
		atm_aal_cfg_back.AAL_PAD_char = i_temp;

	/////////////////////////////////////////////////////////

	memcpy(&atm_aal_cfg, &atm_aal_cfg_back, sizeof(ATMAL_CONFIGURATION_STRUCT));

	done(0);

}

void atm_set_atm_aal_configuration_impl::OnCancelClicked()
{
	DbgPrint1(DBG_ATM_AAL_CFG, "atm_set_atm_aal_configuration_impl::OnCancelClicked()\n");
	close();
}

void atm_set_atm_aal_configuration_impl::closeEvent(QCloseEvent * e)
{
	DbgPrint1(DBG_ATM_AAL_CFG, "atm_set_atm_aal_configuration_impl::closeEvent()\n");
	setResult(1);
	e->accept();
}

void atm_set_atm_aal_configuration_impl::showEvent(QShowEvent * e)
{
	DbgPrint1(DBG_ATM_AAL_CFG, "atm_set_atm_aal_configuration_impl::showEvent()\n");
  
	set_widget_position_on_screen(this);

	//
	// copy values from cfg structure to visual controls
	//
	//
	//
	qsr_tmp.sprintf("%u", atm_aal_cfg.max_length_Tx_user_info);
	lineEdit_max_length_Tx_user_info->setText(qsr_tmp);

	qsr_tmp.sprintf("%u", atm_aal_cfg.max_length_Rx_user_info);
	lineEdit_max_length_Rx_user_info->setText(qsr_tmp);

	qsr_tmp.sprintf("%02X", atm_aal_cfg.AAL_PAD_char);
	lineEdit_AAL_PAD_char->setText(qsr_tmp);


}


