
#include "TesterDefines.h"					//debug macros

#include "MainWizardImpl.h"		//for declarations of global functions

#define DBG_ATM_PHY_CFG 0

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

atm_set_configuration_impl::atm_set_configuration_impl(
								QWidget* parent,
								const char* name,
								bool modal, WFlags fl ):
									atm_set_configuration(	parent,
															name,
															 modal,
															 fl)
{
	DbgPrint1(DBG_ATM_PHY_CFG,
		"atm_set_configuration_impl::atm_set_configuration_impl()\n");

	set_widget_caption(this);

	memset(&atm_phy_cfg_old, 0x00, sizeof(atm_phy_cfg_old));
	memset(&atm_phy_cfg, 0x00, sizeof(atm_phy_cfg));

	atm_phy_cfg.baud_rate = 0;

/*
PHY_INTERFACE_LEVEL_V35
PHY_INTERFACE_LEVEL_RS232
*/
	atm_phy_cfg.line_config_options = PHY_INTERFACE_LEVEL_V35;

/*
PHY_MANUAL_DTR_RTS
PHY_IGNORE_CHG_MODEM_STAT
*/
	atm_phy_cfg.modem_config_options = 0x0000;

	atm_phy_cfg.modem_status_timer = 50;

/*
PHY_DISCARD_RX_IDLE_CELLS
PHY_DISCARD_RX_UNASSIGNED_CELLS
PHY_DISCARD_RX_PHYS_LAYER_CELLS
PHY_TRANSPARENT_TX_RX_CELLS
PHY_DECODED_TX_RX_CELLS
PHY_APP_REVERSES_BIT_ORDER
*/


	atm_phy_cfg.API_options = PHY_DECODED_TX_RX_CELLS | PHY_DISCARD_RX_IDLE_CELLS | 
		PHY_DISCARD_RX_UNASSIGNED_CELLS | PHY_DISCARD_RX_PHYS_LAYER_CELLS;

/*
PHY_MANUAL_RX_SYNC
*/
	atm_phy_cfg.protocol_options = PHY_MANUAL_RX_SYNC;

	atm_phy_cfg.ATM_interface = PHY_UNI | PHY_INTERFACE_SWITCH;

/*
PHY_DISABLE_RX_HEC_CHECK
PHY_DISABLE_AUTO_SYNC_RX_BAD_HEC
PHY_DISABLE_RX_COSET
PHY_DISABLE_TX_COSET
PHY_CUSTOM_RX_COSET
PHY_CUSTOM_TX_COSET
*/
	atm_phy_cfg.HEC_options = 0x0000; //PHY_DISABLE_RX_HEC_CHECK;

	atm_phy_cfg.custom_Rx_COSET = 0x53;
	atm_phy_cfg.custom_Tx_COSET = 0x53;

/*
PHY_TX_ONLY
PHY_RX_ONLY
PHY_SINGLE_TX_BUFFER
*/
	atm_phy_cfg.buffer_options = 0x0000;

	atm_phy_cfg.max_cells_in_Tx_block = 1; //20;

	atm_phy_cfg.Tx_underrun_cell_GFC = 0x00;
	atm_phy_cfg.Tx_underrun_cell_PT = 0x00;
	atm_phy_cfg.Tx_underrun_cell_CLP = 0x01;			/* 0x01 */
	atm_phy_cfg.Tx_underrun_cell_payload = 0x6A;	/* 0x6A */
	atm_phy_cfg.number_cells_Tx_underrun = 1;
	atm_phy_cfg.max_cells_in_Rx_block = 1; //20;
	atm_phy_cfg.Rx_hunt_timer = 100;	
	atm_phy_cfg.Rx_sync_bytes = 0x0152; //0x0001;    //0x0152;
	atm_phy_cfg.Rx_sync_offset = 0x03; //0x02;           //0x03;
	atm_phy_cfg.cell_Rx_sync_loss_timer = 500;
	atm_phy_cfg.Rx_HEC_check_timer = 100;
	atm_phy_cfg.Rx_bad_HEC_timer = 100;//1000;
	atm_phy_cfg.Rx_max_bad_HEC_count = 1;//10;
	atm_phy_cfg.number_cells_Rx_discard = 1;

/*
PHY_TX_BYTE_COUNT_STAT
PHY_RX_BYTE_COUNT_STAT
PHY_TX_THROUGHPUT_STAT
PHY_RX_THROUGHPUT_STAT
PHY_INCL_UNDERRUN_TX_THRUPUT
PHY_INCL_DISC_RX_THRUPUT
*/
	atm_phy_cfg.statistics_options = PHY_TX_BYTE_COUNT_STAT | PHY_RX_BYTE_COUNT_STAT | 
		PHY_TX_THROUGHPUT_STAT | PHY_RX_THROUGHPUT_STAT |
		PHY_INCL_UNDERRUN_TX_THRUPUT | PHY_INCL_DISC_RX_THRUPUT;

}

atm_set_configuration_impl::~atm_set_configuration_impl()
{
	DbgPrint1(DBG_ATM_PHY_CFG, "atm_set_configuration_impl::~atm_set_configuration_impl()\n");

}

void atm_set_configuration_impl::setDefault_ATM_CfgValues()
{
	DbgPrint1(DBG_ATM_PHY_CFG, "atm_set_configuration_impl::setDefault_ATM_CfgValues()()\n");

	//Card number may be passed as command line argument.
	//If so, update configuration structure.

	DbgPrint2(DBG_ATM_PHY_CFG, "setDefault_ATM_CfgValues() : app_command_line_empty : %d\n", 
		app_command_line_empty);

/*
PHY_INTERFACE_CPE
PHY_INTERFACE_SWITCH
PHY_UNI
PHY_NNI
*/

	if(app_command_line_empty == FALSE)
	{	
		if(card_number == CARD_DCE)
		{	
			//clocking internal
			atm_phy_cfg.baud_rate = 1500; //1500000;			
			//V.35
			atm_phy_cfg.line_config_options = PHY_INTERFACE_LEVEL_V35;

			atm_phy_cfg.ATM_interface = PHY_UNI | PHY_INTERFACE_SWITCH;

		}else if(card_number == CARD_DTE)
		{	
			//clocking external
			atm_phy_cfg.baud_rate = 0;
			//V.35
			atm_phy_cfg.line_config_options = PHY_INTERFACE_LEVEL_V35;

			atm_phy_cfg.ATM_interface = PHY_UNI | PHY_INTERFACE_CPE;
		}
	}
}

void atm_set_configuration_impl::OnOkClicked()
{
	DbgPrint1(DBG_ATM_PHY_CFG, "atm_set_configuration_impl::OnOkClicked()\n");
	
	unsigned short i_temp;
	unsigned long l_temp;
	bool ok;

	//create a backup copy and do all the temporary changes in it.
	//if all checks passed, copy the new configuration into
	//the real congig structure.
	memcpy(&atm_phy_cfg_back, &atm_phy_cfg,	sizeof(PHY_CONFIGURATION_STRUCT));

	//
	// copy values from visual controls to cfg structure
	//

	// baud rate
	//i_temp = (unsigned short)get_edit_box_value(lineEdit_BaudRate, &ok);

	l_temp = (lineEdit_BaudRate->text()).toULong(&ok, 10);

	if(ok == false || l_temp < 0 || l_temp > 2750000)
	{	QMessageBox::warning(0,
			"Invalid Configuration Parameter",
			"Baud Rate should be between 0 an 2750000 bps.");
		return;
	}else
		atm_phy_cfg_back.baud_rate = l_temp;
	/////////////////////////////////////////////////////////
	if(radioButton_V35->isChecked() == true)
		atm_phy_cfg_back.line_config_options = PHY_INTERFACE_LEVEL_V35;
	else
		atm_phy_cfg_back.line_config_options = PHY_INTERFACE_LEVEL_RS232;
	/////////////////////////////////////////////////////////
	if(radioButton_UNI->isChecked() == true)
	{	atm_phy_cfg_back.ATM_interface = PHY_UNI;
	}else if(radioButton_NNI->isChecked() == true)
	{	atm_phy_cfg_back.ATM_interface = PHY_NNI;
	}
	/////////////////////////////////////////////////////////
	if(radioButton_CPE->isChecked() == true)
	{	atm_phy_cfg_back.ATM_interface |= PHY_INTERFACE_CPE;
		atm_phy_cfg_back.ATM_interface &= (~PHY_INTERFACE_SWITCH);

	}else if(radioButton_Switch->isChecked() == true)
	{	atm_phy_cfg_back.ATM_interface &= (~PHY_INTERFACE_CPE);
		atm_phy_cfg_back.ATM_interface |= PHY_INTERFACE_SWITCH;
	}
	/////////////////////////////////////////////////////////
	if(checkBox_ManualRxSynch->isChecked() == true)
	{	atm_phy_cfg_back.protocol_options = PHY_MANUAL_RX_SYNC;
	}else if(checkBox_ManualRxSynch->isChecked() == false)
	{	atm_phy_cfg_back.protocol_options = 0;//PHY_MANUAL_RX_SYNC;
	}
	/////////////////////////////////////////////////////////
	// Tx options
	/////////////////////////////////////////////////////////
	i_temp = (unsigned short)get_int_hex_value_from_line_edit_box(
										lineEdit_tx_underrun_cell_gfc, 
										&ok);
	if(ok == false)
	{	QMessageBox::warning(0,
			"Invalid Configuration Parameter",
			"Tx_underrun_cell_GFC.");
		return;
	}else
		atm_phy_cfg_back.Tx_underrun_cell_GFC = i_temp;
	/////////////////////////////////////////////////////////
	i_temp = (unsigned short)get_int_hex_value_from_line_edit_box(
										lineEdit_tx_underrun_cell_pt, 
										&ok);
	if(ok == false)
	{	QMessageBox::warning(0,
			"Invalid Configuration Parameter",
			"Tx_underrun_cell_PT.");
		return;
	}else
		atm_phy_cfg_back.Tx_underrun_cell_PT = i_temp;
	/////////////////////////////////////////////////////////
	i_temp = (unsigned short)get_int_hex_value_from_line_edit_box(
										lineEdit_tx_underrun_cell_clp, 
										&ok);
	if(ok == false)
	{	QMessageBox::warning(0,
			"Invalid Configuration Parameter",
			"Tx_underrun_cell_CLP.");
		return;
	}else
		atm_phy_cfg_back.Tx_underrun_cell_CLP = i_temp;
	/////////////////////////////////////////////////////////
	i_temp = (unsigned short)get_int_hex_value_from_line_edit_box(
										lineEdit_tx_underrun_cell_payload, 
										&ok);
	if(ok == false)
	{	QMessageBox::warning(0,
			"Invalid Configuration Parameter",
			"Tx_underrun_cell_payload.");
		return;
	}else
		atm_phy_cfg_back.Tx_underrun_cell_payload = i_temp;
	/////////////////////////////////////////////////////////
	// Rx options
	/////////////////////////////////////////////////////////
	i_temp = (unsigned short)get_edit_box_value(lineEdit_rx_hunt_timer, &ok);
	if(ok == false)
	{	QMessageBox::warning(0,
			"Invalid Configuration Parameter",
			"Rx_hunt_timer.");
		return;
	}else
		atm_phy_cfg_back.Rx_hunt_timer = i_temp;
	/////////////////////////////////////////////////////////
	i_temp = (unsigned short)get_edit_box_value(lineEdit_rx_cell_synch_loss_timer, &ok);
	if(ok == false)
	{	QMessageBox::warning(0,
			"Invalid Configuration Parameter",
			"cell_Rx_sync_loss_timer.");
		return;
	}else
		atm_phy_cfg_back.cell_Rx_sync_loss_timer = i_temp;
	/////////////////////////////////////////////////////////
	i_temp = (unsigned short)get_int_hex_value_from_line_edit_box(
										lineEdit_rx_synch_bytes, 
										&ok);
	if(ok == false)
	{	QMessageBox::warning(0,
			"Invalid Configuration Parameter",
			"Rx_sync_bytes.");
		return;
	}else
		atm_phy_cfg_back.Rx_sync_bytes = i_temp;
	/////////////////////////////////////////////////////////
	i_temp = (unsigned short)get_int_hex_value_from_line_edit_box(
										lineEdit_rx_synch_offset, 
										&ok);
	if(ok == false)
	{	QMessageBox::warning(0,
			"Invalid Configuration Parameter",
			"Rx_sync_offset.");
		return;
	}else
		atm_phy_cfg_back.Rx_sync_offset = i_temp;
	/////////////////////////////////////////////////////////

	memcpy(&atm_phy_cfg, &atm_phy_cfg_back, sizeof(PHY_CONFIGURATION_STRUCT));

	done(0);
}

void atm_set_configuration_impl::OnCancelClicked()
{
	DbgPrint1(DBG_ATM_PHY_CFG, "atm_set_configuration_impl::OnCancelClicked()\n");
	done(1);
}

void atm_set_configuration_impl::closeEvent(QCloseEvent * e)
{
	DbgPrint1(DBG_ATM_PHY_CFG, "atm_set_configuration_impl::closeEvent()\n");
	setResult(1);
	e->accept();
}

void atm_set_configuration_impl::showEvent(QShowEvent * e)
{
	DbgPrint1(DBG_ATM_PHY_CFG, "atm_set_configuration_impl::showEvent()\n");
  
	set_widget_position_on_screen(this);

	//
	// copy values from cfg structure to visual controls
	//
	//
	// baud rate
	s_tmp.sprintf("%d", atm_phy_cfg.baud_rate);
	lineEdit_BaudRate->setText(s_tmp);
	/////////////////////////////////////////////////////////
	if(atm_phy_cfg.line_config_options == PHY_INTERFACE_LEVEL_V35)
	{	radioButton_V35->setChecked(true);
		radioButton_RS232->setChecked(false);
	}else if(atm_phy_cfg.line_config_options == PHY_INTERFACE_LEVEL_RS232)
	{	radioButton_V35->setChecked(false);
		radioButton_RS232->setChecked(true);
	}
	/////////////////////////////////////////////////////////
	if(atm_phy_cfg.ATM_interface & PHY_UNI)
	{	radioButton_UNI->setChecked(true);
	}else if(atm_phy_cfg.ATM_interface & PHY_NNI)
	{	radioButton_NNI->setChecked(true);
	}
	/////////////////////////////////////////////////////////
	if(atm_phy_cfg.ATM_interface & PHY_INTERFACE_CPE)
	{	radioButton_CPE->setChecked(true);
		radioButton_Switch->setChecked(false);
	}else if(atm_phy_cfg.ATM_interface & PHY_INTERFACE_SWITCH)
	{	radioButton_CPE->setChecked(false);
		radioButton_Switch->setChecked(true);

	}
	/////////////////////////////////////////////////////////
	if(atm_phy_cfg.protocol_options & PHY_MANUAL_RX_SYNC)
	{	checkBox_ManualRxSynch->setChecked(true);
	}else 
	{	checkBox_ManualRxSynch->setChecked(false);
	}
	/////////////////////////////////////////////////////////
	// Tx options
	/////////////////////////////////////////////////////////
	s_tmp.sprintf("%02X", atm_phy_cfg.Tx_underrun_cell_GFC);
	lineEdit_tx_underrun_cell_gfc->setText(s_tmp);
	/////////////////////////////////////////////////////////
	s_tmp.sprintf("%02X", atm_phy_cfg.Tx_underrun_cell_PT);
	lineEdit_tx_underrun_cell_pt->setText(s_tmp);
	/////////////////////////////////////////////////////////
	s_tmp.sprintf("%02X", atm_phy_cfg.Tx_underrun_cell_CLP );
	lineEdit_tx_underrun_cell_clp->setText(s_tmp);
	/////////////////////////////////////////////////////////
	s_tmp.sprintf("%02X", atm_phy_cfg.Tx_underrun_cell_payload );
	lineEdit_tx_underrun_cell_payload->setText(s_tmp);

	/////////////////////////////////////////////////////////
	// Rx options
	/////////////////////////////////////////////////////////
	s_tmp.sprintf("%d", atm_phy_cfg.Rx_hunt_timer );
	lineEdit_rx_hunt_timer->setText(s_tmp);
	/////////////////////////////////////////////////////////
	s_tmp.sprintf("%d", atm_phy_cfg.cell_Rx_sync_loss_timer );
	lineEdit_rx_cell_synch_loss_timer->setText(s_tmp);
	/////////////////////////////////////////////////////////
	s_tmp.sprintf("%04X", atm_phy_cfg.Rx_sync_bytes );
	lineEdit_rx_synch_bytes->setText(s_tmp);
	/////////////////////////////////////////////////////////
	s_tmp.sprintf("%04X", atm_phy_cfg.Rx_sync_offset );
	lineEdit_rx_synch_offset->setText(s_tmp);
	/////////////////////////////////////////////////////////
}

