// atm_vpi_vci_status_impl.cpp: implementation of the atm_vpi_vci_status_impl class.
//
//////////////////////////////////////////////////////////////////////

#include "MainWizardImpl.h"

extern TesterWizardImpl * globalTesterWizPtr;

#define DBG_ATMM_VPI_VCI_STATUS	1

#define VPI_COL		0
#define VCI_COL		1
#define STATUS_COL	2

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

atm_vpi_vci_status_impl::atm_vpi_vci_status_impl()
{
	window_closed = true;
	set_widget_caption(this);

	//remove all columns
	for(int i = listView_VPI_VCI->columns() - 1; i >= 0; i--)
	{	listView_VPI_VCI->removeColumn(i);
	}


	//add needed columns
	listView_VPI_VCI->setSorting(-1);
	listView_VPI_VCI->addColumn("  VPI  ");		//0
	listView_VPI_VCI->setColumnAlignment(VPI_COL, Qt::AlignHCenter);
	listView_VPI_VCI->addColumn("  VCI  ");		//1
	listView_VPI_VCI->setColumnAlignment(VCI_COL, Qt::AlignHCenter);
	listView_VPI_VCI->addColumn("Status");	//2
	listView_VPI_VCI->setColumnAlignment(STATUS_COL, Qt::AlignHCenter);

}

atm_vpi_vci_status_impl::~atm_vpi_vci_status_impl()
{

}

void atm_vpi_vci_status_impl::timerEvent(QTimerEvent * e)
{
	if(window_closed == true)
	{	return;
	}

	globalTesterWizPtr->atm_api_obj.atm_cb.command = ATMAL_READ_PVC_STATUS;
	globalTesterWizPtr->atm_api_obj.atm_cb.command_qualifier = CMD_QUAL_RD_STAT_ALL_PVCS;
	globalTesterWizPtr->atm_api_obj.atm_cb.buffer_length = 0;
	globalTesterWizPtr->atm_api_obj.atm_cb.return_code = Z80_TIMEOUT_ERROR;

	globalTesterWizPtr->atm_do_command(
		&globalTesterWizPtr->atm_api_obj.atm_cb, globalTesterWizPtr->m_cCard
		);
	
	if(globalTesterWizPtr->atm_api_obj.atm_cb.return_code)
	{	
		qstr_err_message.sprintf("ATMAL_READ_PVC_STATUS failed!!. Return code 0x%02X",
			globalTesterWizPtr->atm_api_obj.atm_cb.return_code );
		QMessageBox::warning(this, "Command Error", qstr_err_message );

		window_closed = true;
		return;
	}

/*
	DbgPrint2(DBG_ATMM_VPI_VCI_STATUS, "ATMAL_LIST_ADDED_PVCS : buffer_length : %u\n",
		globalTesterWizPtr->atm_api_obj.atm_cb.buffer_length
		);
*/
	int buffer_len_backup = globalTesterWizPtr->atm_api_obj.atm_cb.buffer_length;

	if(buffer_len_backup)
	{
		//the list is not empty
		memcpy(
			new_list_of_added_pvcs,
			globalTesterWizPtr->atm_api_obj.atm_cb.data,
			globalTesterWizPtr->atm_api_obj.atm_cb.buffer_length
			);
	}else
	{	memset(new_list_of_added_pvcs, 0x00, SIZEOF_MB_DATA_BFR);
	}


	if(memcmp(	new_list_of_added_pvcs,
				old_list_of_added_pvcs,
				sizeof(new_list_of_added_pvcs)
				)
	  )
	{
		memcpy(
			old_list_of_added_pvcs,
			globalTesterWizPtr->atm_api_obj.atm_cb.data,
			globalTesterWizPtr->atm_api_obj.atm_cb.buffer_length
			);

		listView_VPI_VCI->clear();

		for(int i=0; i < buffer_len_backup;	i += sizeof(ATMAL_READ_PVC_STATUS_STRUCT))
		{
			update_status_list(i);
		}

	}

	if(!buffer_len_backup)
	{
		initialize();
	}
}

void atm_vpi_vci_status_impl::showEvent(QShowEvent * ev)
{
	set_widget_position_on_screen(this);
}

bool atm_vpi_vci_status_impl::initialize()
{
	memset(old_list_of_added_pvcs, 0x00, SIZEOF_MB_DATA_BFR);
	memset(new_list_of_added_pvcs, 0x00, SIZEOF_MB_DATA_BFR);

	window_closed = false;
	return TRUE;
}

void atm_vpi_vci_status_impl::closeEvent(QCloseEvent * e)
{
	window_closed = true;

	globalTesterWizPtr->atm_atmal_level_status_and_stats_menu->
		setItemChecked (
			globalTesterWizPtr->atm_atmal_level_status_and_stats_menu->idAt(0), FALSE );
	
	e->accept();
}

void atm_vpi_vci_status_impl::update_status_list(int i)
{
	//DbgPrint2(DBG_ATMM_VPI_VCI_STATUS, "update_status_list() i : %u\n", i);

	ATMAL_READ_PVC_STATUS_STRUCT * pvc_status_ptr = 
		(ATMAL_READ_PVC_STATUS_STRUCT *)&new_list_of_added_pvcs[i];

	qstr_vpi.sprintf("%u", pvc_status_ptr->VPI);
	qstr_vci.sprintf("%u", pvc_status_ptr->VCI);

	DbgPrint2(DBG_ATMM_VPI_VCI_STATUS, "pvc status : 0x%X\n", pvc_status_ptr->status);

	if(!(pvc_status_ptr->status & PVC_STAT_ALLOC))
	{	qstr_status = "Not Added";
		goto show_status;
	}
	
	if(pvc_status_ptr->status & PVC_STAT_CONNECTED)
	{	qstr_status = "Connected";
	}	else
	{	qstr_status = "Disconnected";
	}

show_status:

	list_view_item = 
		new QListViewItem(	listView_VPI_VCI,
							listView_VPI_VCI->lastItem()
							);

	list_view_item->setText(VPI_COL, qstr_vpi);
	list_view_item->setText(VCI_COL, qstr_vci);
	list_view_item->setText(STATUS_COL, qstr_status);

	//LIST_VIEW_ADDSTR(listView_VPI_VCI, NULL);

}

