/******************************************************************************
 *                                                                            *
 * AALAPI.H - the 'C' header file for the Sangoma S508/S514 AAL ATM code API. *
 *                                                                            *
 ******************************************************************************
*/

#ifndef AALAPI_H
#define AALAPI_H

#pragma pack(1)


/* ----------------------------------------------------------------------------
 *                        Interface commands
 * --------------------------------------------------------------------------*/

#define ATMAL_READ_CODE_VERSION				0x40	/* read the ATM/AAL code version */
#define ATMAL_READ_EXCEPTION_CONDITION		0x41	/* read an ATM/AAL exception condition from the adapter */
#define ATMAL_SET_CONFIGURATION				0x42	/* set the ATM/AAL configuration */
#define ATMAL_READ_CONFIGURATION				0x43	/* read the AAL configuration */
/* ??????ffff do we actually need enable/disable SAR?????? */
#define ATMAL_ENABLE_SAR						0x44	/* enable cell segmentation and reassembly */
#define ATMAL_DISABLE_SAR						0x45	/* disable cell segmentation and reassembly */
#define ATMAL_READ_STATUS						0x46	/* read the ATM/AAL status */
#define ATMAL_READ_OPERATIONAL_STATS		0x47	/* retrieve the ATM/AAL operational statistics */
#define ATMAL_FLUSH_OPERATIONAL_STATS		0x48	/* flush the ATM/AAL operational statistics */
#define ATMAL_FLUSH_TX_BUFFERS				0x4A	/* flush any queued AAL transmit buffers */
#define ATMAL_FLUSH_RX_BUFFERS				0x4B	/* flush any queued AAL receiev buffers */
#define ATMAL_ADD_PVC							0x50
#define ATMAL_DELETE_PVC						0x51
#define ATMAL_CONNECT_PVC						0x52
#define ATMAL_DISCONNECT_PVC					0x53
#define ATMAL_LIST_ADDED_PVCS					0x54
#define ATMAL_LIST_CONNECTED_PVCS			0x55
#define ATMAL_READ_PVC_STATUS					0x56
#define ATMAL_OAM_ACT_DEACT					0x5A
#define ATMAL_OAM_TRANSMIT_LOOPBACK			0x5B
#define ATMAL_OAM_READ_LOOPBACK_RESP		0x5C
#define ATMAL_SET_INTERRUPT_TRIGGERS		0x60	/* set the ATM/AAL application interrupt triggers */
#define ATMAL_READ_INTERRUPT_TRIGGERS		0x61	/* read the ATM/AAL application interrupt trigger configuration */



/* ----------------------------------------------------------------------------
 *                     Return codes from interface commands
 * --------------------------------------------------------------------------*/
#define ATMAL_NO_EXCEP_COND_TO_REPORT			0x41	/* there is no ATM/AAL exception condition to report */
#define ATMAL_SAR_DISABLED						0x41	/* the SAR is not currently enabled */
#define ATMAL_SAR_ENABLED							0x41	/* the SAR is currently enabled */
#define ATMAL_CFG_BEFORE_ADD_PVC				0x41
#define ATMAL_DISABLE_COMMS_BEFORE_CFG		0x41	/* communications must be disabled before setting the configuration */
#define ATMAL_CFG_BEFORE_SAR_ENABLED			0x42	/* perform a AAL_SET_CONFIGURATION before enabling the SAR */
#define ATMAL_LGTH_CFG_DATA_INVALID			0x42	/* the length of the passed configuration data is invalid */
#define ATMAL_INVALID_CFG_DATA					0x43	/* the passed configuration data is invalid */
#define ATMAL_PHY_NOT_CFG							0x44	/* the PHY configuration must be set before performing an */
																/* AAL_SET_CONFIGURATION command */
#define ATMAL_NO_CONNECTIONS_EXCEEDED			0x45
#define ATMAL_PHY_CFG_INVALID					0x45	/* the PHY configuration is invalid for AAL operation */
#define ATMAL_PVC_ADDED							0x46
#define ATMAL_PVC_NOT_ADDED					0x46
#define ATMAL_PVC_CONNECTED					0x47
#define ATMAL_PVC_DISCONNECTED				0x47
#define ATMAL_OAM_INVALID_STATE				0x50
#define ATMAL_OAM_WAIT_LPBK_RESP				0x51
#define ATMAL_INVALID_COMMAND					0xAF	/* the defined interface command is invalid */



/* ----------------------------------------------------------------------------
 *          Constants for the ATMAL_READ_EXCEPTION_CONDITION command
 * --------------------------------------------------------------------------*/

/* the structure returned on a ATMAL_READ_EXCEPTION_CONDITION command */
typedef struct {
	unsigned short exception;
} ATMAL_READ_EXCEP_COND_STRUCT;

/* settings for the 'command_qualifier' in the mailbox structure */
#define ATMAL_EXCEP_GLOBAL						0x00		/* the exception condition is global */
#define ATMAL_EXCEP_VC							0x01		/* the exception condition is for a specified VC */

/* bit settings for the 'status' flag (ATMAL_EXCEP_GLOBAL) */
#define ATMAL_EXCEP_TX_INVALID_VC		0x0001		/* User Information transmitted by the application on invalid VC */
#define ATMAL_EXCEP_RX_INVALID_VC		0x0010		/* cells received on an invalid VC */

/* bit settings for the 'status' flag (ATMAL_EXCEP_VC) */
#define ATMAL_EXCEP_VC_AUTO_ADD			0x0001		/* the VC has automatically added */
#define ATMAL_EXCEP_VC_CONNECTED			0x0002		/* the VC is connected */
#define ATMAL_EXCEP_VC_DISCONNECTED		0x0004		/* the VC is disconnected */
#define ATMAL_EXCEP_OAM_AIS_START		0x0010		/* the VC has entered the OAM AIS state */
#define ATMAL_EXCEP_OAM_AIS_END			0x0020		/* the VC has exited the OAM AIS state */
#define ATMAL_EXCEP_RX_LGTH_ERR			0x0100		/* the received User Information block exceeded the maximum length */
#define ATMAL_EXCEP_INVALID_CRC			0x0200
#define ATMAL_EXCEP_RX_DISCARD			0x0400		/* an incoming block of data was discarded */
#define ATMAL_EXCEP_RX_PDU_TIMEOUT		0x0800		/* a timeout occurred waiting for a PDU */
#define ATMAL_EXCEP_TX_LGTH_ERR			0x1000		/* User Information transmitted by the application was of invalid length */





/* ----------------------------------------------------------------------------
 *  Constants for the ATMAL_SET_CONFIGURATION/ATMAL_READ_CONFIGURATION command
 * ---------------------------------------------------------------------------*/

/* the ATM/AAL configuration structure */
typedef struct {
	unsigned short API_options;						/* API options */
	unsigned short protocol_options;					/* protocol options */
	unsigned short buffer_options;					/* Tx/Rx buffer options */
	unsigned short max_length_Tx_user_info;		/* the maximum length of transmitted User Information */
	unsigned short max_length_Rx_user_info;		/* the maximum length of received User Information */
	unsigned char AAL_PAD_char;						/* the PAD character used in formatting PDUs */
	unsigned short Rx_inter_PDU_timeout;			/* the receive inter-PDU timeout value */
	unsigned short statistics_options;				/* operational statistics options */
	unsigned long ptr_shared_mem_info_struct;		/* a pointer to the shared memory area information structure */
	unsigned long ptr_AAL_Tx_control_struct;		/* a pointer to the AAL transmit control structure */
	unsigned long ptr_AAL_Rx_control_struct;		/* a pointer to the AAL receive control structure */
} ATMAL_CONFIGURATION_STRUCT;

/* bit settings for the 'buffer_options' */
#define AAL_TX_ONLY								0x0001 /* transmit only (no reception) */
#define AAL_RX_ONLY								0x0002 /* receive only (no transmission) */
#define AAL_SINGLE_TX_BUFFER					0x0010 /* configure a single transmit buffer */

/* permitted minimum and maximum values for setting the AAL configuration */
#define AAL_MIN_LENGTH_TX_USER_INFO			1	  	/* the minimum length of transmitted User Information */
#define AAL_MAX_LENGTH_TX_USER_INFO			32768	/* the maximum length of transmitted User Information */
#define AAL_MIN_LENGTH_RX_USER_INFO			1	  	/* the minimum length of received User Information */
#define AAL_MAX_LENGTH_RX_USER_INFO			32768	/* the maximum length of received User Information */
#define ATMAL_MIN_RX_INTER_PDU_TIMEOUT		0		/* the minimum Rx inter-PDU timeout */
#define ATMAL_MAX_RX_INTER_PDU_TIMEOUT		6000	/* the maximum Rx inter-PDU timeout */



/* ----------------------------------------------------------------------------
 *         Constants for the ATMAL_ADD_PVC/ATMAL_DELETE_PVC command
 * --------------------------------------------------------------------------*/

/* the structure used for adding a PVC */
typedef struct {
	unsigned short VC_options;							/* Virtual Channel options */
	unsigned short OAM_CC_Tx_timer;					/* OAM Continuity Check transmit timer */
	unsigned short OAM_CC_Rx_timer;					/* OAM Continuity Check receive timer */
	unsigned short OAM_CC_Rx_timeout_counter;		/* OAM Continuity Check receive disconnection counter */
	unsigned short OAM_AIS_Tx_timer;					/* OAM Alarm Indication Signal transmit timer */
	unsigned short OAM_AIS_state_Rx_timeout;		/* OAM Alarm Indication Signal state receive timeout */
	unsigned short OAM_Loopback_Tx_timer;			/* OAM Loopback transmit timer */
	unsigned short OAM_Loopback_Rx_timeout;		/* OAM Loopback receive timeout */
	unsigned short OAM_Loopback_Rx_timeout_counter;	/* OAM Loopback timeout counter */

} ATMAL_ADD_PVC_STRUCT;

/* bit settings for the 'VC_options' */
#define VC_OPT_STATE_INDEP_CONT_CHECK	0x0001	/* the state of the VC is independant of OAM Continuity Check PDUs */
#define VC_OPT_STATE_INDEP_LPBK_RESP	0x0002	/* the state of the VC is independant of OAM Loopback responses */
#define VC_OPT_MANUAL_CC_ACTIVATION		0x0010	/* manual OAM Continuity Check activation */

#define MIN_OAM_CC_TIMER					0	  	  	/* the minimum OAM Cell Continuity Check timer */
#define MAX_OAM_CC_TIMER					6000 	  	/* the maximum OAM Cell Continuity Check timer */
#define MIN_OAM_CC_TIMEOUT_COUNT			1	  	  	/* the minimum OAM Cell Continuity Check timeout counter */
#define MAX_OAM_CC_TIMEOUT_COUNT			100  	  	/* the maximum OAM Cell Continuity Check timeout counter */
#define MIN_OAM_AIS_TIMER					0	  	  	/* the minimum OAM Alarm Indication Signal timer */
#define MAX_OAM_AIS_TIMER					6000 	  	/* the maximum OAM Alarm Indication Signal timer */
#define MIN_OAM_AIS_STATE_RX_TIMEOUT	1	  	  	/* the minimum OAM Alarm Indication Signal state receive timer */
#define MAX_OAM_AIS_STATE_RX_TIMEOUT	6000  	/* the maximum OAM Alarm Indication Signal state receive timer */
#define MIN_OAM_LPBK_TIMER					0	  	  	/* the minimum OAM Cell Loopback timer */
#define MAX_OAM_LPBK_TIMER					6000 	  	/* the maximum OAM Cell Loopback timer */
#define MIN_OAM_LPBK_TIMEOUT				0	  	  	/* the minimum OAM Cell Loopback timeout */
#define MAX_OAM_LPBK_TIMEOUT				6000 	  	/* the maximum OAM Cell Loopback timeout */
#define MIN_OAM_LPBK_TIMEOUT_COUNT		1	  	  	/* the minimum OAM Cell Loopback timeout counter */
#define MAX_OAM_LPBK_TIMEOUT_COUNT		100  	  	/* the maximum OAM Cell Loopback timeout counter */



/* ----------------------------------------------------------------------------
 *         Constants for the ATMAL_LIST_ADDED_PVCS command
 * --------------------------------------------------------------------------*/

/* the structure used for listing added PVCs */
typedef struct {
	unsigned short VPI;									/* VPI (Virtual Path Identifier) */
	unsigned short VCI;									/* VCI (Virtual Channel Identifier) */
} ATMAL_LIST_ADDED_PVC_STRUCT;



/* ----------------------------------------------------------------------------
 *         Constants for the ATMAL_LIST_CONNECTED_PVCS command
 * --------------------------------------------------------------------------*/

/* the structure used for listing connected PVCs */
typedef struct {
	unsigned short VPI;									/* VPI (Virtual Path Identifier) */
	unsigned short VCI;									/* VCI (Virtual Channel Identifier) */
} ATMAL_LIST_CONN_PVC_STRUCT;




/* ----------------------------------------------------------------------------
 *         Constants for the ATMAL_READ_PVC_STATUS command
 * --------------------------------------------------------------------------*/

/* the structure used for reading the PVC status */
typedef struct {
	unsigned short VPI;									/* VPI (Virtual Path Identifier) */
	unsigned short VCI;									/* VCI (Virtual Channel Identifier) */
	unsigned char status;								/* PVC status */
	unsigned short no_user_info_Tx_blocks_queued;/* number of User Information blocks queued for transmission */
} ATMAL_READ_PVC_STATUS_STRUCT;

/* settings for the 'command_qualifier' in the mailbox structure */
#define CMD_QUAL_RD_STAT_SINGLE_PVC		0x00		/* read the status of a single PVC */
#define CMD_QUAL_RD_STAT_ALL_PVCS		0x01		/* read the status of all added PVCs */

/* bit settings for the 'status' flag */
#define PVC_STAT_ALLOC						0x01		/* the PVC has been allocated */
#define PVC_STAT_CONNECTED					0x02		/* the PVC is connected */



/* ----------------------------------------------------------------------------
 *         Constants for the ATMAL_OAM_ACT_DEACT command
 * --------------------------------------------------------------------------*/

/* settings for the 'command_qualifier' in the mailbox structure */
#define ACTIVATE_OAM_SIGNAL				0x80
#define DEACTIVATE_OAM_SIGNAL				0x40
#define OAM_PERFORMANCE_MONITORING		0x01
#define OAM_CONTINUITY_CHECK				0x02
#define OAM_FWD_PERFORMANCE_MONITORING	0x04





/* ----------------------------------------------------------------------------
 *                 Constants for transmission of User Information
 * --------------------------------------------------------------------------*/

/* the control structure for transmitting User Information */
typedef struct {
	unsigned short number_Tx_status_els;			/* number of transmit status elements */
	unsigned char reserved_0[14];						/* reserved for internal use */
	unsigned char opp_flag;								/* opp flag */
	unsigned char reserved_1[3];						/* reserved for internal use */
	unsigned long ptr_Tx_status_el;					/* pointer to the transmit status element to be used */
} AAL_TX_CONTROL_STRUCT;

/* the User Information transmit status element structure */
typedef struct {
	unsigned short user_info_length;					/* length of the User Information to be transmitted */
	unsigned char misc_Tx_bits;						/* miscellaneous Tx bits */
	unsigned short GFC_VPI;								/* GFC (Generic Flow Control)/VPI (Virtual Path Identifier) */
	unsigned short VCI;									/* VCI (Virtual Channel Identifier) */
	unsigned char PT;										/* PT (Payload Type) */
	unsigned char CLP;									/* CLP (Cell Loss Priority) */
	unsigned char reserved[3];							/* reserved for internal use */
	unsigned long ptr_data_bfr;						/* pointer to the data area */
} AAL_TX_STATUS_EL_STRUCT;



/* ----------------------------------------------------------------------------
 *                 Constants for reception of User Information
 * --------------------------------------------------------------------------*/

/* the control structure for receiving User Information */
typedef struct {
	unsigned short number_Rx_status_els;			/* number of receive status elements */
	unsigned char reserved_0[14];						/* reserved for internal use */
	unsigned char opp_flag;								/* opp flag */
	unsigned char reserved_1[3];						/* reserved for internal use */
	unsigned long ptr_Rx_status_el;					/* pointer to the receive status element to be used */
} AAL_RX_CONTROL_STRUCT;

/* the User Information receive status element structure */
typedef struct {
	unsigned short user_info_length;					/* length of the received User Information */
	unsigned char misc_Rx_bits;						/* miscellaneous Rx bits */
	unsigned short GFC_VPI;								/* GFC (Generic Flow Control)/VPI (Virtual Path Identifier) */
	unsigned short VCI;									/* VCI (Virtual Channel Identifier) */
	unsigned char PT;										/* PT (Payload Type) */
	unsigned char CLP;									/* CLP (Cell Loss Priority) */
	unsigned short time_stamp;							/* receive time stamp */
	unsigned char reserved;
	unsigned long ptr_data_bfr;						/* pointer to the data area */
} AAL_RX_STATUS_EL_STRUCT;

/* settings for the 'misc_Rx_bits' */
#define AAL_RX_LENGTH_ERROR						0x04	/* the received User Information is of invalid length */
#define AAL_RX_CRC_ERROR							0x08	/* the received User Information has a CRC error */
#define AAL_RX_PDU_TIMEOUT							0x10	/* a timeout occurred waiting for a PDU */


typedef struct {
	unsigned long user_info_Tx_invalid_VC_count;	/* number of User Information blocks transmitted User Information transmitted on an invalid VC */
	unsigned long cells_Rx_invalid_VC_count;		/* number of cells received on an invalid VC */
} ATMAL_GLOB_OP_STATS_STRUCT;

typedef struct {

	/* transmission statistics */
	unsigned long user_info_Tx_count;				/* number of User Information blocks transmitted */
	unsigned long user_info_bytes_Tx_count;		/* number of User Information bytes transmitted */
	unsigned long SAR_PDU_Tx_count;					/* number of SAR PDUs transmitted */
	unsigned long Tx_length_error_discard_count;	/* number of User Information blocks discarded (length error) */

	/* reception statistics */
	unsigned long user_info_Rx_count;				/* number of User Information blocks received */
	unsigned long user_info_bytes_Rx_count;		/* number of User Information bytes received */
	unsigned long SAR_PDU_Rx_count;					/* number of SAR PDUs received */
	unsigned long Rx_user_info_discard_count;		/* number of incoming User Information blocks discarded */
	unsigned long Rx_length_error_count;			/* number of incoming User Information blocks of invalid length */
	unsigned long Rx_bad_CRC_count;					/* number of User Information blocks received with a bad CRC */
	unsigned long Rx_PDU_timeout_count;				/* number of Rx PDU timeouts */

	/* OAM statistics */
	unsigned short OAM_AIS_Tx_count;					/* OAM Alarm Indication Signal transmit counter */
	unsigned short OAM_AIS_Rx_count;					/* OAM Alarm Indication Signal receive counter */
	unsigned short OAM_RDI_Tx_count;					/* OAM Remote Defect Indication transmit counter */
	unsigned short OAM_RDI_Rx_count;					/* OAM Remote Defect Indication receive counter */
	unsigned short OAM_CC_Tx_count;					/* OAM Continuity Check transmit counter */
	unsigned short OAM_CC_Rx_count;					/* OAM Continuity Check receive counter */
	unsigned short OAM_CC_Rx_timeout_count;		/* OAM Continuity Check receive timeout counter */
	unsigned short OAM_CC_act_req_Tx_count;		/* OAM Continuity Check activation request transmit counter */
	unsigned short OAM_CC_act_conf_Tx_count;		/* OAM Continuity Check activation confirmation transmit counter */
	unsigned short OAM_CC_act_req_deny_Tx_count;	/* OAM Continuity Check activation request denied transmit counter */
	unsigned short OAM_CC_deact_req_Tx_count;		/* OAM Continuity Check deactivation request transmit counter */
	unsigned short OAM_CC_deact_conf_Tx_count;	/* OAM Continuity Check deactivation confirmation transmit counter */
	unsigned short OAM_CC_deact_req_deny_Tx_count;/* OAM Continuity Check deactivation request denied transmit counter */
	unsigned short OAM_CC_act_req_Rx_count;		/* OAM Continuity Check activation request receive counter */
	unsigned short OAM_CC_act_conf_Rx_count;		/* OAM Continuity Check activation confirmation receive counter */
	unsigned short OAM_CC_act_req_deny_Rx_count;	/* OAM Continuity Check activation request denied receive counter */
	unsigned short OAM_CC_deact_req_Rx_count;		/* OAM Continuity Check deactivation request receive counter */
	unsigned short OAM_CC_deact_conf_Rx_count;	/* OAM Continuity Check deactivation confirmation receive counter */
	unsigned short OAM_CC_deact_req_deny_Rx_count;/* OAM Continuity Check deactivation request denied receive counter */

	unsigned short OAM_Loopback_LI0_Tx_count;		/* OAM Loopback (LI0) transmit counter */
	unsigned short OAM_Loopback_LI1_Tx_count;		/* OAM Loopback (LI1) transmit counter */
	unsigned short OAM_Loopback_LI0_Rx_count;		/* OAM Loopback (LI0) receive counter */
	unsigned short OAM_Loopback_LI1_Rx_count;		/* OAM Loopback (LI1) receive counter */
	unsigned short OAM_Loopback_Rx_timeout_count;/* OAM Loopback receive timeout counter */
} ATMAL_PVC_OP_STATS_STRUCT;



typedef struct {
	union {
		ATMAL_GLOB_OP_STATS_STRUCT GLOB;
		ATMAL_PVC_OP_STATS_STRUCT PVC;
	} OP_STATS;
} ATMAL_OPERATIONAL_STATS_STRUCT;


/* settings for the 'command_qualifier' in the mailbox structure */
#define CMD_QUAL_GLOBAL_OP_STATS			0x00		/* read/flush the global operational statistics */
#define CMD_QUAL_PVC_OP_STATS				0x01		/* read/flush the VC operational statistics */


#pragma pack()


#endif
