/***************************************************************************
     fapi.h  -     Application include file for FAPI functions.
                              -------------------
     begin                : Fri Sep 21 2001
     copyright            : (C) 2001 by DavidR
     email                : drokhvarg@sangoma.com
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

#ifndef FAPI_H
#define FAPI_H

#include "api_base.h"
#include "frelay.h"			//Fr code offsets, commands, structures

#define	FR_MDATALEN			4096
#define	FR_CB_HEADERLEN		15

typedef struct _SDLA_T{
	// RX & TX elements offsets for S514
	ULONG rx_base514, rx_curr514, rx_end514, rx_bfr_base514, rx_bfr_end514;
	
	//INFORMATION_WRITE temporary storage.
	USHORT temp_tx_buffer_length;
	UCHAR  temp_tx_data_buff[FR_MDATALEN];
} SDLA_T, *PSDLA_T;

/********** Control block structure **********/

#pragma pack(1)
typedef struct {
	BYTE command;
	WORD buffer_length;
	BYTE return_code;
	WORD dlci;
	BYTE FECN_BECN_DE_CR_bits;
	WORD rx_frm_disc_int;
	DWORD rx_frm_disc_app;
	BYTE reserved[2];
	BYTE data[FR_MDATALEN];
} FCBLOCK;
#pragma pack()

/**
  *@author DavidR
  */

class Fapi : public api_base
{

public:

	SDLA_T sdla_card;

	// line tracing specific data and functions
	USHORT	num_frames;
	ULONG   curr_trace_addr;
	ULONG	start_trace_addr;
	USHORT  available_buffer_space;

  Fapi();
  ~Fapi();

  /* FAPI will always return 1 */
  int FAPIAlive(void);

  /* Emulate FRIP interrupt call with control block
    returns return_code value in structure
  */
  char FAPIControl(FCBLOCK * block);

  /* returns number of frames pending for read
  */
  unsigned char FAPInDataToRead();

  /* returns 1 if clear to send bit is set, 0 if not
  */
  char FAPIqClearToXmit();

  /* DEBUGGING COMMAND TRACE MESSAGES (wParam values) */
  #define	TRACEMSG_CLOSE	0

  /* returns a byte indicating current modem status as follows:
 	[ 0 0 CTS 0 DCD 0 0 0 ]
    CTS and DCD are high when these corresponding bits are set
  */
  char FAPIGetModemStatus();

  /* Returns zero if no events to report, or the event code.
    The event code corresponds to the return codes 0x10 thru 0x14
    inclusive which would normally be returned on a FAPIControl
    call for any given command - therefore, once retrieved using this
    command, the event is cleared and will not be reported by a
    FAPIControl call in the future
    This is a very fast call, and should be used when polling for
    changes in channel status or DLCI status instead of using
    FAPIControl.  However, these sames codes may be obtained separately
    from a FAPIControl call and those same events will not be returned
    on a FAPIGetEvent call.

    The pertinent info is stored in cb if anything happens, otherwise
    it isn't used.
  */
  char FAPIGetEvent(FCBLOCK *cb);

  /* returns events pending byte:
 	[ 0 MODEM 0 0 0 UNCONF_DLCI DLCI_STAT CHAN_STAT ]
  */
  char FAPIGetEventsPending();

private:
  /* Undocumented exports - use at your own risk */
  /** prototypes of functions which should not be used directly by an application **/

  void SayControlNumber(void);
  void AddStringListBox(char * m);
  void ClearTrace(void);
  char SayReturnResult(char rval, char dval, WORD bl);
  char InternalDoControl(FCBLOCK * block);

  void RetrieveBufferPointers();
  unsigned char ReadData(FCBLOCK * block);
  UCHAR SendFrame(FCBLOCK * block);

  USHORT GetTraceInfo(FCBLOCK * block);
  void SetTraceParam(FCBLOCK * block);

  void show_error (int err);

};

#endif//#ifndef FAPI_H
