/*	$Header$ */


/*
 ************************************************************************
 *                                                                      *
 * OAM.H is the 'C' OAM header file for the Sangoma S508/S514 ATM code. *
 *                                                                      *
 ************************************************************************
*/


/*
 *	$Log$
 * 
 * 
 */

#ifndef OAM_H
#define OAM_H

#pragma pack(1)

/* internal definitions for various interface commands */
#define LGTH_OAM_CODE_VERSION_STRING		0x04		/* the length of the string containing the OAM code version */

#define CRC_10_POLYNOMIAL	0x633

#define LGTH_OAM_AIS_RDI_UNUSED				28
#define LGTH_OAM_AIS_RDI_DEFECT_LOC 		16
#define OAM_AIS_DEFAULT_DEFECT_TYPE	0x6A
#define OAM_AIS_DEFAULT_DEFECT_LOC	0x6A
#define OAM_AIS_RDI_DEFAULT_UNUSED	0x6A

typedef struct {
	UCHAR defect_type;
	UCHAR defect_location[LGTH_OAM_AIS_RDI_DEFECT_LOC];
	UCHAR unused[LGTH_OAM_AIS_RDI_UNUSED];
} OAM_AIS_RDI_STRUCT;


#define LGTH_OAM_LPBK_LPBK_LOC_ID 	12
#define LGTH_OAM_LPBK_SRC_ID			12
#define LGTH_OAM_LPBK_UNUSED			16

typedef struct {
	UCHAR loopback_indication;
	ULONG correlation_tag;
	UCHAR loopback_location_ID[LGTH_OAM_LPBK_LPBK_LOC_ID];
	UCHAR source_ID[LGTH_OAM_LPBK_SRC_ID];
	UCHAR unused[LGTH_OAM_LPBK_UNUSED];
} OAM_LOOPBACK_STRUCT;

#define OAM_LPBK_LOC_ID_ENDPOINT	0xFF
#define OAM_LPBK_DEFAULT_SRC_ID	0xFF
#define OAM_LPBK_DEFAULT_UNUSED	0x6A

#define LGTH_OAM_CC_UNUSED			45
typedef struct {
	UCHAR unused[LGTH_OAM_CC_UNUSED];
} OAM_CONT_CHK_STRUCT;

#define OAM_CC_DEFAULT_UNUSED	0x6A



#define LGTH_OAM_CTRL_UNUSED			42
typedef struct {
	UCHAR MID_DOA;	/* Message ID/Direction of Action */
	UCHAR correlation_tag;
	UCHAR PM_block_size;
	UCHAR unused[LGTH_OAM_CTRL_UNUSED];
} OAM_CONTROL_STRUCT;

#define OAM_CTRL_DEFAULT_UNUSED	0x6A


typedef struct {
	UCHAR OAM_type_func_type;
	union {
		OAM_AIS_RDI_STRUCT AIS_RDI_STR;
		OAM_LOOPBACK_STRUCT LPBK_STR;
		OAM_CONT_CHK_STRUCT CC_STR;
		OAM_CONTROL_STRUCT CTRL_STR;
	} FUNC_SPEC;
	USHORT CRC_10;
} OAM_PAYLOAD_STRUCT;

/* OAM Cell Type (found in the high nibble of the 'OAM_type_func_type' byte) */
#define OAM_TYPE_FAULT_MGMT			0x01
#define OAM_TYPE_PREFORMANCE_MGMT	0x02
#define OAM_TYPE_ACT_DEACT				0x08
#define OAM_TYPE_SYSTEM_MGMT			0x0F

/* Function Type (found in the low nibble of the 'OAM_type_func_type' byte) */
/* for OAM Cell Type Fault Management */
#define FAULT_MGMT_FUNC_TYPE_AIS				0x00
#define FAULT_MGMT_FUNC_TYPE_RDI				0x01
#define FAULT_MGMT_FUNC_TYPE_CONTINUITY	0x04
#define FAULT_MGMT_FUNC_TYPE_LOOPBACK		0x08


/* Function Type (found in the low nibble of the 'OAM_type_func_type' byte) */
/* for OAM Cell Type Activation/Deactivation */
#define ACT_DEACT_FUNC_TYPE_PM				0x00
#define ACT_DEACT_FUNC_TYPE_CC				0x01
#define ACT_DEACT_FUNC_TYPE_FPM				0x02

/* pre-formatted combinations of OAM Cell Type and Function Type */
#define OAM_FAULT_MGMT_AIS				0x10
#define OAM_FAULT_MGMT_RDI				0x11
#define OAM_FAULT_MGMT_CONTINUITY	0x14
#define OAM_FAULT_MGMT_LOOPBACK		0x18
#define OAM_ACT_DEACT_CONTINUITY		0x81


/* definitions for the Message ID and Direction of Activation fields of an OAM Activation/Deactivation cell - note */
/* that we have bit-shifted these definitions as appropriate for ease of use in cell formatting and decioding */
#define OAM_ACT_REQ				0x04
#define OAM_ACT_CONF				0x08
#define OAM_ACT_REQ_DENIED		0x0C
#define OAM_DEACT_REQ			0x14
#define OAM_DEACT_CONF			0x08
#define OAM_DEACT_REQ_DENIED	0x0C
#define OAM_ACT_DEACT_DIR_A_TO_B	0x02
#define OAM_ACT_DEACT_DIR_B_TO_A	0x01

/* Loopback Indication fields */
#define LOOPBACK_IND_DISC				0x00
#define LOOPBACK_IND_LPBK				0x01

/* definitions for the 'OAM_state' variable */
#define OAM_STATE_RESET					0x0000
#define OAM_STATE_TX_LPBK				0x0001
#define OAM_STATE_WT_LPBK_RESPONSE	0x0002
#define OAM_STATE_TX_CC_ACT_REQ		0x0010
#define OAM_STATE_TX_CC					0x0100
#define OAM_STATE_EXPECT_RX_CC		0x0200
#define OAM_STATE_TX_CC_DEACT_REQ	0x0400
#define OAM_STATE_TX_AIS				0x1000


#define OAM_ACT_DEACT_MID_MASK		0xFC
#define OAM_ACT_DEACT_DOA_MASK		0x03

#pragma pack()

#endif //OAM_H

