/***************************************************************************
                          x25config_impl.cpp  -  description
                             -------------------
    begin                : Tue Oct 2 2001
    copyright            : (C) 2001 by DavidR
    email                : drokhvarg@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "MainWizardImpl.h"

#define DBG_X25_CFG 0

extern TesterWizardImpl * globalAppPtr;

X25_CONFIG_STRUCT * global_x25cfg_struct;


static const char * baud_rate_list[] = {
	"Extern. Clock", "1200 bps", "2400 bps", "4800 bps", "9600 bps", "19200 bps",
	"38400 bps", "45000 bps", "56000 bps", "64000 bps", "74000 bps", "112000 bps", "128000 bps", 0 };

X25Config_Impl::X25Config_Impl(QWidget* parent,
																 const char* name,
  															 bool modal, WFlags fl ):Form_X25_Config(parent,
																																		 name,
																																		 modal,
																																		 fl)
{
  setDefaultX25CfgValues();
  set_widget_caption(this);
}

void X25Config_Impl::setDefaultX25CfgValues()
{
  DbgPrint1(DBG_X25_CFG, "X25Config_Impl::setDefaultX25CfgValues()\n");

	memset(&m_X25_cfg_structOld, 0x00, sizeof(X25_CONFIG_STRUCT));
	memset(&m_X25_cfg_structNew, 0x00, sizeof(X25_CONFIG_STRUCT));

	////////////////////////////////////////////////////////////////////
	//----------Default X.25 Configuration --------
	m_X25_cfg_structOld.line_speed_number = 0x0;

	m_X25_cfg_structOld.T2 = 0x0;
	m_X25_cfg_structOld.N2 = 0xA;
	m_X25_cfg_structOld.max_I_field_length = 0x103;
	m_X25_cfg_structOld.k = 0x7;

	m_X25_cfg_structOld.auto_modem_err_fixup = 0x1;
	m_X25_cfg_structOld.auto_HDLC = 0x1;
	
											 //V.35             SABM     RETRANS
	m_X25_cfg_structOld.HDLC_config_options = 0x80 | 0x02;// |  0x01  |  0x02;//may cause retry count exceeded on commands
																	 //so disabled by default
/*
  //FIXIT RS232 only for development
	m_X25_cfg_structOld.HDLC_config_options = 0x02;
*/
	//card number may be passed as command line argument
	if(app_command_line_empty == FALSE)
	{	if(card_number == CARD_DCE)
		{	m_X25_cfg_structOld.DTE = 0x00;//DCE
			m_X25_cfg_structOld.line_speed_number = 0x2;
			m_X25_cfg_structOld.T1 = 0x3;
			m_X25_cfg_structOld.T4 = 0x0;
			//Flow control parameter negotiation facilities included automatically,
			//no restart on link initialization
			m_X25_cfg_structOld.X25_config_options = 0x20 |	0x04;
		}
		else if(card_number == CARD_DTE)
		{	m_X25_cfg_structOld.DTE = 0x1;//DTE
			m_X25_cfg_structOld.T1 = 0x1;
			m_X25_cfg_structOld.T4 = 0x5;
			//Flow control parameter negotiation facilities included automatically,
			m_X25_cfg_structOld.X25_config_options = 0x20;
		}
	}else
	{	//no command line
	    m_X25_cfg_structOld.DTE = 0x1;//DTE
		m_X25_cfg_structOld.T1 = 0x3;
		m_X25_cfg_structOld.T4 = 0x0;
		//Flow control parameter negotiation facilities included automatically
		m_X25_cfg_structOld.X25_config_options = 0x20;
	}

	m_X25_cfg_structOld.packet_window = 0x2;
	m_X25_cfg_structOld.default_packet_size = 0x80;
	m_X25_cfg_structOld.maximum_packet_size = 0x100;
	m_X25_cfg_structOld.lowest_PVC = 0x1;
	m_X25_cfg_structOld.highest_PVC = 0x3;
	m_X25_cfg_structOld.lowest_incomming_channel = 0x0;
	m_X25_cfg_structOld.highest_incomming_channel = 0x0;
	m_X25_cfg_structOld.lowest_two_way_channel = 0x4;
	m_X25_cfg_structOld.highest_two_way_channel = 0x5;
	m_X25_cfg_structOld.lowest_outgoing_channel = 0x0;
	m_X25_cfg_structOld.highest_outgoing_channel = 0x0;
						
	m_X25_cfg_structOld.X25_response_options = 0x0;
	m_X25_cfg_structOld.genl_facilities_supported_1 = 0xFFFF;
	m_X25_cfg_structOld.genl_facilities_supported_2 = 0xF;
	m_X25_cfg_structOld.CCITT_facilities_supported = 0x7F;
	m_X25_cfg_structOld.non_X25_facilities_supported = 0x0;
	m_X25_cfg_structOld.CCITT_compatibility = 0x7C4;
	m_X25_cfg_structOld.T10_T20 = 0x3C;
	m_X25_cfg_structOld.T11_T21 = 0x3C;
	m_X25_cfg_structOld.T12_T22 = 0x3C;
	m_X25_cfg_structOld.T13_T23 = 0x3C;
	m_X25_cfg_structOld.T16_T26 = 0x3C;
	m_X25_cfg_structOld.T28 = 0x3C;
	m_X25_cfg_structOld.R10_R20 = 0xA;
	m_X25_cfg_structOld.R12_R22 = 0xA;
	m_X25_cfg_structOld.R13_R23 = 0xA;
	//---------- End of X.25 Default Configuration --------
	////////////////////////////////////////////////////////////////////

	//copy into New, so it is possible to compare and prompt for "SET_X25_HDLC_CFG" command
	memcpy(&m_X25_cfg_structNew, &m_X25_cfg_structOld, sizeof(X25_CONFIG_STRUCT));

	//this is for access to the structure from other objects
	global_x25cfg_struct = &m_X25_cfg_structNew;

	ComboBox_BaudRate->insertStrList(baud_rate_list, 13, -1);

  DbgPrint1(DBG_X25_CFG, "X25Config_Impl::setDefaultX25CfgValues() - End\n");
}

X25Config_Impl::~X25Config_Impl()
{

}

void X25Config_Impl::on_Cancel_clicked()
{
	done(IDCANCEL);
}

void X25Config_Impl::on_timeouts_and_counters_clicked()
{
	timeouts_and_counters.exec();
}

void X25Config_Impl::on_CCITT_facilities_clicked()
{
	ccitt_facil.exec();
}

void X25Config_Impl::on_configuration_options_clicked()
{
	cfg_and_response_options.exec();
}

void X25Config_Impl::on_general_facilities_clicked()
{
	general_facilities.exec();
}

void X25Config_Impl::on_logical_channels_clicked()
{
	logical_chann_cfg.exec();
}

void X25Config_Impl::on_packet_configuration_clicked()
{
	packet_cfg.exec();
}

/*

*/
void X25Config_Impl::showEvent(QShowEvent * ev)
{
	set_widget_position_on_screen(this);

	(m_X25_cfg_structNew.HDLC_config_options & 0x01) ?	
						CheckBox_IDC_SABM->setChecked(true):
						CheckBox_IDC_SABM->setChecked(false);
		
	(m_X25_cfg_structNew.HDLC_config_options & 0x02) ?	
						CheckBox_IDC_RETRANS->setChecked(true):
						CheckBox_IDC_RETRANS->setChecked(false);

	if(m_X25_cfg_structNew.HDLC_config_options & 0x80)
		rb_V35->setChecked(true);
	else
		rb_RS232->setChecked(true);

	if(m_X25_cfg_structNew.DTE)
		IDC_DTE->setChecked(true);
	else
		IDC_DCE->setChecked(true);

	set_edit_box_value(IDC_K, m_X25_cfg_structNew.k);
	set_edit_box_value(LineEdit_IDC_T1, m_X25_cfg_structNew.T1);
	set_edit_box_value(LineEdit_IDC_T2, m_X25_cfg_structNew.T2);
	set_edit_box_value(LineEdit_IDC_T4, m_X25_cfg_structNew.T4);
	set_edit_box_value(LineEdit_IDC_N2, m_X25_cfg_structNew.N2);

	ComboBox_BaudRate->setCurrentItem(m_X25_cfg_structNew.line_speed_number);
}

void X25Config_Impl::on_OK_clicked()
{
	if(CheckBox_IDC_SABM->isChecked() == true)
		m_X25_cfg_structNew.HDLC_config_options |= 0x01;	
	else
		m_X25_cfg_structNew.HDLC_config_options &= 0xFE;

	if(CheckBox_IDC_RETRANS->isChecked() == true)
		m_X25_cfg_structNew.HDLC_config_options |= 0x02;
	else
		m_X25_cfg_structNew.HDLC_config_options &= 0xFD;		

	if(rb_V35->isChecked() == true)
		m_X25_cfg_structNew.HDLC_config_options |= 0x80;
	else
		m_X25_cfg_structNew.HDLC_config_options  &= (~0x80);	

	if(IDC_DTE->isChecked() == true)
		m_X25_cfg_structNew.DTE = 1;
	else
		m_X25_cfg_structNew.DTE = 0;

	m_X25_cfg_structNew.T1 = get_edit_box_value(LineEdit_IDC_T1);
	m_X25_cfg_structNew.T2 = get_edit_box_value(LineEdit_IDC_T2);
	m_X25_cfg_structNew.T4 = get_edit_box_value(LineEdit_IDC_T4);
	m_X25_cfg_structNew.N2 = get_edit_box_value(LineEdit_IDC_N2);

	m_X25_cfg_structNew.line_speed_number = ComboBox_BaudRate->currentItem();
//	printf("line_speed_number : %d\n", m_X25_cfg_structNew.line_speed_number);

	if(PassedChecks(m_X25_cfg_structNew) == true)	
		done(IDOK);

	//else	ReportError("Configuration may not be set as is!");
}

void X25Config_Impl::set_edit_box_value(QLineEdit * ed, int value)
{
	char s[100];

	sprintf(s, "%d", value);
	QString tmp(s);
	ed->setText(tmp);
}

unsigned short X25Config_Impl::get_edit_box_value(QLineEdit * ed)
{
	bool ok = false;
	QString s_tmp;

	
	s_tmp = ed->text();
	return s_tmp.toUShort(&ok, 10);
}

bool X25Config_Impl::PassedChecks(X25_CONFIG_STRUCT cfg_block)
{
	if (cfg_block.T2 >= cfg_block.T1) {
		ReportError("Error: T2 must always be less than T1");
		return false;
	}
	if (cfg_block.maximum_packet_size < cfg_block.default_packet_size) {
		ReportError("Error: Maximum packet size must be greater than or equal to the default packet size");
		return false;
	}
	// first test that highest is not lower than lowest
	if(bounds_check(cfg_block.lowest_PVC, cfg_block.highest_PVC, "PVC") == false)
		return false;
	if(bounds_check(cfg_block.lowest_incomming_channel, cfg_block.highest_incomming_channel, "Incoming SVC") == false)
		return false;
	if(bounds_check(cfg_block.lowest_two_way_channel, cfg_block.highest_two_way_channel, "Two-way SVC") == false)
		return false;
	if(bounds_check(cfg_block.lowest_outgoing_channel, cfg_block.highest_outgoing_channel, "Outgoing SVC") == false)
		return false;

	// now check that the ranges do not overlap
	if(overlap_check(cfg_block.lowest_incomming_channel, cfg_block.highest_PVC, "Incoming SVC", "PVC") == false)
		return false;
	if(overlap_check(cfg_block.lowest_two_way_channel, cfg_block.highest_incomming_channel, "Two-way SVC", "Incoming SVC") == false)
		return false;
	if(overlap_check(cfg_block.lowest_outgoing_channel, cfg_block.highest_two_way_channel, "Outgoing SVC", "Two-way SVC") == false)
		return false;

	// check total allocation of channels doesn't exceed 255
	if ( (total_channels_check(&(cfg_block.lowest_PVC))) == false) {
		ReportError("Error: Total number of configured channels exceeds 255");
		return false;
	}

	return true;
}

void X25Config_Impl::ReportError(char * message)
{
  QMessageBox::warning(	0, "X25 Configuration Error.", message);
}

bool X25Config_Impl::bounds_check(WORD low, WORD hi, char *item)
{
	char s[150];

	if (low > hi) {
		sprintf(s, "Error: Highest %s must be greater than or equal to lowest %s", item, item);
		ReportError(s);
		return(false);
	}
	if (low == 0 && hi > 0) {
		sprintf(s, "Error: Highest %s must also be zero if lowest is zero", item);
		ReportError(s);
		return(false);
	}
	if (low > 4095) {
		sprintf(s, "Error: The channel number range for %s must be below 4096", item);
		ReportError(s);
		return(false);
	}
	if (hi > 4095) {
		sprintf(s, "Error: Highest %s must be in the range %u-4095", item, low);
		ReportError(s);
		return(false);
	}
	return(true);
}

bool X25Config_Impl::overlap_check(WORD low, WORD hi, char *item1, char *item2)
{
	char s[150];

	if (hi >= low && low > 0) {
		sprintf(s, "Error: Lowest %s must be greater than highest %s", item1, item2);
		ReportError(s);
		return(false);
	}
	return(true);
}

bool X25Config_Impl::total_channels_check(WORD *start)
{
	int i;
	WORD low, hi, total;

	total = 0;
	for (i=0; i < 4; i++) {
		low = *(start+(i*2));
		hi = *(start+(i*2)+1);
		if (low != 0) total += (hi-low+1);
	}
	if (total > 255) return(false);
	return(true);
}
