/***************************************************************************
                          xapi.cpp  -  description
                             -------------------
    begin                : Fri Sep 28 2001
    copyright            : (C) 2001 by DavidR
    email                : drokhvarg@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tester.h"					//data types
#include "xapi.h"						//class header file

#define DBG_XAPI  0 //if 0 no debug output, else there will be debug output

///////////////////////////////////////////////////////////////////
#define MAX_CARDS 16

typedef union {
	WORD lowPVC, hiPVC, lowIC, hiIC, lowTW, hiTW, lowOC, hiOC;
	WORD range[8];
} LCDISTRIB;
LCDISTRIB lc_ranges[MAX_CARDS];
WORD highest_logical_number[MAX_CARDS];

//////////////////////////////////////////////////////////////////////

/*********** Defines *****************/

#ifndef min
#define min(a,b)        (((a)<(b))?(a):(b))
#endif

/********************* Functions Implementation **************************/

Xapi::Xapi()
{
	update_timestamp = TRUE;
}

Xapi::~Xapi()
{
	DbgPrint1(DBG_XAPI, "~Xapi()\n");
}

int Xapi::XAPIAlive(void)
{ //not implemented.
	return 1;
}

/*******************************************************************
* Show error message.
*******************************************************************/
void Xapi::show_error (int err)
{
	if (err == ERR_SYSTEM) fprintf(stderr, "%s: SYSTEM ERROR %d: %s!\n",
		progname, errno, strerror(errno))
	;
	else fprintf(stderr, "%s: %s!\n", progname,
		err_messages[min(err, ERR_LIMIT) - 2])
	;
}

char Xapi::XAPIControl(XCBLOCK * block)
{
	char rcode;

#if defined(BUILD_FOR_UNIX)

#elif defined(WIN32)
  	DWORD mresult;

	// start system-wide critical section
	// release mutex in SayReturnResult!!!
	mresult = WaitForSingleObject(api_base_control_mutex, timeout_ticks);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED) {
		block->return_code = Z80_TIMEOUT_ERROR;
		return block->return_code;
	}
#endif

  if(!(internal_state & CONNECTED_TO_DRIVER) || !(internal_state & FIRMWARE_LOADED))
  {
    DbgPrint1(DBG_XAPI, "Xapi::XAPIControl() : not connected!!!\n");
    rcode = block->return_code = Z80_TIMEOUT_ERROR;
		// someone else can do a mail box call now
#if defined(BUILD_FOR_UNIX)

#elif defined(WIN32)
    ReleaseMutex(api_base_control_mutex);
#endif

    return rcode;
  }

	rcode = InternalDoControl(block);

	if (block->command == X25_SET_HDLC_X25_CONFIGURATION)
  {	ReadConfigNTimes();
	}

		// someone else can do a mail box call now
#if defined(BUILD_FOR_UNIX)

#elif defined(WIN32)
    ReleaseMutex(api_base_control_mutex);
#endif

	return(rcode);
}

char Xapi::InternalDoControl(XCBLOCK * block)
{
  struct tm *newtime;
  time_t long_time;
	BYTE stamp[5];
	char flag;
  ULONG offset, num_bytes, buffer;

//	DbgPrint2(DBG_XAPI, "InternalDoControl():block->command		0x%x.\n", block->command);
//	DbgPrint2(DBG_XAPI, "InternalDoControl():block->buffer_length	0x%x.\n", block->buffer_length);

	if (update_timestamp)
 	{
#define LEN_OF_TIME_STAMP_BUFF  5
		// set the timestamp on the board
    time( &long_time );                // Get time as long integer.
    newtime = localtime( &long_time ); // Convert to local time.


		stamp[0] = MakeBCD(newtime->tm_mon);
		stamp[1] = MakeBCD(newtime->tm_mday);
		stamp[2] = MakeBCD(newtime->tm_hour);
		stamp[3] = MakeBCD(newtime->tm_min);
		stamp[4] = MakeBCD(newtime->tm_sec);

		//copy control block onto the card
		API_BASE_WriteBlock((ULONG)XMO_TSTAMP, (ULONG)LEN_OF_TIME_STAMP_BUFF, (void *)stamp);
	}

	// decide on correct mailbox to use according to command
	if (block->command == X25_READ)   offset = XMO_RECVBOX+1;
	else                              offset = XMO_SENDBOX+1;

	// reject block if buffer_length is too high, no matter what
	if (block->buffer_length > XMDATALEN)
	  return(SayReturnResult(block->return_code=X25_DATA_LENGTH_TOO_BIG, block->data[0], (block->buffer_length > 0)));

	switch(block->command)
 	{
	case X25_WRITE:
		if (!XAPIqClearToXmit(block->logical_channel))
			return(SayReturnResult(block->return_code=X25_NO_BUFFS_OR_CLOSED_WIN, block->data[0], block->buffer_length));
		break;
	case X25_READ:
		if (!XAPIqDataToRead(block->logical_channel))
			return(SayReturnResult(block->return_code=X25_NO_DATA_AVAILABLE, block->data[0], block->buffer_length));
		break;
	}

	// submit to appropriate mailbox via driver
	num_bytes = block->buffer_length + CB_HEADERLEN;
	buffer = (unsigned int)block;

	//copy control block onto the card
	API_BASE_WriteBlock(offset, num_bytes, (void *)buffer);

	// submitted to mailbox... now set flag and wait to clear
	flag = SetWaitOppFlag((unsigned short)offset - 1);

	if (!flag) block->return_code = Z80_TIMEOUT_ERROR;
	else {
		// give application a copy of the mailbox after the transaction
		num_bytes = CB_HEADERLEN;

		API_BASE_ReadBlock(offset, num_bytes, block);
		
		num_bytes = block->buffer_length;
		buffer = (unsigned int)block->data;
		offset += CB_HEADERLEN;

    API_BASE_ReadBlock(offset, num_bytes, block->data);
	}
	//DbgPrint2(DBG_XAPI, "InternalDoControl():block->return_code	0x%x.\n", block->return_code );
	return(SayReturnResult(block->return_code, block->data[0], block->buffer_length));
}


// 's' must be of size less than (80-5) bytes incl. null
char Xapi::SayReturnResult(char rval, char dval, WORD bl)
{
	char a[80];
	if (bl > 0)
		sprintf(a, "0x%02x  Data[0] = 0x%02x", (BYTE)rval, (BYTE)dval);
	else
		sprintf(a, "0x%02x", (BYTE)rval);
	
	//DbgPrint2(DBG_XAPI, "SayReturnResult : %s\n", a);

	return(rval);
}

int Xapi::XAPIqLinkInABM()
{
	unsigned char value;

  API_BASE_ReadBlock(XMO_XSTAT, 1, &value);

	return (value & XBM_IN_ABM);
}

char Xapi::ReadConfigNTimes()
/* returns 0x7f if failed after num_read_config_retries, 0 if ok */
/* if ReadConfiguration gives a TIMEOUT error, forget retries */
{
	int i;
	BYTE rc;

	i=0;
	do

	{
		i++;
		rc = (BYTE)ReadConfiguration();
		if (rc == Z80_TIMEOUT_ERROR) return(0x7f);
	} while (rc && i <= num_read_config_retries);

	if (i > num_read_config_retries)
		return(0x7f);
	return(0);
}

//!!!!! non-thread-reentrant including all tests involving
//  lc_ranges array
char Xapi::ReadConfiguration()
/* returns rcode (non-zero) if couldn't read it, return_code (0) if did */
{
	XCBLOCK cb;
	int c;
	WORD largest, n;
	unsigned char rcode;
	int cardnumber = 0;

	cb.command = X25_READ_CHANNEL_CONFIGURATION;
	cb.buffer_length = 0;
	cb.logical_channel = 0;
	cb.return_code = 0x7f;
	if ((rcode=InternalDoControl(&cb)) != 0)  return(rcode);
	/* let's keep a local copy */
	memcpy((void *)&lc_ranges[cardnumber], cb.data, 16);

	/* find highest logical channel number to decide if need
	complex or simple mapping */
	largest = 0;
	for (c=0; c < 4; c++)  {
		if ((n=lc_ranges[cardnumber].range[2*c+1]) > largest)
			largest = n;
	}

	highest_logical_number[cardnumber] = largest;
	return(rcode);
}

char Xapi::XAPIqClearToXmit(WORD logical_channel)

/* logic:
first test LCN in xfer mode: no -> passthru (return code 1)
logical and of transmit window open and buffers available: yes -> passthru
else return 0
*/
{
	BYTE x, pc;
	pc = MapLCNtoPhysical(logical_channel);
	API_BASE_ReadBlock((XMO_XSTAT+(WORD)pc), 1, &x);

	if (!(x & XBM_LCN_DTM))  return(1);
	if (x & XBM_LCN_TWINOPEN)  {
		x = API_BASE_ReadBlock(XMO_XSTAT, 1, &x);

		if (x & XBM_CTS_X)  return(1);
	}
	return (0);
}

unsigned char Xapi::MapLCNtoPhysical(WORD logical_channel)
/* translate logical channel number to physical offset in LC table */
/* returns 0 if invalid channel number */
{
	BYTE type, low, hi;
	WORD subt;
  BYTE cardnumber = 0;

	/* check if only need simple mapping first */
	if (highest_logical_number[cardnumber] < 0x100)
			return((unsigned char)logical_channel);

	type = XAPIGetLCType(logical_channel);
	if (type == 0x7f)  return(0);

	low = API_BASE_ReadBlock((XMO_PVCMAP+(type*2)), 1, &low);

	hi = API_BASE_ReadBlock((XMO_PVCMAP+(type*2)+1), 1, &hi);

	subt = ((WORD)hi * 256) + (WORD)low;
	return((unsigned char)(logical_channel - subt));
}

unsigned char Xapi::XAPIGetLCType(WORD lc)
/* returns 0 if in PVC range, 1 in IC, 2 in TW, 3 in OC, 0x7f if invalid */
{
	int c;
  BYTE cardnumber = 0;

	for (c=0; c < 4; c++)  {
		if (lc >= lc_ranges[cardnumber].range[2*c] &&
			lc <= lc_ranges[cardnumber].range[2*c+1])
			return(c);
	}
	return(0x7f);
}

char Xapi::XAPIqDataToRead(WORD logical_channel)
/* logic:
first test LCN in xfer mode: no -> passthru (return code 1)
if there's something to receive -> passthru
if there's an async packet -> passthru
else return 0
*/
{
	BYTE x, pc;

	pc = MapLCNtoPhysical(logical_channel);

	x = API_BASE_ReadBlock((XMO_XSTAT+(WORD)pc), 1, &x);

	if (!(x & XBM_LCN_DTM))  return(1);
	if (x & XBM_LCN_NUMTOREAD)  return(1);
	if (XAPIqNotification())  return(1);
	return (0);
}

char Xapi::XAPIqNotification()
{
	BYTE x;

	x = API_BASE_ReadBlock(XMO_XSTAT, 1, &x);
	return (x & XBM_NOTIFICATION);
}

WORD Xapi::XAPIGetHighestLCN()
{
	return(highest_logical_number[0]);
}

char Xapi::XAPIqTraceDataAvailable()
{
	BYTE x;

	x = API_BASE_ReadBlock(XMO_XSTAT, 1, &x);

	return (x & XBM_TRACE_AVAIL);
}

/////////////////////////////////////////////////////////////////////////////////////////

