/***************************************************************************
                          xapi.h  -  description
                             -------------------
    begin                : Fri Sep 28 2001
    copyright            : (C) 2001 by DavidR
    email                : drokhvarg@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef XAPI_H
#define XAPI_H

#include "api_base.h"

#include "rcode.h"	//return codes from X25 and HDLC commands
#include "x25.h"    //memory locations and bit masks for X25 microcode

////////////////////////////////////////////

// X25 level Mailbox commands
#define X25_READ_CODE_VERSION     0x15
#define	X25_READ		              0X22
#define	X25_WRITE		              0X23
#define	X25_PLACE_CALL		        0X30
#define	X25_ACCEPT_CALL		        0X31
#define	X25_CLEAR_CALL		        0X32
#define	X25_CONFIRM_CLEAR	        0X33
#define	X25_RESET_LOGICAL_CHANNEL	0X34
#define	X25_CONFIRM_RESET		      0X35
#define	X25_RESTART			          0X36
#define	X25_CONFIRM_RESTART		    0X37
#define	X25_INTERRUPT		          0X38
#define	X25_CONFIRM_INTERRUPT	    0X39
#define	X25_REGISTRATION_REQUEST	0X3a
#define	X25_REGISTRATION_CONFIRMATION	0X3b
#define	X25_IS_DATA_AVAILABLE	        0X40
#define	X25_INCOMING_CALL_CONTROL	    0X41
#define	X25_CONFIGURE_PVC		          0X42
#define	X25_READ_ACTIVE_CHANNELS	    0X43
#define	X25_READ_CHANNEL_CONFIGURATION	0X44
#define	X25_FLUSH_X25_DATA_BUFFERS	    0X45
#define	X25_READ_X25_HISTORY_TABLE	    0X46
#define	X25_HISTORY_TABLE_CONTROL	      0X47
#define	X25_READ_X25_STATISTICS	        0x49
#define	X25_FLUSH_X25_STATISTICS	      0x4a
#define	X25_READ_HDLC_X25_CONFIGURATION	0x50
#define	X25_SET_HDLC_X25_CONFIGURATION	0x51

// HDLC level Mailbox commands
#define	CONFIGURE_LINK				0x01

#define	HDLC_OPEN					    0x02
#define HDLC_CLOSE					  0x03
#define	LINK_SETUP					  0x04
#define	LINK_DISCONNECT				0x05
#define	LINK_STATUS					  0x06
#define	READ_OPERATIONAL_STATS		  0x07
#define	FLUSH_OPERATIONAL_STATS		  0x08
#define	HDLC_READ_COMMS_ERR_STATS	  0x09
#define	HDLC_FLUSH_COMMS_ERR_STATS	0x0a
#define	SET_GLOBAL_VARIABLES		0x0b
#define	HDLC_READ_MODEM_STATUS	0x0c
#define	FLUSH_DATA_BUFFERS			0x0d
#define	I_FRM_SUP_COUNT_STAT		0x0e
#define	HDLC_WRITE					    0x11
#define	READ_CONFIGURATION			0x12
#define	SET_CONFIGURATION			  0x13
#define	OPERATE_DATALINE_MONITOR	0x14
#define	HDLC_READ_CODE_VERSION		0x15
#define	READ_TRACE_DATA				0x16
#define	HDLC_READ					    0x21
#define SET_GLOBAL_VARIABLES	0x0b


/* error messages */
#define	X25_NO_BUFFS_OR_CLOSED_WIN	0x33
#define	X25_DATA_LENGTH_TOO_BIG	    0x32
#define	X25_NO_DATA_AVAILABLE	      0x33

#define XMDATALEN           1027
#define CB_HEADERLEN        15
#define MAX_NUMBER_OF_LCNS  4096

/* XIP control block */
#pragma pack(1)
typedef struct {
    BYTE command;
		WORD buffer_length;
    BYTE return_code;
    BYTE reserved_HDLC;
		WORD logical_channel;
    BYTE Q_D_M_bits;
    BYTE cause;
    BYTE diagnostic;
    BYTE X25_packet_type;
    BYTE time_stamp[4];
    BYTE data[XMDATALEN];
} XCBLOCK;
#pragma pack()

/* bitmasks for Q_D_M_bits above */
#define MASK_QBIT       0x04
#define MASK_DBIT       0x02
#define MASK_MBIT       0x01

#define MakeBCD(v) ((BYTE)( (((BYTE)v/10) << 4) | ((BYTE)v % 10) ))

/* X.25 configuration structure - goes directly to the card */
#pragma pack(1)
typedef struct {
	BYTE line_speed_number;
	BYTE T1;
	BYTE T2;
	BYTE N2;
	WORD max_I_field_length;
	BYTE k;
	BYTE T4;
	BYTE auto_modem_err_fixup;
	BYTE auto_HDLC;
	BYTE HDLC_config_options;
	BYTE DTE;
	BYTE packet_window;
	WORD default_packet_size;
	WORD maximum_packet_size;
	WORD lowest_PVC;
	WORD highest_PVC;
	WORD lowest_incomming_channel;
	WORD highest_incomming_channel;
	WORD lowest_two_way_channel;
	WORD highest_two_way_channel;
	WORD lowest_outgoing_channel;
	WORD highest_outgoing_channel;
	WORD X25_config_options;
	BYTE X25_response_options;
	WORD genl_facilities_supported_1;
	WORD genl_facilities_supported_2;
	WORD CCITT_facilities_supported;
	WORD non_X25_facilities_supported;
	WORD CCITT_compatibility;
	BYTE T10_T20;
	BYTE T11_T21;
	BYTE T12_T22;
	BYTE T13_T23;
	BYTE T16_T26;

	BYTE T28;
	BYTE R10_R20;
	BYTE R12_R22;
	BYTE R13_R23;
} X25_CONFIG_STRUCT;
#pragma pack()

/////////////////////////////////////////////////////////////////////
/**
  *@author DavidR
  */

class Xapi : public api_base
{
public:
  BOOL update_timestamp;
  int num_read_config_retries;

	Xapi();
	~Xapi();

  /************ Functions used internally by the class *********/
  char ReadConfiguration();
  unsigned char MapLCNtoPhysical(WORD logical_channel);
  void SayMsg(char * m);
  void ClearTrace(void);
  char SayReturnResult(char *s, char rval, char dval, WORD bl);
  char ReadConfigNTimes();

  /* will always return 1 */
  int XAPIAlive(void);

  /* returns non-zero if link in ABM */
  int XAPIqLinkInABM();

  /* Emulate XIP interrupt call with control block
    returns return_code value in structure
  */
  char XAPIControl(XCBLOCK * block);

  /* returns non-zero if notify bit is set */
  char XAPIqNotification();

  /* returns non-zero if trace data is available to be read by
   a READ_TRACE_DATA call
  */
  char XAPIqTraceDataAvailable();

  /* returns non-zero if there is data to receive, or if the specified
   LC is not in data transfer mode, in which case an X25_READ should be
   issued by the application anyway to look at the return code, which
   would hopefully explain why the link is down
  */
  char XAPIqDataToRead(WORD logical_channel);

  /* returns non-zero if there are available buffers to issue a successful
     X25_WRITE and if the transmit window is open, or if the specified LC
     is not in data transfer mode, in which case an X25_WRITE should be
     issues anyway by the application to look at the return code, which
     would hopefully explain why the link is down
   */
  char XAPIqClearToXmit(WORD logical_channel);

  /* returns:
   0 = PVC, 1 = I-SVC (incoming only), 2 = T-SVC (two-way),
   3 = O-SVC (outgoing only), or 0xff if invalid LCN
  */
  unsigned char XAPIGetLCType(WORD lc);

  /* returns highest LCN configured, of all SVC's or PVC's */
  WORD XAPIGetHighestLCN();

  /* checks the board status bits for the specified information,
   without issuing a mailbox command (fast!)
  */
  char XAPISupervisoryCall(BYTE infocode, WORD lcn);

  /* infocode as follows... */
  /* returns number of packets available to be read */
  #define XSC_LCINFO_NUMPACKETS   0x80
  /* returns non-zero if the transmit window is open */
  #define XSC_LCINFO_TXWINDOWOPEN 0x81
  /* returns non-zero if the LC is in data transfer mode */
  #define XSC_LCINFO_IN_DTM       0x82
  #define XSC_INFO_PC             0x80    /* internal */
  /* returns non-zero if there is data to read on any channel */
  #define XSC_INFO_DATATOREAD     0x01
  /* returns non-zero if there are buffers available to write a
   packet to the transmit queue (indep. of LC)
  */
  #define XSC_INFO_BUFFSTOXMIT    0x02

  /* DEBUGGING COMMAND TRACE MESSAGES (wParam values) */
  #define TRACEMSG_CLOSE  0

  /* Undocumented exports - use at own risk */
  /** prototypes of functions which should not be used directly by an application **/
private:
  void SayControlNumber(void);
  void AddStringListBox(char * m);
  char SayReturnResult(char rval, char dval, WORD bl);
  char InternalDoControl(XCBLOCK * block);

  void show_error (int err);
  int read_code_version();
};

#endif//#ifndef XAPI_H
